@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* wee-config.c: WeeChat configuration options */
+/*
+ * wee-config.c: WeeChat configuration options (file weechat.conf)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -66,7 +68,6 @@ struct t_config_option *config_startup_command_after_plugins;
 struct t_config_option *config_startup_command_before_plugins;
 struct t_config_option *config_startup_display_logo;
 struct t_config_option *config_startup_display_version;
-struct t_config_option *config_startup_weechat_slogan;
 
 /* config, look & feel section */
 
@@ -74,9 +75,11 @@ struct t_config_option *config_look_buffer_notify_default;
 struct t_config_option *config_look_buffer_time_format;
 struct t_config_option *config_look_color_nicks_number;
 struct t_config_option *config_look_color_real_white;
+struct t_config_option *config_look_command_chars;
 struct t_config_option *config_look_day_change;
 struct t_config_option *config_look_day_change_time_format;
 struct t_config_option *config_look_highlight;
+struct t_config_option *config_look_hline_char;
 struct t_config_option *config_look_hotlist_names_count;
 struct t_config_option *config_look_hotlist_names_length;
 struct t_config_option *config_look_hotlist_names_level;
@@ -98,12 +101,14 @@ struct t_config_option *config_look_prefix_buffer_align_max;
 struct t_config_option *config_look_prefix_buffer_align_more;
 struct t_config_option *config_look_prefix_suffix;
 struct t_config_option *config_look_read_marker;
+struct t_config_option *config_look_read_marker_always_show;
 struct t_config_option *config_look_save_config_on_exit;
 struct t_config_option *config_look_save_layout_on_exit;
 struct t_config_option *config_look_scroll_amount;
 struct t_config_option *config_look_scroll_page_percent;
 struct t_config_option *config_look_search_text_not_found_alert;
 struct t_config_option *config_look_set_title;
+struct t_config_option *config_look_time_format;
 
 /* config, colors section */
 
@@ -141,16 +146,17 @@ struct t_config_option *config_color_status_data_private;
 struct t_config_option *config_color_status_data_highlight;
 struct t_config_option *config_color_status_data_other;
 struct t_config_option *config_color_status_more;
+struct t_config_option *config_color_status_time;
 struct t_config_option *config_color_input_text_not_found;
 struct t_config_option *config_color_input_actions;
 struct t_config_option *config_color_nicklist_group;
 struct t_config_option *config_color_nicklist_away;
+struct t_config_option *config_color_nicklist_offline;
 struct t_config_option *config_color_nicklist_prefix1;
 struct t_config_option *config_color_nicklist_prefix2;
 struct t_config_option *config_color_nicklist_prefix3;
 struct t_config_option *config_color_nicklist_prefix4;
 struct t_config_option *config_color_nicklist_prefix5;
-struct t_config_option *config_color_nicklist_more;
 
 /* config, completion section */
 
@@ -174,7 +180,7 @@ struct t_config_option *config_history_display_default;
 
 /* config, network section */
 
-struct t_config_option *config_network_gnutls_dh_prime_bits;
+struct t_config_option *config_network_gnutls_ca_file;
 
 /* config, plugin section */
 
@@ -184,9 +190,10 @@ struct t_config_option *config_plugin_extension;
 struct t_config_option *config_plugin_path;
 struct t_config_option *config_plugin_save_config_on_unload;
 
-/* hooks */
+/* other */
 
 struct t_hook *config_day_change_timer = NULL;
+int config_day_change_old_day = -1;
 
 
 /*
@@ -221,9 +228,7 @@ config_change_title (void *data, struct t_config_option *option)
     (void) option;
     
     if (CONFIG_BOOLEAN(config_look_set_title))
-	gui_window_set_title (PACKAGE_NAME " " PACKAGE_VERSION);
-    else
-	gui_window_set_title (NULL);
+        gui_window_set_title (PACKAGE_NAME " " PACKAGE_VERSION);
 }
 
 /*
@@ -343,7 +348,7 @@ config_day_change_timer_cb (void *data, int remaining_calls)
 {
     struct timeval tv_time;
     struct tm *local_time;
-    char text_time[1024], *text_time2;
+    char text_time[256], *text_time2;
     struct t_gui_buffer *ptr_buffer;
     
     /* make C compiler happy */
@@ -352,57 +357,39 @@ config_day_change_timer_cb (void *data, int remaining_calls)
     
     gettimeofday (&tv_time, NULL);
     local_time = localtime (&tv_time.tv_sec);
-    
-    strftime (text_time, sizeof (text_time),
-              CONFIG_STRING(config_look_day_change_time_format),
-              local_time);
-    text_time2 = string_iconv_to_internal (NULL, text_time);
-    gui_add_hotlist = 0;
-    for (ptr_buffer = gui_buffers; ptr_buffer;
-         ptr_buffer = ptr_buffer->next_buffer)
-    {
-        if (ptr_buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-            gui_chat_printf (ptr_buffer,
-                             _("\t\tDay changed to %s"),
-                             (text_time2) ?
-                             text_time2 : text_time);
-    }
-    if (text_time2)
-        free (text_time2);
-    gui_add_hotlist = 1;
 
-    return WEECHAT_RC_OK;
-}
-
-/*
- * config_change_day_change: called when day_change option changed
- */
-
-void
-config_change_day_change (void *data, struct t_config_option *option)
-{
-    /* make C compiler happy */
-    (void) data;
-    (void) option;
-    
-    if (CONFIG_BOOLEAN(config_look_day_change))
+    if ((config_day_change_old_day >= 0)
+        && (local_time->tm_mday != config_day_change_old_day))
     {
-        if (!config_day_change_timer)
-            config_day_change_timer = hook_timer (NULL,
-                                                  24 * 3600 * 1000,
-                                                  24 * 3600,
-                                                  0,
-                                                  &config_day_change_timer_cb,
-                                                  NULL);
-    }
-    else
-    {
-        if (config_day_change_timer)
+        if (CONFIG_BOOLEAN(config_look_day_change))
         {
-            unhook (config_day_change_timer);
-            config_day_change_timer = NULL;
+            strftime (text_time, sizeof (text_time),
+                      CONFIG_STRING(config_look_day_change_time_format),
+                      local_time);
+            text_time2 = string_iconv_to_internal (NULL, text_time);
+            gui_add_hotlist = 0;
+            for (ptr_buffer = gui_buffers; ptr_buffer;
+                 ptr_buffer = ptr_buffer->next_buffer)
+            {
+                if (ptr_buffer->type == GUI_BUFFER_TYPE_FORMATTED)
+                    gui_chat_printf (ptr_buffer,
+                                     _("\t\tDay changed to %s"),
+                                     (text_time2) ?
+                                     text_time2 : text_time);
+            }
+            if (text_time2)
+                free (text_time2);
+            gui_add_hotlist = 1;
         }
+        
+        /* send signal "day_changed" */
+        strftime (text_time, sizeof (text_time), "%Y-%m-%d", local_time);
+        hook_signal_send ("day_changed", WEECHAT_HOOK_SIGNAL_STRING, text_time);
     }
+    
+    config_day_change_old_day = local_time->tm_mday;
+    
+    return WEECHAT_RC_OK;
 }
 
 /*
@@ -422,7 +409,7 @@ config_weechat_reload_cb (void *data, struct t_config_file *config_file)
     (void) data;
     
     /* remove all keys */
-    gui_keyboard_free_all (&gui_keys, &last_gui_key);
+    gui_keyboard_free_all (&gui_keys, &last_gui_key, &gui_keys_count);
     
     /* remove all proxies */
     proxy_free_all ();
@@ -822,33 +809,44 @@ config_weechat_layout_read_cb (void *data, struct t_config_file *config_file,
  * config_weechat_layout_write: write windows layout in configuration file
  */
 
-void
+int
 config_weechat_layout_write_tree (struct t_config_file *config_file,
                                   struct t_gui_layout_window *layout_window)
 {
-    config_file_write_line (config_file, "window", "\"%d;%d;%d;%d;%s;%s\"",
-                            layout_window->internal_id,
-                            (layout_window->parent_node) ?
-                            layout_window->parent_node->internal_id : 0,
-                            layout_window->split_pct,
-                            layout_window->split_horiz,
-                            (layout_window->plugin_name) ?
-                            layout_window->plugin_name : "-",
-                            (layout_window->buffer_name) ?
-                            layout_window->buffer_name : "-");
+    if (!config_file_write_line (config_file, "window", "\"%d;%d;%d;%d;%s;%s\"",
+                                 layout_window->internal_id,
+                                 (layout_window->parent_node) ?
+                                 layout_window->parent_node->internal_id : 0,
+                                 layout_window->split_pct,
+                                 layout_window->split_horiz,
+                                 (layout_window->plugin_name) ?
+                                 layout_window->plugin_name : "-",
+                                 (layout_window->buffer_name) ?
+                                 layout_window->buffer_name : "-"))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     if (layout_window->child1)
-        config_weechat_layout_write_tree (config_file, layout_window->child1);
+    {
+        if (config_weechat_layout_write_tree (config_file,
+                                              layout_window->child1) != WEECHAT_CONFIG_WRITE_OK)
+            return WEECHAT_CONFIG_WRITE_ERROR;
+    }
     
     if (layout_window->child2)
-        config_weechat_layout_write_tree (config_file, layout_window->child2);
+    {
+        if (config_weechat_layout_write_tree (config_file,
+                                              layout_window->child2) != WEECHAT_CONFIG_WRITE_OK)
+            return WEECHAT_CONFIG_WRITE_ERROR;
+    }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
  * config_weechat_layout_write_cb: write layout section in configuration file
  */
 
-void
+int
 config_weechat_layout_write_cb (void *data, struct t_config_file *config_file,
                                 const char *section_name)
 {
@@ -857,19 +855,27 @@ config_weechat_layout_write_cb (void *data, struct t_config_file *config_file,
     /* make C compiler happy */
     (void) data;
     
-    config_file_write_line (config_file, section_name, NULL);
+    if (!config_file_write_line (config_file, section_name, NULL))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     for (ptr_layout_buffer = gui_layout_buffers; ptr_layout_buffer;
          ptr_layout_buffer = ptr_layout_buffer->next_layout)
     {
-        config_file_write_line (config_file, "buffer", "\"%s;%s;%d\"",
-                                ptr_layout_buffer->plugin_name,
-                                ptr_layout_buffer->buffer_name,
-                                ptr_layout_buffer->number);
+        if (!config_file_write_line (config_file, "buffer", "\"%s;%s;%d\"",
+                                     ptr_layout_buffer->plugin_name,
+                                     ptr_layout_buffer->buffer_name,
+                                     ptr_layout_buffer->number))
+            return WEECHAT_CONFIG_WRITE_ERROR;
     }
     
     if (gui_layout_windows)
-        config_weechat_layout_write_tree (config_file, gui_layout_windows);
+    {
+        if (config_weechat_layout_write_tree (config_file,
+                                              gui_layout_windows) != WEECHAT_CONFIG_WRITE_OK)
+            return WEECHAT_CONFIG_WRITE_ERROR;
+    }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
@@ -886,7 +892,7 @@ config_weechat_notify_change_cb (void *data, struct t_config_option *option)
     gui_buffer_notify_set_all ();
 }
 
-/* 
+/*
  * config_weechat_notify_create_option_cb: callback to create option in "notify"
  *                                         section
  */
@@ -1057,7 +1063,7 @@ config_weechat_filter_read_cb (void *data,
  * config_weechat_filter_write_cb: write filter section in configuration file
  */
 
-void
+int
 config_weechat_filter_write_cb (void *data, struct t_config_file *config_file,
                                 const char *section_name)
 {
@@ -1066,21 +1072,25 @@ config_weechat_filter_write_cb (void *data, struct t_config_file *config_file,
     /* make C compiler happy */
     (void) data;
     
-    config_file_write_line (config_file, section_name, NULL);
+    if (!config_file_write_line (config_file, section_name, NULL))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     for (ptr_filter = gui_filters; ptr_filter;
          ptr_filter = ptr_filter->next_filter)
     {
-        config_file_write_line (config_file,
-                                ptr_filter->name,
-                                "%s;%s%s%s;%s;%s",
-                                (ptr_filter->enabled) ? "on" : "off",
-                                (ptr_filter->plugin_name) ? ptr_filter->plugin_name : "",
-                                (ptr_filter->plugin_name) ? "." : "",
-                                ptr_filter->buffer_name,
-                                ptr_filter->tags,
-                                ptr_filter->regex);
+        if (!config_file_write_line (config_file,
+                                     ptr_filter->name,
+                                     "%s;%s%s%s;%s;%s",
+                                     (ptr_filter->enabled) ? "on" : "off",
+                                     (ptr_filter->plugin_name) ? ptr_filter->plugin_name : "",
+                                     (ptr_filter->plugin_name) ? "." : "",
+                                     ptr_filter->buffer_name,
+                                     ptr_filter->tags,
+                                     ptr_filter->regex))
+            return WEECHAT_CONFIG_WRITE_ERROR;
     }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
@@ -1118,31 +1128,37 @@ config_weechat_key_read_cb (void *data, struct t_config_file *config_file,
  * config_weechat_key_write_cb: write key section in configuration file
  */
 
-void
+int
 config_weechat_key_write_cb (void *data, struct t_config_file *config_file,
                              const char *section_name)
 {
     struct t_gui_key *ptr_key;
     char *expanded_name;
+    int rc;
     
     /* make C compiler happy */
     (void) data;
     
-    config_file_write_line (config_file, section_name, NULL);
+    if (!config_file_write_line (config_file, section_name, NULL))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     for (ptr_key = gui_keys; ptr_key; ptr_key = ptr_key->next_key)
     {
         expanded_name = gui_keyboard_get_expanded_name (ptr_key->key);
         if (expanded_name)
         {
-            config_file_write_line (config_file,
-                                    (expanded_name) ?
-                                    expanded_name : ptr_key->key,
-                                    "\"%s\"",
-                                    ptr_key->command);
+            rc = config_file_write_line (config_file,
+                                         (expanded_name) ?
+                                         expanded_name : ptr_key->key,
+                                         "\"%s\"",
+                                         ptr_key->command);
             free (expanded_name);
+            if (!rc)
+                return WEECHAT_CONFIG_WRITE_ERROR;
         }
     }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
@@ -1206,11 +1222,6 @@ config_weechat_init_options ()
         "display_version", "boolean",
         N_("display WeeChat version at startup"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
-    config_startup_weechat_slogan = config_file_new_option (
-        weechat_config_file, ptr_section,
-        "weechat_slogan", "string",
-        N_("WeeChat slogan (if empty, slogan is not used)"),
-        NULL, 0, 0, _("the geekiest chat client!"), NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     
     /* look */
     ptr_section = config_file_new_section (weechat_config_file, "look",
@@ -1234,7 +1245,7 @@ config_weechat_init_options ()
     config_look_buffer_time_format = config_file_new_option (
         weechat_config_file, ptr_section,
         "buffer_time_format", "string",
-        N_("time format for buffers"),
+        N_("time format for each line displayed in buffers"),
         NULL, 0, 0, "%H:%M:%S", NULL, 0, NULL, NULL, &config_change_buffer_time_format, NULL, NULL, NULL);
     config_look_color_nicks_number = config_file_new_option (
         weechat_config_file, ptr_section,
@@ -1250,11 +1261,18 @@ config_weechat_init_options ()
            "see real white instead of default term foreground "
            "color)"),
         NULL, 0, 0, "off", NULL, 0, NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_look_command_chars = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "command_chars", "string",
+        N_("chars used to determine if input string is a command or not: "
+           "input must start with one of these chars; the slash (\"/\") is "
+           "always considered as command prefix (example: \".$\")"),
+        NULL, 0, 0, "", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_look_day_change = config_file_new_option (
         weechat_config_file, ptr_section,
         "day_change", "boolean",
         N_("display special message when day changes"),
-        NULL, 0, 0, "on", NULL, 0, NULL, NULL, &config_change_day_change, NULL, NULL, NULL);
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_look_day_change_time_format = config_file_new_option (
         weechat_config_file, ptr_section,
         "day_change_time_format", "string",
@@ -1266,6 +1284,13 @@ config_weechat_init_options ()
         N_("comma separated list of words to highlight (case insensitive "
            "comparison, words may begin or end with \"*\" for partial match)"),
         NULL, 0, 0, "", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    config_look_hline_char = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "hline_char", "string",
+        N_("char used to draw horizontal lines, note that empty value will "
+           "draw a real line with ncurses, but may cause bugs with URL "
+           "selection under some terminals"),
+        NULL, 0, 0, "-", NULL, 0, NULL, NULL, &config_change_buffers, NULL, NULL, NULL);
     config_look_hotlist_names_count = config_file_new_option (
         weechat_config_file, ptr_section,
         "hotlist_names_count", "integer",
@@ -1407,6 +1432,11 @@ config_weechat_init_options ()
         N_("use a marker (line or char) on buffers to show first unread line"),
         "none|line|dotted-line|char",
         0, 0, "dotted-line", NULL, 0, NULL, NULL, &config_change_read_marker, NULL, NULL, NULL);
+    config_look_read_marker_always_show = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "read_marker_always_show", "boolean",
+        N_("always show read marker, even if it is after last buffer line"),
+        NULL, 0, 0, "off", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_look_save_config_on_exit = config_file_new_option (
         weechat_config_file, ptr_section,
         "save_config_on_exit", "boolean",
@@ -1440,6 +1470,12 @@ config_weechat_init_options ()
         N_("set title for window (terminal for Curses GUI) with "
            "name and version"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL, &config_change_title, NULL, NULL, NULL);
+    config_look_time_format = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "time_format", "string",
+        N_("time format for dates converted to strings and displayed in "
+           "messages"),
+        NULL, 0, 0, "%a, %d %b %Y %T", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     
     /* colors */
     ptr_section = config_file_new_section (weechat_config_file, "color",
@@ -1740,6 +1776,12 @@ config_weechat_init_options ()
         N_("text color for buffer with new data (status bar)"),
         NULL, -1, 0, "yellow", NULL, 0,
         NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_color_status_time = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "status_time", "color",
+        N_("text color for time (status bar)"),
+        NULL, -1, 0, "default", NULL, 0,
+        NULL, NULL, &config_change_color, NULL, NULL, NULL);
     /* input window */
     config_color_input_text_not_found = config_file_new_option (
         weechat_config_file, ptr_section,
@@ -1766,6 +1808,12 @@ config_weechat_init_options ()
         N_("text color for away nicknames"),
         NULL, -1, 0, "cyan", NULL, 0,
         NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_color_nicklist_offline = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "nicklist_offline", "color",
+        N_("text color for offline nicknames"),
+        NULL, -1, 0, "blue", NULL, 0,
+        NULL, NULL, &config_change_color, NULL, NULL, NULL);
     config_color_nicklist_prefix1 = config_file_new_option (
         weechat_config_file, ptr_section,
         "nicklist_prefix1", "color",
@@ -1796,12 +1844,6 @@ config_weechat_init_options ()
         N_("text color for prefix #5 in nicklist"),
         NULL, -1, 0, "brown", NULL, 0,
         NULL, NULL, &config_change_color, NULL, NULL, NULL);
-    config_color_nicklist_more = config_file_new_option (
-        weechat_config_file, ptr_section,
-        "nicklist_more", "color",
-        N_("text color for '+' when scrolling nicks in nicklist"),
-        NULL, -1, 0, "lightmagenta", NULL, 0,
-        NULL, NULL, &config_change_color, NULL, NULL, NULL);
     
     /* completion */
     ptr_section = config_file_new_section (weechat_config_file, "completion",
@@ -1931,12 +1973,11 @@ config_weechat_init_options ()
         return 0;
     }
     
-    config_network_gnutls_dh_prime_bits = config_file_new_option (
+    config_network_gnutls_ca_file = config_file_new_option (
         weechat_config_file, ptr_section,
-        "gnutls_dh_prime_bitsmax_lines", "integer",
-        N_("minimum size in bits for handshake using Diffie Hellman key "
-           "exchange"),
-        NULL, 0, INT_MAX, "512", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+        "gnutls_ca_file", "string",
+        N_("file containing the certificate authorities"),
+        NULL, 0, 0, "%h/ssl/CAs.pem", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     
     /* plugin */
     ptr_section = config_file_new_section (weechat_config_file, "plugin",
@@ -1953,9 +1994,10 @@ config_weechat_init_options ()
         weechat_config_file, ptr_section,
         "autoload", "string",
         N_("comma separated list of plugins to load automatically "
-           "at startup, \"*\" means all plugins found (names may "
-           "be partial, for example \"perl\" is ok for "
-           "\"perl.so\")"),
+           "at startup, \"*\" means all plugins found, a name beginning with "
+           "\"!\" is a negative value to prevent a plugin from being loaded, "
+           "names can start or end with \"*\" to match several plugins "
+           "(examples: \"*\" or \"*,!lua,!tcl\")"),
         NULL, 0, 0, "*", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_plugin_debug = config_file_new_option (
         weechat_config_file, ptr_section,
@@ -1969,11 +2011,11 @@ config_weechat_init_options ()
         N_("standard plugins extension in filename (for example "
            "\".so\" under Linux or \".dll\" under Microsoft Windows)"),
         NULL, 0, 0,
-#ifdef WIN32
+#if defined(WIN32) || defined(__CYGWIN__)
         ".dll",
 #else
         ".so",
-#endif              
+#endif
         NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_plugin_path = config_file_new_option (
         weechat_config_file, ptr_section,
@@ -2067,6 +2109,8 @@ int
 config_weechat_init ()
 {
     int rc;
+    struct timeval tv_time;
+    struct tm *local_time;
     
     rc = config_weechat_init_options ();
     
@@ -2076,6 +2120,20 @@ config_weechat_init ()
                          _("FATAL: error initializing configuration options"));
     }
     
+    /* create timer to check if day has changed */
+    if (!config_day_change_timer)
+    {
+        gettimeofday (&tv_time, NULL);
+        local_time = localtime (&tv_time.tv_sec);
+        config_day_change_old_day = local_time->tm_mday;
+        config_day_change_timer = hook_timer (NULL,
+                                              60 * 1000, /* each minute */
+                                              60, /* when second is 00 */
+                                              0,
+                                              &config_day_change_timer_cb,
+                                              NULL);
+    }
+    
     return rc;
 }
 
@@ -2095,7 +2153,6 @@ config_weechat_read ()
     rc = config_file_read (weechat_config_file);
     if (rc == WEECHAT_CONFIG_READ_OK)
     {
-        config_change_day_change (NULL, NULL);
         proxy_use_temp_proxies ();
         gui_bar_use_temp_bars ();
         gui_bar_create_default ();
