@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -181,9 +181,17 @@ struct t_hook_process
 /* hook connect */
 
 typedef int (t_hook_callback_connect)(void *data, int status,
+                                      int gnutls_rc,
                                       const char *error,
                                       const char *ip_address);
 
+#ifdef HAVE_GNUTLS
+typedef int (gnutls_callback_t)(void *data, gnutls_session_t tls_session,
+                                const gnutls_datum_t *req_ca, int nreq,
+                                const gnutls_pk_algorithm_t *pk_algos,
+                                int pk_algos_len, gnutls_retr_st *answer);
+#endif
+
 struct t_hook_connect
 {
     t_hook_callback_connect *callback; /* connect callback                  */
@@ -194,6 +202,8 @@ struct t_hook_connect
     int ipv6;                          /* IPv6 connection ?                 */
 #ifdef HAVE_GNUTLS
     gnutls_session_t *gnutls_sess;     /* GnuTLS session (SSL connection)   */
+    gnutls_callback_t *gnutls_cb;      /* GnuTLS callback during handshake  */
+    int gnutls_dhkey_size;             /* Diffie Hellman Key Exchange size  */
 #endif
     char *local_hostname;              /* force local hostname (optional)   */
     int child_read;                    /* to read data in pipe from child   */
@@ -280,6 +290,7 @@ struct t_hook_info
     t_hook_callback_info *callback;    /* info callback                     */
     char *info_name;                   /* name of info returned             */
     char *description;                 /* description                       */
+    char *args_description;            /* description of arguments          */
 };
 
 /* hook infolist */
@@ -294,6 +305,8 @@ struct t_hook_infolist
     t_hook_callback_infolist *callback; /* infolist callback                */
     char *infolist_name;                /* name of infolist returned        */
     char *description;                  /* description                      */
+    char *pointer_description;          /* description of pointer           */
+    char *args_description;             /* description of arguments         */
 };
 
 /* hook variables */
@@ -345,10 +358,18 @@ extern struct t_hook *hook_process (struct t_weechat_plugin *plugin,
 extern struct t_hook *hook_connect (struct t_weechat_plugin *plugin,
                                     const char *proxy, const char *address,
                                     int port, int sock, int ipv6,
-                                    void *gnutls_session,
+                                    void *gnutls_session, void *gnutls_cb,
+                                    int gnutls_dhkey_size,
                                     const char *local_hostname,
                                     t_hook_callback_connect *callback,
                                     void *callback_data);
+#ifdef HAVE_GNUTLS
+extern int hook_connect_gnutls_set_certificates (gnutls_session_t tls_session,
+                                                 const gnutls_datum_t *req_ca, int nreq,
+                                                 const gnutls_pk_algorithm_t *pk_algos,
+                                                 int pk_algos_len,
+                                                 gnutls_retr_st *answer);
+#endif
 extern struct t_hook *hook_print (struct t_weechat_plugin *plugin,
                                   struct t_gui_buffer *buffer,
                                   const char *tags, const char *message,
@@ -391,6 +412,7 @@ extern char *hook_modifier_exec (struct t_weechat_plugin *plugin,
 extern struct t_hook *hook_info (struct t_weechat_plugin *plugin,
                                  const char *info_name,
                                  const char *description,
+                                 const char *args_description,
                                  t_hook_callback_info *callback,
                                  void *callback_data);
 extern const char *hook_info_get (struct t_weechat_plugin *plugin,
@@ -399,6 +421,8 @@ extern const char *hook_info_get (struct t_weechat_plugin *plugin,
 extern struct t_hook *hook_infolist (struct t_weechat_plugin *plugin,
                                      const char *infolist_name,
                                      const char *description,
+                                     const char *pointer_description,
+                                     const char *args_description,
                                      t_hook_callback_infolist *callback,
                                      void *callback_data);
 extern struct t_infolist *hook_infolist_get (struct t_weechat_plugin *plugin,
