@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,10 @@
 struct t_gui_buffer;
 struct t_weechat_plugin;
 
-extern int input_exec_command (struct t_gui_buffer *buffer,
-                               int any_plugin,
-                               struct t_weechat_plugin *plugin,
-                               const char *string);
+extern void input_exec_command (struct t_gui_buffer *buffer,
+                                int any_plugin,
+                                struct t_weechat_plugin *plugin,
+                                const char *string);
 extern void input_data (struct t_gui_buffer *buffer, const char *data);
 
 #endif /* wee-input.h */
