@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* wee-network.c: network functions for WeeChat */
+/*
+ * wee-network.c: network functions for WeeChat
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +39,10 @@
 #include <gnutls/gnutls.h>
 #endif
 
+#ifdef HAVE_GCRYPT
+#include <gcrypt.h>
+#endif
+
 #include "weechat.h"
 #include "wee-network.h"
 #include "wee-hook.h"
@@ -47,6 +53,7 @@
 
 
 #ifdef HAVE_GNUTLS
+int network_init_ok = 0;
 gnutls_certificate_credentials gnutls_xcred; /* GnuTLS client credentials   */
 const int gnutls_cert_type_prio[] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
 #if LIBGNUTLS_VERSION_NUMBER >= 0x010700
@@ -67,9 +74,32 @@ void
 network_init ()
 {
 #ifdef HAVE_GNUTLS
+    char *ca_path, *ca_path2;
+
     gnutls_global_init ();
     gnutls_certificate_allocate_credentials (&gnutls_xcred);
-    gnutls_certificate_set_x509_trust_file (gnutls_xcred, "ca.pem", GNUTLS_X509_FMT_PEM);
+    
+    ca_path = string_replace (CONFIG_STRING(config_network_gnutls_ca_file),
+                              "~", getenv ("HOME"));
+    if (ca_path)
+    {
+        ca_path2 = string_replace (ca_path, "%h", weechat_home);
+        if (ca_path2)
+        {
+            gnutls_certificate_set_x509_trust_file (gnutls_xcred, ca_path2,
+                                                    GNUTLS_X509_FMT_PEM);
+            free (ca_path2);
+        }
+        free (ca_path);
+    }
+    gnutls_certificate_client_set_retrieve_function (gnutls_xcred,
+                                                     &hook_connect_gnutls_set_certificates);
+    network_init_ok = 1;
+#endif
+#ifdef HAVE_GCRYPT
+    gcry_check_version (GCRYPT_VERSION);
+    gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+    gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 #endif
 }
 
@@ -81,72 +111,15 @@ void
 network_end ()
 {
 #ifdef HAVE_GNUTLS
-    gnutls_certificate_free_credentials (gnutls_xcred);
-    gnutls_global_deinit();
+    if (network_init_ok)
+    {
+        network_init_ok = 0;
+        gnutls_certificate_free_credentials (gnutls_xcred);
+        gnutls_global_deinit();
+    }
 #endif
 }
 
-/*
- * network_convbase64_8x3_to_6x4 : convert 3 bytes of 8 bits in 4 bytes of 6 bits
- */
-
-void
-network_convbase64_8x3_to_6x4 (const char *from, char *to)
-{
-    unsigned char base64_table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-        "abcdefghijklmnopqrstuvwxyz0123456789+/";
-    
-    to[0] = base64_table [ (from[0] & 0xfc) >> 2 ];
-    to[1] = base64_table [ ((from[0] & 0x03) << 4) + ((from[1] & 0xf0) >> 4) ];
-    to[2] = base64_table [ ((from[1] & 0x0f) << 2) + ((from[2] & 0xc0) >> 6) ];
-    to[3] = base64_table [ from[2] & 0x3f ];
-}
-
-/*
- * network_base64encode: encode a string in base64
- */
-
-void
-network_base64encode (const char *from, char *to)
-{
-    const char *f;
-    char *t;
-    int from_len;
-    
-    from_len = strlen (from);
-    
-    f = from;
-    t = to;
-    
-    while (from_len >= 3)
-    {
-        network_convbase64_8x3_to_6x4 (f, t);
-        f += 3 * sizeof (*f);
-        t += 4 * sizeof (*t);
-        from_len -= 3;
-    }
-    
-    if (from_len > 0)
-    {
-        char rest[3] = { 0, 0, 0 };
-        switch (from_len)
-        {
-            case 1 :
-                rest[0] = f[0];
-                network_convbase64_8x3_to_6x4 (rest, t);
-                t[2] = t[3] = '=';
-                break;
-            case 2 :
-                rest[0] = f[0];
-                rest[1] = f[1];
-                network_convbase64_8x3_to_6x4 (rest, t);
-                t[3] = '=';
-                break;
-        }
-        t[4] = 0;
-    }
-}
-
 /*
  * network_pass_httpproxy: establish connection/authentification to an
  *                         http proxy
@@ -169,12 +142,12 @@ network_pass_httpproxy (struct t_proxy *proxy, int sock, const char *address,
                   CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME]),
                   (CONFIG_STRING(proxy->options[PROXY_OPTION_PASSWORD])) ?
                   CONFIG_STRING(proxy->options[PROXY_OPTION_PASSWORD]) : "");
-        network_base64encode (authbuf, authbuf_base64);
+        string_encode_base64 (authbuf, strlen (authbuf), authbuf_base64);
         n = snprintf (buffer, sizeof (buffer),
                       "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n",
                       address, port, authbuf_base64);
     }
-    else 
+    else
     {
         /* no authentification */
         n = snprintf (buffer, sizeof (buffer),
@@ -289,7 +262,7 @@ int
 network_pass_socks5proxy (struct t_proxy *proxy, int sock, const char *address,
                           int port)
 {
-    /* 
+    /*
      * socks5 protocol is explained in RFC 1928
      * socks5 authentication with username/pass is explained in RFC 1929
      */
@@ -316,8 +289,9 @@ network_pass_socks5proxy (struct t_proxy *proxy, int sock, const char *address,
     if (CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME])
         && CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME])[0])
     {
-        /* with authentication */
-        /*   -> socks server must respond with :
+        /*
+         * with authentication
+         *   -> socks server must respond with :
          *       - socks version (buffer[0]) = 5 => socks5
          *       - socks method  (buffer[1]) = 2 => authentication
          */
@@ -349,8 +323,9 @@ network_pass_socks5proxy (struct t_proxy *proxy, int sock, const char *address,
     }
     else
     {
-        /* without authentication */
-        /*   -> socks server must respond with :
+        /*
+         * without authentication
+         *   -> socks server must respond with :
          *       - socks version (buffer[0]) = 5 => socks5
          *       - socks method  (buffer[1]) = 0 => no authentication
          */
@@ -386,7 +361,8 @@ network_pass_socks5proxy (struct t_proxy *proxy, int sock, const char *address,
     switch (buffer[3])
     {
         case 1:
-            /* ipv4 
+            /*
+             * ipv4
              * server socks return server bound address and port
              * address of 4 bytes and port of 2 bytes (= 6 bytes)
              */
@@ -394,19 +370,21 @@ network_pass_socks5proxy (struct t_proxy *proxy, int sock, const char *address,
                 return 0;
             break;
         case 3:
-            /* domainname
+            /*
+             * domainname
              * server socks return server bound address and port
              */
-            /* reading address length */
+            /* read address length */
             if (recv (sock, buffer, 1, 0) != 1)
-                return 0;    
+                return 0;
             addr_len = buffer[0];
-            /* reading address + port = addr_len + 2 */
+            /* read address + port = addr_len + 2 */
             if (recv (sock, buffer, addr_len + 2, 0) != (addr_len + 2))
                 return 0;
             break;
         case 4:
-            /* ipv6
+            /*
+             * ipv6
              * server socks return server bound address and port
              * address of 16 bytes and port of 2 bytes (= 18 bytes)
              */
@@ -601,8 +579,8 @@ network_connect_child (struct t_hook *hook_connect)
             freeaddrinfo (res);
             return;
         }
-        
-	status_str[0] = '0' + WEECHAT_HOOK_CONNECT_OK;
+
+        status_str[0] = '0' + WEECHAT_HOOK_CONNECT_OK;
     }
     else
     {
@@ -804,8 +782,10 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
             {
                 gnutls_transport_set_ptr (*HOOK_CONNECT(hook_connect, gnutls_sess),
                                           (gnutls_transport_ptr) ((unsigned long) HOOK_CONNECT(hook_connect, sock)));
-                gnutls_dh_set_prime_bits (*HOOK_CONNECT(hook_connect, gnutls_sess),
-                                          CONFIG_INTEGER(config_network_gnutls_dh_prime_bits));
+                if (HOOK_CONNECT(hook_connect, gnutls_dhkey_size) > 0) {
+                    gnutls_dh_set_prime_bits (*HOOK_CONNECT(hook_connect, gnutls_sess),
+                                              (unsigned int) HOOK_CONNECT(hook_connect, gnutls_dhkey_size));
+                }
                 while (1)
                 {
                     rc = gnutls_handshake (*HOOK_CONNECT(hook_connect, gnutls_sess));
@@ -819,6 +799,7 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                     (void) (HOOK_CONNECT(hook_connect, callback))
                         (hook_connect->callback_data,
                          WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
+                         rc,
                          gnutls_strerror (rc),
                          ip_address);
                     unhook (hook_connect);
@@ -830,7 +811,7 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
 #endif
         }
         (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data, buffer[0] - '0', NULL, ip_address);
+            (hook_connect->callback_data, buffer[0] - '0', 0, NULL, ip_address);
         unhook (hook_connect);
     }
     
@@ -861,7 +842,7 @@ network_connect_with_fork (struct t_hook *hook_connect)
             (void) (HOOK_CONNECT(hook_connect, callback))
                 (hook_connect->callback_data,
                  '0' + WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
-                 NULL, NULL);
+                 0, NULL, NULL);
             unhook (hook_connect);
             return;
         }
@@ -884,7 +865,7 @@ network_connect_with_fork (struct t_hook *hook_connect)
         (void) (HOOK_CONNECT(hook_connect, callback))
             (hook_connect->callback_data,
              '0' + WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
-             NULL, NULL);
+             0, NULL, NULL);
         unhook (hook_connect);
         return;
     }
@@ -892,12 +873,13 @@ network_connect_with_fork (struct t_hook *hook_connect)
     HOOK_CONNECT(hook_connect, child_write) = child_pipe[1];
     
 #ifdef __CYGWIN__
-    /* connection may block under Cygwin, there's no other known way
-       to do better today, since connect() in child process seems not to work
-       any suggestion is welcome to improve that!
+    /*
+     * connection may block under Cygwin, there's no other known way
+     * to do better today, since connect() in child process seems not to work
+     * any suggestion is welcome to improve that!
     */
     network_connect_child (hook_connect);
-    network_connect_child_read_cb (hook_connect);
+    network_connect_child_read_cb (hook_connect, 0);
 #else
     switch (pid = fork ())
     {
@@ -906,7 +888,7 @@ network_connect_with_fork (struct t_hook *hook_connect)
             (void) (HOOK_CONNECT(hook_connect, callback))
                 (hook_connect->callback_data,
                  '0' + WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
-                 NULL, NULL);
+                 0, NULL, NULL);
             unhook (hook_connect);
             return;
         /* child process */
