@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* wee-string.c: string functions for WeeChat */
+/*
+ * wee-string.c: string functions for WeeChat
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +52,7 @@
 
 #include "weechat.h"
 #include "wee-string.h"
+#include "wee-config.h"
 #include "wee-utf8.h"
 #include "../gui/gui-color.h"
 
@@ -652,7 +655,7 @@ string_has_highlight (const char *string, const char *highlight_words)
                 startswith = ((match == msg) || (!string_is_word_char (match_pre)));
                 endswith = ((!match_post[0]) || (!string_is_word_char (match_post)));
                 if ((wildcard_start && wildcard_end) ||
-                    (!wildcard_start && !wildcard_end && 
+                    (!wildcard_start && !wildcard_end &&
                      startswith && endswith) ||
                     (wildcard_start && endswith) ||
                     (wildcard_end && startswith))
@@ -940,65 +943,65 @@ string_split_command (const char *command, char separator)
     const char *ptr;
 
     if (!command || !command[0])
-	return NULL;
+        return NULL;
     
     nb_substr = 1;
     ptr = command;
     while ( (p = strchr(ptr, separator)) != NULL)
     {
-	nb_substr++;
-	ptr = ++p;
+        nb_substr++;
+        ptr = ++p;
     }
 
     array = malloc ((nb_substr + 1) * sizeof (array[0]));
     if (!array)
-	return NULL;
+        return NULL;
     
     buffer = malloc (strlen(command) + 1);
     if (!buffer)
     {
-	free (array);
-	return NULL;
+        free (array);
+        return NULL;
     }
     
     ptr = command;
     str_idx = 0;
     arr_idx = 0;
-    while(*ptr != '\0') 
-    {	
-	type = 0;
-	if (*ptr == ';')
-	{
-	    if (ptr == command)
-		type = 1;
-	    else if ( *(ptr-1) != '\\')
-		type = 1;
-	    else if ( *(ptr-1) == '\\')
-		type = 2;
-	}	
-	if (type == 1)
-	{
-	    buffer[str_idx] = '\0';
-	    str_idx = -1;
-	    p = buffer;
-	    /* strip white spaces a the begining of the line */
-	    while (*p == ' ') p++;
-	    if (p  && p[0])
-		array[arr_idx++] = strdup (p);
-	}	
-	else if (type == 2)
-	    buffer[--str_idx] = *ptr;
-	else
-	    buffer[str_idx] = *ptr;
-	str_idx++;
-	ptr++;
+    while(*ptr != '\0')
+    {
+        type = 0;
+        if (*ptr == ';')
+        {
+            if (ptr == command)
+                type = 1;
+            else if ( *(ptr-1) != '\\')
+                type = 1;
+            else if ( *(ptr-1) == '\\')
+                type = 2;
+        }
+        if (type == 1)
+        {
+            buffer[str_idx] = '\0';
+            str_idx = -1;
+            p = buffer;
+            /* strip white spaces a the begining of the line */
+            while (*p == ' ') p++;
+            if (p  && p[0])
+                array[arr_idx++] = strdup (p);
+        }
+        else if (type == 2)
+            buffer[--str_idx] = *ptr;
+        else
+            buffer[str_idx] = *ptr;
+        str_idx++;
+        ptr++;
     }
     
     buffer[str_idx] = '\0';
     p = buffer;
     while (*p == ' ') p++;
     if (p  && p[0])
-	array[arr_idx++] = strdup (p);
+        array[arr_idx++] = strdup (p);
     
     array[arr_idx] = NULL;
 
@@ -1155,8 +1158,10 @@ string_iconv_to_internal (const char *charset, const char *string)
     if (!input)
         return NULL;
     
-    /* optimize for UTF-8: if charset is NULL => we use term charset =>
-       if ths charset is already UTF-8, then no iconv needed */
+    /*
+     * optimized for UTF-8: if charset is NULL => we use term charset => if
+     * this charset is already UTF-8, then no iconv is needed
+     */
     if (local_utf8 && (!charset || !charset[0]))
         return input;
     
@@ -1193,8 +1198,10 @@ string_iconv_from_internal (const char *charset, const char *string)
     if (!input)
         return NULL;
     
-    /* optimize for UTF-8: if charset is NULL => we use term charset =>
-       if ths charset is already UTF-8, then no iconv needed */
+    /*
+     * optimized for UTF-8: if charset is NULL => we use term charset => if
+     * this charset is already UTF-8, then no iconv needed
+     */
     if (local_utf8 && (!charset || !charset[0]))
         return input;
     
@@ -1212,28 +1219,34 @@ string_iconv_from_internal (const char *charset, const char *string)
 
 /*
  * string_iconv_fprintf: encode to terminal charset, then call fprintf on a file
+ *                       return 1 if ok, 0 if error
  */
 
-void
+int
 string_iconv_fprintf (FILE *file, const char *data, ...)
 {
     va_list argptr;
     char *buf, *buf2;
-
+    int rc, num_written;
+    
     buf = malloc (128 * 1024);
     if (!buf)
-        return;
+        return 0;
     
     va_start (argptr, data);
     vsnprintf (buf, 128 * 1024, data, argptr);
     va_end (argptr);
     
     buf2 = string_iconv_from_internal (NULL, buf);
-    fprintf (file, "%s", (buf2) ? buf2 : buf);
+    num_written = fprintf (file, "%s", (buf2) ? buf2 : buf);
+    
+    rc = (num_written == (int)strlen ((buf2) ? buf2 : buf)) ? 1 : 0;
     
     free (buf);
     if (buf2)
         free (buf2);
+    
+    return rc;
 }
 
 /*
@@ -1271,3 +1284,222 @@ string_format_size (unsigned long size)
     
     return strdup (str_size);
 }
+
+/*
+ * string_convbase64_8x3_to_6x4 : convert 3 bytes of 8 bits in 4 bytes of 6 bits
+ */
+
+void
+string_convbase64_8x3_to_6x4 (const char *from, char *to)
+{
+    unsigned char base64_table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+        "abcdefghijklmnopqrstuvwxyz0123456789+/";
+    
+    to[0] = base64_table [ (from[0] & 0xfc) >> 2 ];
+    to[1] = base64_table [ ((from[0] & 0x03) << 4) + ((from[1] & 0xf0) >> 4) ];
+    to[2] = base64_table [ ((from[1] & 0x0f) << 2) + ((from[2] & 0xc0) >> 6) ];
+    to[3] = base64_table [ from[2] & 0x3f ];
+}
+
+/*
+ * string_encode_base64: encode a string in base64
+ *                       length is number of bytes in "from" to convert
+ *                       (commonly strlen(from))
+ */
+
+void
+string_encode_base64 (const char *from, int length, char *to)
+{
+    const char *ptr_from;
+    char *ptr_to;
+    
+    ptr_from = from;
+    ptr_to = to;
+    
+    while (length >= 3)
+    {
+        string_convbase64_8x3_to_6x4 (ptr_from, ptr_to);
+        ptr_from += 3 * sizeof (*ptr_from);
+        ptr_to += 4 * sizeof (*ptr_to);
+        length -= 3;
+    }
+    
+    if (length > 0)
+    {
+        char rest[3] = { 0, 0, 0 };
+        switch (length)
+        {
+            case 1 :
+                rest[0] = ptr_from[0];
+                string_convbase64_8x3_to_6x4 (rest, ptr_to);
+                ptr_to[2] = ptr_to[3] = '=';
+                break;
+            case 2 :
+                rest[0] = ptr_from[0];
+                rest[1] = ptr_from[1];
+                string_convbase64_8x3_to_6x4 (rest, ptr_to);
+                ptr_to[3] = '=';
+                break;
+        }
+        ptr_to[4] = 0;
+    }
+    else
+        ptr_to[0] = '\0';
+}
+
+/*
+ * string_convbase64_6x4_to_8x3 : convert 4 bytes of 6 bits to 3 bytes of 8 bits
+ */
+
+void
+string_convbase64_6x4_to_8x3 (const unsigned char *from, unsigned char *to)
+{
+    to[0] = from[0] << 2 | from[1] >> 4;
+    to[1] = from[1] << 4 | from[2] >> 2;
+    to[2] = ((from[2] << 6) & 0xc0) | from[3];
+}
+
+/*
+ * string_decode_base64: decode a base64 string
+ *                       return length of string in *to
+ *                       (does not count final \0)
+ */
+
+int
+string_decode_base64 (const char *from, char *to)
+{
+    const char *ptr_from;
+    int length, to_length, i;
+    char *ptr_to;
+    unsigned char c, in[4], out[3];
+    unsigned char base64_table[]="|$$$}rstuvwxyz{$$$$$$$>?"
+        "@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";
+    
+    ptr_from = from;
+    ptr_to = to;
+    
+    ptr_to[0] = '\0';
+    to_length = 0;
+    
+    while (ptr_from && ptr_from[0])
+    {
+        length = 0;
+        for (i = 0; i < 4; i++)
+        {
+            c = 0;
+            while (ptr_from[0] && (c == 0))
+            {
+                c = (unsigned char) ptr_from[0];
+                ptr_from++;
+                c = ((c < 43) || (c > 122)) ? 0 : base64_table[c - 43];
+                if (c)
+                    c = (c == '$') ? 0 : c - 61;
+            }
+            if (ptr_from[0])
+            {
+                length++;
+                if (c)
+                    in[i] = c - 1;
+            }
+            else
+            {
+                in[i] = '\0';
+                break;
+            }
+        }
+        if (length)
+        {
+            string_convbase64_6x4_to_8x3 (in, out);
+            for (i = 0; i < length - 1; i++)
+            {
+                ptr_to[0] = out[i];
+                ptr_to++;
+                to_length++;
+            }
+        }
+    }
+    
+    ptr_to[0] = '\0';
+    
+    return to_length;
+}
+
+/*
+ * string_is_command_char: return 1 if first char of string is a command char,
+ *                         otherwise 0
+ */
+
+int
+string_is_command_char (const char *string)
+{
+    const char *ptr_command_chars;
+    
+    if (!string)
+        return 0;
+    
+    if (string[0] == '/')
+        return 1;
+    
+    ptr_command_chars = CONFIG_STRING(config_look_command_chars);
+    if (!ptr_command_chars || !ptr_command_chars[0])
+        return 0;
+    
+    while (ptr_command_chars && ptr_command_chars[0])
+    {
+        if (utf8_charcmp (ptr_command_chars, string) == 0)
+            return 1;
+        ptr_command_chars = utf8_next_char (ptr_command_chars);
+    }
+    
+    return 0;
+}
+
+/*
+ * string_input_for_buffer: return pointer to input text for buffer (pointer
+ *                          inside "string" argument)
+ *                          or return NULL if it's a command
+ *                          (by default, a command starts with a single '/')
+ */
+
+const char *
+string_input_for_buffer (const char *string)
+{
+    char *pos_slash, *pos_space, *next_char;
+    
+    /* special case for C comments pasted in input line */
+    if (strncmp (string, "/*", 2) == 0)
+        return string;
+    
+    /*
+     * special case if string starts with '/': to allow to paste a path line
+     * "/path/to/file.txt", we check if next '/' is after a space or not
+     */
+    if (string[0] == '/')
+    {
+        pos_slash = strchr (string + 1, '/');
+        pos_space = strchr (string + 1, ' ');
+        
+        /*
+         * if there's no other '/' of if '/' is after first space,
+         * then it is a command, and return NULL
+         */
+        if (!pos_slash || (pos_space && pos_slash > pos_space))
+            return NULL;
+        
+        return (string[1] == '/') ? string + 1 : string;
+    }
+    
+    /* if string does not start with a command char, then it's not command */
+    if (!string_is_command_char (string))
+        return string;
+    
+    /* check if first char is doubled: if yes, then it's not a command */
+    next_char = utf8_next_char (string);
+    if (!next_char || !next_char[0])
+        return string;
+    if (utf8_charcmp (string, next_char) == 0)
+        return next_char;
+    
+    /* string is a command */
+    return NULL;
+}
