@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* wee-upgrade.c: save/restore session data */
+/*
+ * wee-upgrade.c: save/restore session data of WeeChat core
+ *                (buffers, command history, hotlist, uptime)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -135,7 +138,7 @@ upgrade_weechat_save_buffers (struct t_upgrade_file *upgrade_file)
             ptr_infolist = infolist_new ();
             if (!ptr_infolist)
                 return 0;
-            if (!gui_nicklist_add_to_infolist (ptr_infolist, ptr_buffer))
+            if (!gui_nicklist_add_to_infolist (ptr_infolist, ptr_buffer, NULL))
             {
                 infolist_free (ptr_infolist);
                 return 0;
@@ -332,8 +335,10 @@ upgrade_weechat_read_cb (void *data,
                 }
                 else
                 {
-                    /* create buffer if it was created by a plugin (ie not
-                       WeeChat main buffer) */
+                    /*
+                     * create buffer if it was created by a plugin (ie not
+                     * WeeChat main buffer)
+                     */
                     ptr_buffer_for_merge = last_gui_buffer;
                     upgrade_current_buffer = gui_buffer_new (
                         NULL,
@@ -542,7 +547,7 @@ upgrade_weechat_read_cb (void *data,
                     hotlist_reset = 1;
                 }
                 plugin_name = infolist_string (infolist, "plugin_name");
-                buffer_name = infolist_string (infolist, "plugin_name");
+                buffer_name = infolist_string (infolist, "buffer_name");
                 if (plugin_name && buffer_name)
                 {
                     ptr_buffer = gui_buffer_search_by_name (plugin_name,
