@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-curses-chat.c: chat display functions for Curses GUI */
+/*
+ * gui-curses-chat.c: chat display functions for Curses GUI
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -67,6 +69,11 @@ gui_chat_marker_for_line (struct t_gui_buffer *buffer, struct t_gui_line *line)
 {
     struct t_gui_line *last_read_line;
     
+    /* marker is disabled in config? */
+    if ((CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_LINE)
+        && (CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_DOTTED_LINE))
+        return 0;
+    
     /* marker is not set for buffer? */
     if (!buffer->lines->last_read_line)
         return 0;
@@ -78,15 +85,14 @@ gui_chat_marker_for_line (struct t_gui_buffer *buffer, struct t_gui_line *line)
     if (!last_read_line)
         return 0;
     
-    /* marker is disabled in config? */
-    if ((CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_LINE)
-        && (CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_DOTTED_LINE))
-        return 0;
-    
     while (line)
     {
         if (last_read_line == line)
-            return 1;
+        {
+            if (CONFIG_BOOLEAN(config_look_read_marker_always_show))
+                return 1;
+            return (gui_line_get_next_displayed (line) != NULL) ? 1 : 0;
+        }
         
         if (line->data->displayed)
             break;
@@ -128,7 +134,9 @@ gui_chat_display_new_line (struct t_gui_window *window, int num_lines, int count
 void
 gui_chat_display_horizontal_line (struct t_gui_window *window, int simulate)
 {
-    int i, n;
+    int i, n, hline_char;
+    
+    hline_char = gui_window_get_hline_char ();
     
     if (!simulate)
     {
@@ -139,8 +147,7 @@ gui_chat_display_horizontal_line (struct t_gui_window *window, int simulate)
             case CONFIG_LOOK_READ_MARKER_LINE:
                 mvwhline (GUI_WINDOW_OBJECTS(window)->win_chat,
                           window->win_chat_cursor_y, window->win_chat_cursor_x,
-                          ACS_HLINE,
-                          window->win_chat_width - 1);
+                          hline_char, window->win_chat_width - 1);
                 break;
             case CONFIG_LOOK_READ_MARKER_DOTTED_LINE:
                 wmove (GUI_WINDOW_OBJECTS(window)->win_chat,
@@ -152,7 +159,7 @@ gui_chat_display_horizontal_line (struct t_gui_window *window, int simulate)
                     if (i % 2 == n)
                         mvwhline (GUI_WINDOW_OBJECTS(window)->win_chat,
                                   window->win_chat_cursor_y, i,
-                                  ACS_HLINE, 1);
+                                  hline_char, 1);
                 }
                 break;
             default:
@@ -245,6 +252,7 @@ gui_chat_string_next_char (struct t_gui_window *window,
                             case GUI_COLOR_BAR_DELIM_CHAR:
                             case GUI_COLOR_BAR_BG_CHAR:
                             case GUI_COLOR_BAR_START_INPUT_CHAR:
+                            case GUI_COLOR_BAR_START_INPUT_HIDDEN_CHAR:
                             case GUI_COLOR_BAR_MOVE_CURSOR_CHAR:
                                 string += 2;
                                 break;
@@ -563,12 +571,8 @@ gui_chat_display_time_and_prefix (struct t_gui_window *window,
     {
         if (!simulate)
             gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
-        
-        gui_chat_display_word (window, line, str_space,
-                               NULL, 1, num_lines, count, lines_displayed,
-                               simulate);
     }
-
+    
     /* display buffer name (if many buffers are merged) */
     mixed_lines = line->data->buffer->mixed_lines;
     if (mixed_lines)
@@ -699,7 +703,7 @@ gui_chat_display_time_and_prefix (struct t_gui_window *window,
                 gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
                                               GUI_COLOR_CHAT_HIGHLIGHT);
             }
-        } 
+        }
         
         /* not enough space to display full prefix? => truncate it! */
         if ((CONFIG_INTEGER(config_look_prefix_align) != CONFIG_LOOK_PREFIX_ALIGN_NONE)
@@ -928,7 +932,7 @@ gui_chat_display_line (struct t_gui_window *window, struct t_gui_line *line,
         }
     }
     
-    if (marker_line && gui_line_get_next_displayed (line))
+    if (marker_line)
     {
         gui_chat_display_horizontal_line (window, simulate);
         gui_chat_display_new_line (window, num_lines, count,
@@ -1291,8 +1295,10 @@ gui_chat_draw (struct t_gui_buffer *buffer, int erase)
 void
 gui_chat_draw_line (struct t_gui_buffer *buffer, struct t_gui_line *line)
 {
-    /* This function does nothing in Curses GUI,
-       line will be displayed by gui_buffer_draw_chat()  */
+    /*
+     * This function does nothing in Curses GUI,
+     * line will be displayed by gui_buffer_draw_chat()
+     */
     (void) buffer;
     (void) line;
 }
