@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-curses-keyboard.c: keyboard functions for Curses GUI */
+/*
+ * gui-curses-keyboard.c: keyboard functions for Curses GUI
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +45,8 @@
 #include "../gui-window.h"
 #include "gui-curses.h"
 
+#define BIND(key, command) gui_keyboard_default_bind(key, command)
+
 
 /*
  * gui_keyboard_default_bind: create key bind, only if it does not exist yet
@@ -56,7 +60,8 @@ gui_keyboard_default_bind (const char *key, const char *command)
     
     internal_code = gui_keyboard_get_internal_code (key);
     
-    ptr_key = gui_keyboard_search (NULL, (internal_code) ? internal_code : key);
+    ptr_key = gui_keyboard_search (gui_keys,
+                                   (internal_code) ? internal_code : key);
     if (!ptr_key)
     {
         gui_keyboard_new (NULL, key, command);
@@ -76,113 +81,119 @@ gui_keyboard_default_bindings ()
     int i;
     char key_str[32], command[32];
     
-    gui_keyboard_default_bind (/* RC            */ "ctrl-M",             "/input return");
-    gui_keyboard_default_bind (/* RC            */ "ctrl-J",             "/input return");
-    gui_keyboard_default_bind (/* tab           */ "ctrl-I",             "/input complete_next");
-    gui_keyboard_default_bind (/* s-tab         */ "meta2-Z",            "/input complete_previous");
-    gui_keyboard_default_bind (/* ^R            */ "ctrl-R",             "/input search_text");
-    gui_keyboard_default_bind (/* basckpace     */ "ctrl-H",             "/input delete_previous_char");
-    gui_keyboard_default_bind (/* basckpace     */ "ctrl-?",             "/input delete_previous_char");
-    gui_keyboard_default_bind (/* del           */ "meta2-3~",           "/input delete_next_char");
-    gui_keyboard_default_bind (/* ^D            */ "ctrl-D",             "/input delete_next_char");
-    gui_keyboard_default_bind (/* ^W            */ "ctrl-W",             "/input delete_previous_word");
-    gui_keyboard_default_bind (/* ^X            */ "ctrl-X",             "/input switch_active_buffer");
-    gui_keyboard_default_bind (/* m-d           */ "meta-d",             "/input delete_next_word");
-    gui_keyboard_default_bind (/* ^K            */ "ctrl-K",             "/input delete_end_of_line");
-    gui_keyboard_default_bind (/* m-r           */ "meta-r",             "/input delete_line");
-    gui_keyboard_default_bind (/* ^T            */ "ctrl-T",             "/input transpose_chars");
-    gui_keyboard_default_bind (/* ^U            */ "ctrl-U",             "/input delete_beginning_of_line");
-    gui_keyboard_default_bind (/* ^Y            */ "ctrl-Y",             "/input clipboard_paste");
-    gui_keyboard_default_bind (/* home          */ "meta2-1~",           "/input move_beginning_of_line");
-    gui_keyboard_default_bind (/* home          */ "meta2-H",            "/input move_beginning_of_line");
-    gui_keyboard_default_bind (/* home          */ "meta2-7~",           "/input move_beginning_of_line");
-    gui_keyboard_default_bind (/* ^A            */ "ctrl-A",             "/input move_beginning_of_line");
-    gui_keyboard_default_bind (/* end           */ "meta2-4~",           "/input move_end_of_line");
-    gui_keyboard_default_bind (/* end           */ "meta2-F",            "/input move_end_of_line");
-    gui_keyboard_default_bind (/* end           */ "meta2-8~",           "/input move_end_of_line");
-    gui_keyboard_default_bind (/* ^E            */ "ctrl-E",             "/input move_end_of_line");
-    gui_keyboard_default_bind (/* left          */ "meta2-D",            "/input move_previous_char");
-    gui_keyboard_default_bind (/* ^B            */ "ctrl-B",             "/input move_previous_char");
-    gui_keyboard_default_bind (/* right         */ "meta2-C",            "/input move_next_char");
-    gui_keyboard_default_bind (/* ^F            */ "ctrl-F",             "/input move_next_char");
-    gui_keyboard_default_bind (/* m-b           */ "meta-b",             "/input move_previous_word");
-    gui_keyboard_default_bind (/* ^left         */ "meta-Od",            "/input move_previous_word");
-    gui_keyboard_default_bind (/* m-f           */ "meta-f",             "/input move_next_word");
-    gui_keyboard_default_bind (/* ^right        */ "meta-Oc",            "/input move_next_word");
-    gui_keyboard_default_bind (/* up            */ "meta2-A",            "/input history_previous");
-    gui_keyboard_default_bind (/* down          */ "meta2-B",            "/input history_next");
-    gui_keyboard_default_bind (/* ^up           */ "meta-Oa",            "/input history_global_previous");
-    gui_keyboard_default_bind (/* ^down         */ "meta-Ob",            "/input history_global_next");
-    gui_keyboard_default_bind (/* m-a           */ "meta-a",             "/input jump_smart");
-    gui_keyboard_default_bind (/* m-j,m-l       */ "meta-jmeta-l",       "/input jump_last_buffer");
-    gui_keyboard_default_bind (/* m-j,m-r       */ "meta-jmeta-r",       "/server raw");
-    gui_keyboard_default_bind (/* m-j,m-s       */ "meta-jmeta-s",       "/server jump");
-    gui_keyboard_default_bind (/* m-h           */ "meta-h",             "/input hotlist_clear");
-    gui_keyboard_default_bind (/* m-k           */ "meta-k",             "/input grab_key");
-    gui_keyboard_default_bind (/* m-u           */ "meta-u",             "/input scroll_unread");
-    gui_keyboard_default_bind (/* ^S^U          */ "ctrl-Sctrl-U",       "/input set_unread");
-    gui_keyboard_default_bind (/* ^Cb           */ "ctrl-Cb",            "/input insert \\x02");
-    gui_keyboard_default_bind (/* ^Cc           */ "ctrl-Cc",            "/input insert \\x03");
-    gui_keyboard_default_bind (/* ^Ci           */ "ctrl-Ci",            "/input insert \\x1D");
-    gui_keyboard_default_bind (/* ^Co           */ "ctrl-Co",            "/input insert \\x0F");
-    gui_keyboard_default_bind (/* ^Cr           */ "ctrl-Cr",            "/input insert \\x12");
-    gui_keyboard_default_bind (/* ^Cu           */ "ctrl-Cu",            "/input insert \\x15");
-    gui_keyboard_default_bind (/* m-left        */ "meta-meta2-D",       "/buffer -1");
-    gui_keyboard_default_bind (/* m-left (kde)  */ "meta2-1;3D",         "/buffer -1");
-    gui_keyboard_default_bind (/* m-up          */ "meta-meta2-A",       "/buffer -1");
-    gui_keyboard_default_bind (/* F5            */ "meta2-15~",          "/buffer -1");
-    gui_keyboard_default_bind (/* ^P            */ "ctrl-P",             "/buffer -1");
-    gui_keyboard_default_bind (/* m-right       */ "meta-meta2-C",       "/buffer +1");
-    gui_keyboard_default_bind (/* m-right (kde) */ "meta2-1;3C",         "/buffer +1");
-    gui_keyboard_default_bind (/* m-down        */ "meta-meta2-B",       "/buffer +1");
-    gui_keyboard_default_bind (/* F6            */ "meta2-17~",          "/buffer +1");
-    gui_keyboard_default_bind (/* ^N            */ "ctrl-N",             "/buffer +1");
-    gui_keyboard_default_bind (/* pgup          */ "meta2-5~",           "/window page_up");
-    gui_keyboard_default_bind (/* pgup          */ "meta2-I",            "/window page_up");
-    gui_keyboard_default_bind (/* pgdn          */ "meta2-6~",           "/window page_down");
-    gui_keyboard_default_bind (/* pgdn          */ "meta2-G",            "/window page_down");
-    gui_keyboard_default_bind (/* m-pgup        */ "meta-meta2-5~",      "/window scroll_up");
-    gui_keyboard_default_bind (/* m-pgdn        */ "meta-meta2-6~",      "/window scroll_down");
-    gui_keyboard_default_bind (/* m-home        */ "meta-meta2-1~",      "/window scroll_top");
-    gui_keyboard_default_bind (/* m-home        */ "meta-meta2-7~",      "/window scroll_top");
-    gui_keyboard_default_bind (/* m-end         */ "meta-meta2-4~",      "/window scroll_bottom");
-    gui_keyboard_default_bind (/* m-end         */ "meta-meta2-8~",      "/window scroll_bottom");
-    gui_keyboard_default_bind (/* m-n           */ "meta-n",             "/window scroll_next_highlight");
-    gui_keyboard_default_bind (/* m-p           */ "meta-p",             "/window scroll_previous_highlight");
-    gui_keyboard_default_bind (/* F9            */ "meta2-20~",          "/bar scroll title * x-50%");
-    gui_keyboard_default_bind (/* F10           */ "meta2-21~",          "/bar scroll title * x+50%");
-    gui_keyboard_default_bind (/* F11           */ "meta2-23~",          "/bar scroll nicklist * y-100%");
-    gui_keyboard_default_bind (/* F12           */ "meta2-24~",          "/bar scroll nicklist * y+100%");
-    gui_keyboard_default_bind (/* m-F11         */ "meta-meta2-23~",     "/bar scroll nicklist * yb");
-    gui_keyboard_default_bind (/* m-F12         */ "meta-meta2-24~",     "/bar scroll nicklist * ye");
-    gui_keyboard_default_bind (/* ^L            */ "ctrl-L",             "/window refresh");
-    gui_keyboard_default_bind (/* F7            */ "meta2-18~",          "/window -1");
-    gui_keyboard_default_bind (/* F8            */ "meta2-19~",          "/window +1");
-    gui_keyboard_default_bind (/* m-w,m-up      */ "meta-wmeta-meta2-A", "/window up");
-    gui_keyboard_default_bind (/* m-w,m-down    */ "meta-wmeta-meta2-B", "/window down");
-    gui_keyboard_default_bind (/* m-w,m-left    */ "meta-wmeta-meta2-D", "/window left");
-    gui_keyboard_default_bind (/* m-w,m-right   */ "meta-wmeta-meta2-C", "/window right");
-    gui_keyboard_default_bind (/* m-z           */ "meta-z",             "/window zoom");
-    gui_keyboard_default_bind (/* m-=           */ "meta-=",             "/filter toggle");
-    gui_keyboard_default_bind (/* m-0           */ "meta-0",             "/buffer *10");
-    gui_keyboard_default_bind (/* m-1           */ "meta-1",             "/buffer *1");
-    gui_keyboard_default_bind (/* m-2           */ "meta-2",             "/buffer *2");
-    gui_keyboard_default_bind (/* m-3           */ "meta-3",             "/buffer *3");
-    gui_keyboard_default_bind (/* m-4           */ "meta-4",             "/buffer *4");
-    gui_keyboard_default_bind (/* m-5           */ "meta-5",             "/buffer *5");
-    gui_keyboard_default_bind (/* m-6           */ "meta-6",             "/buffer *6");
-    gui_keyboard_default_bind (/* m-7           */ "meta-7",             "/buffer *7");
-    gui_keyboard_default_bind (/* m-8           */ "meta-8",             "/buffer *8");
-    gui_keyboard_default_bind (/* m-9           */ "meta-9",             "/buffer *9");
-    gui_keyboard_default_bind (/* m-<           */ "meta-<",             "/input jump_previously_visited_buffer");
-    gui_keyboard_default_bind (/* m->           */ "meta->",             "/input jump_next_visited_buffer");
+    BIND(/* RC              */ "ctrl-M",             "/input return");
+    BIND(/* RC              */ "ctrl-J",             "/input return");
+    BIND(/* tab             */ "ctrl-I",             "/input complete_next");
+    BIND(/* s-tab           */ "meta2-Z",            "/input complete_previous");
+    BIND(/* ^R              */ "ctrl-R",             "/input search_text");
+    BIND(/* basckpace       */ "ctrl-H",             "/input delete_previous_char");
+    BIND(/* basckpace       */ "ctrl-?",             "/input delete_previous_char");
+    BIND(/* del             */ "meta2-3~",           "/input delete_next_char");
+    BIND(/* ^D              */ "ctrl-D",             "/input delete_next_char");
+    BIND(/* ^W              */ "ctrl-W",             "/input delete_previous_word");
+    BIND(/* ^X              */ "ctrl-X",             "/input switch_active_buffer");
+    BIND(/* m-d             */ "meta-d",             "/input delete_next_word");
+    BIND(/* ^K              */ "ctrl-K",             "/input delete_end_of_line");
+    BIND(/* m-r             */ "meta-r",             "/input delete_line");
+    BIND(/* ^T              */ "ctrl-T",             "/input transpose_chars");
+    BIND(/* ^U              */ "ctrl-U",             "/input delete_beginning_of_line");
+    BIND(/* ^Y              */ "ctrl-Y",             "/input clipboard_paste");
+    BIND(/* home            */ "meta2-1~",           "/input move_beginning_of_line");
+    BIND(/* home            */ "meta2-H",            "/input move_beginning_of_line");
+    BIND(/* home            */ "meta2-7~",           "/input move_beginning_of_line");
+    BIND(/* ^A              */ "ctrl-A",             "/input move_beginning_of_line");
+    BIND(/* end             */ "meta2-4~",           "/input move_end_of_line");
+    BIND(/* end             */ "meta2-F",            "/input move_end_of_line");
+    BIND(/* end             */ "meta2-8~",           "/input move_end_of_line");
+    BIND(/* ^E              */ "ctrl-E",             "/input move_end_of_line");
+    BIND(/* left            */ "meta2-D",            "/input move_previous_char");
+    BIND(/* ^B              */ "ctrl-B",             "/input move_previous_char");
+    BIND(/* right           */ "meta2-C",            "/input move_next_char");
+    BIND(/* ^F              */ "ctrl-F",             "/input move_next_char");
+    BIND(/* m-b             */ "meta-b",             "/input move_previous_word");
+    BIND(/* ^left           */ "meta-Od",            "/input move_previous_word");
+    BIND(/* m-f             */ "meta-f",             "/input move_next_word");
+    BIND(/* ^right          */ "meta-Oc",            "/input move_next_word");
+    BIND(/* up              */ "meta2-A",            "/input history_previous");
+    BIND(/* down            */ "meta2-B",            "/input history_next");
+    BIND(/* ^up             */ "meta-Oa",            "/input history_global_previous");
+    BIND(/* ^down           */ "meta-Ob",            "/input history_global_next");
+    BIND(/* m-a             */ "meta-a",             "/input jump_smart");
+    BIND(/* m-j,m-l         */ "meta-jmeta-l",       "/input jump_last_buffer");
+    BIND(/* m-j,m-r         */ "meta-jmeta-r",       "/server raw");
+    BIND(/* m-j,m-s         */ "meta-jmeta-s",       "/server jump");
+    BIND(/* m-h             */ "meta-h",             "/input hotlist_clear");
+    BIND(/* m-k             */ "meta-k",             "/input grab_key_command");
+    BIND(/* m-u             */ "meta-u",             "/input scroll_unread");
+    BIND(/* ^S^U            */ "ctrl-Sctrl-U",       "/input set_unread");
+    BIND(/* ^Cb             */ "ctrl-Cb",            "/input insert \\x02");
+    BIND(/* ^Cc             */ "ctrl-Cc",            "/input insert \\x03");
+    BIND(/* ^Ci             */ "ctrl-Ci",            "/input insert \\x1D");
+    BIND(/* ^Co             */ "ctrl-Co",            "/input insert \\x0F");
+    BIND(/* ^Cr             */ "ctrl-Cr",            "/input insert \\x12");
+    BIND(/* ^Cu             */ "ctrl-Cu",            "/input insert \\x15");
+    BIND(/* m-right         */ "meta-meta2-C",       "/buffer +1");
+    BIND(/* m-right (K)     */ "meta2-1;3C",         "/buffer +1");
+    BIND(/* m-down          */ "meta-meta2-B",       "/buffer +1");
+    BIND(/* m-down (K)      */ "meta2-1;3B",         "/buffer +1");
+    BIND(/* F6              */ "meta2-17~",          "/buffer +1");
+    BIND(/* ^N              */ "ctrl-N",             "/buffer +1");
+    BIND(/* m-left          */ "meta-meta2-D",       "/buffer -1");
+    BIND(/* m-left (K)      */ "meta2-1;3D",         "/buffer -1");
+    BIND(/* m-up            */ "meta-meta2-A",       "/buffer -1");
+    BIND(/* m-up (K)        */ "meta2-1;3A",         "/buffer -1");
+    BIND(/* F5              */ "meta2-15~",          "/buffer -1");
+    BIND(/* ^P              */ "ctrl-P",             "/buffer -1");
+    BIND(/* pgup            */ "meta2-5~",           "/window page_up");
+    BIND(/* pgup            */ "meta2-I",            "/window page_up");
+    BIND(/* pgdn            */ "meta2-6~",           "/window page_down");
+    BIND(/* pgdn            */ "meta2-G",            "/window page_down");
+    BIND(/* m-pgup          */ "meta-meta2-5~",      "/window scroll_up");
+    BIND(/* m-pgdn          */ "meta-meta2-6~",      "/window scroll_down");
+    BIND(/* m-home          */ "meta-meta2-1~",      "/window scroll_top");
+    BIND(/* m-home          */ "meta-meta2-7~",      "/window scroll_top");
+    BIND(/* m-end           */ "meta-meta2-4~",      "/window scroll_bottom");
+    BIND(/* m-end           */ "meta-meta2-8~",      "/window scroll_bottom");
+    BIND(/* m-n             */ "meta-n",             "/window scroll_next_highlight");
+    BIND(/* m-p             */ "meta-p",             "/window scroll_previous_highlight");
+    BIND(/* F9              */ "meta2-20~",          "/bar scroll title * x-50%");
+    BIND(/* F10             */ "meta2-21~",          "/bar scroll title * x+50%");
+    BIND(/* F11             */ "meta2-23~",          "/bar scroll nicklist * y-100%");
+    BIND(/* F12             */ "meta2-24~",          "/bar scroll nicklist * y+100%");
+    BIND(/* m-F11           */ "meta-meta2-23~",     "/bar scroll nicklist * yb");
+    BIND(/* m-F12           */ "meta-meta2-24~",     "/bar scroll nicklist * ye");
+    BIND(/* ^L              */ "ctrl-L",             "/window refresh");
+    BIND(/* F7              */ "meta2-18~",          "/window -1");
+    BIND(/* F8              */ "meta2-19~",          "/window +1");
+    BIND(/* m-w,m-up        */ "meta-wmeta-meta2-A", "/window up");
+    BIND(/* m-w,m-up (K)    */ "meta-wmeta2-1;3A",   "/window up");
+    BIND(/* m-w,m-down      */ "meta-wmeta-meta2-B", "/window down");
+    BIND(/* m-w,m-down (K)  */ "meta-wmeta2-1;3B",   "/window down");
+    BIND(/* m-w,m-right     */ "meta-wmeta-meta2-C", "/window right");
+    BIND(/* m-w,m-right (K) */ "meta-wmeta2-1;3C",   "/window right");
+    BIND(/* m-w,m-left      */ "meta-wmeta-meta2-D", "/window left");
+    BIND(/* m-w,m-left (K)  */ "meta-wmeta2-1;3D",   "/window left");
+    BIND(/* m-z             */ "meta-z",             "/window zoom");
+    BIND(/* m-=             */ "meta-=",             "/filter toggle");
+    BIND(/* m-0             */ "meta-0",             "/buffer *10");
+    BIND(/* m-1             */ "meta-1",             "/buffer *1");
+    BIND(/* m-2             */ "meta-2",             "/buffer *2");
+    BIND(/* m-3             */ "meta-3",             "/buffer *3");
+    BIND(/* m-4             */ "meta-4",             "/buffer *4");
+    BIND(/* m-5             */ "meta-5",             "/buffer *5");
+    BIND(/* m-6             */ "meta-6",             "/buffer *6");
+    BIND(/* m-7             */ "meta-7",             "/buffer *7");
+    BIND(/* m-8             */ "meta-8",             "/buffer *8");
+    BIND(/* m-9             */ "meta-9",             "/buffer *9");
+    BIND(/* m-<             */ "meta-<",             "/input jump_previously_visited_buffer");
+    BIND(/* m->             */ "meta->",             "/input jump_next_visited_buffer");
     
     /* bind meta-j + {01..99} to switch to buffers # > 10 */
     for (i = 1; i < 100; i++)
     {
         sprintf (key_str, "meta-j%02d", i);
         sprintf (command, "/buffer %d", i);
-        gui_keyboard_default_bind (key_str, command);
+        BIND(key_str, command);
     }
 }
 
@@ -196,8 +207,10 @@ gui_keyboard_flush ()
     int i, key, insert_ok;
     char key_str[32], *key_utf, *input_old;
     
-    /* if there's no paste pending, then we use buffer and do actions
-       according to keys */
+    /*
+     * if there's no paste pending, then we use buffer and do actions
+     * according to keys
+     */
     if (!gui_keyboard_paste_pending)
     {
         if (gui_keyboard_buffer_size > 0)
@@ -426,8 +439,10 @@ gui_keyboard_read_cb (void *data, int fd)
     }
     else
     {
-        /* detect user paste or large amount of text
-           if so, ask user what to do */
+        /*
+         * detect user paste or large amount of text
+         * if so, ask user what to do
+         */
         if (CONFIG_INTEGER(config_look_paste_max_lines) > 0)
         {
             paste_lines = gui_keyboard_get_paste_lines ();
