@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-curses-main.c: main loop for Curses GUI */
+/*
+ * gui-curses-main.c: main loop for Curses GUI
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -72,8 +74,8 @@ gui_main_pre_init (int *argc, char **argv[])
     /* pre-init colors */
     gui_color_pre_init ();
     
-    /* build empty prefixes (before reading config) */
-    gui_chat_prefix_build_empty ();
+    /* init some variables for chat area */
+    gui_chat_init ();
 }
 
 /*
@@ -138,8 +140,10 @@ gui_main_init ()
                 gui_window_set_title (PACKAGE_NAME " " PACKAGE_VERSION);
         }
         
-        /* create bar windows for root bars (they were read from config,
-           but no window was created (GUI was not initialized) */
+        /*
+         * create bar windows for root bars (they were read from config,
+         * but no window was created (GUI was not initialized)
+         */
         for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
         {
             if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_ROOT)
@@ -190,11 +194,12 @@ gui_main_signal_sigterm ()
 void
 gui_main_signal_sighup ()
 {
-    /* SIGHUP signal is received when terminal is closed (exit of WeeChat
-       without using /quit command), that's why we set only flag to reload
-       config files later (when terminal is closed, config files are NOT
-       reloaded, but they are if signal SIGHUP is sent to WeeChat by user)
-    */
+    /*
+     * SIGHUP signal is received when terminal is closed (exit of WeeChat
+     * without using /quit command), that's why we set only flag to reload
+     * config files later (when terminal is closed, config files are NOT
+     * reloaded, but they are if signal SIGHUP is sent to WeeChat by user)
+     */
     gui_reload_config = 1;
 }
 
@@ -346,6 +351,17 @@ gui_main_end (int clean_exit)
 {
     if (clean_exit)
     {
+        /*
+         * final refreshs, to see messages just before exiting
+         * (if we are upgrading, don't refresh anything!)
+         */
+        if (!weechat_upgrading)
+        {
+            gui_main_refreshs ();
+            if (gui_window_refresh_needed)
+                gui_main_refreshs ();
+        }
+        
         /* remove bar items and bars */
         gui_bar_item_end ();
         gui_bar_free_all ();
@@ -387,8 +403,8 @@ gui_main_end (int clean_exit)
         /* end color */
         gui_color_end ();
         
-        /* free chat buffer */
-        gui_chat_free_buffer ();
+        /* free some variables used for chat area */
+        gui_chat_end ();
     }
     
     /* end of Curses output */
