@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-curses-window.c: window display functions for Curses GUI */
+/*
+ * gui-curses-window.c: window display functions for Curses GUI
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +36,7 @@
 #include "../../core/wee-hook.h"
 #include "../../core/wee-log.h"
 #include "../../core/wee-string.h"
+#include "../../core/wee-utf8.h"
 #include "../../plugins/plugin.h"
 #include "../gui-window.h"
 #include "../gui-bar.h"
@@ -159,6 +162,26 @@ gui_window_utf_char_valid (const char *utf_char)
     return 1;
 }
 
+/*
+ * gui_window_get_hline_char: get char used to draw horizontal lines
+ *                            Note: ACS_HLINE from ncurses is better for
+ *                                  render, but it introduces bug with URLs
+ *                                  selected by terminal: below this line,
+ *                                  some URLs are not visible or shifted
+ */
+
+int
+gui_window_get_hline_char ()
+{
+    const char *hline_char;
+
+    hline_char = CONFIG_STRING(config_look_hline_char);
+    if (!hline_char || !hline_char[0])
+        return ACS_HLINE;
+    
+    return utf8_char_int (hline_char);
+}
+
 /*
  * gui_window_wprintw: decode then display string with wprintw
  */
@@ -209,10 +232,10 @@ gui_window_clear (WINDOW *window, int bg)
 
     if ((bg >= 0) && (bg < GUI_CURSES_NUM_WEECHAT_COLORS))
     {
-        color = gui_weechat_colors[bg].foreground;
+        color = gui_weechat_colors[bg].background;
         wbkgdset (window,
                   ' ' | COLOR_PAIR (((color == -1) || (color == 99)) ?
-                                    63 : color * 8));
+                                    gui_color_last_pair : (color * gui_color_num_bg) + 1));
         werase (window);
         wmove (window, 0, 0);
     }
@@ -269,14 +292,14 @@ gui_window_set_color (WINDOW *window, int fg, int bg)
     
     if (((fg == -1) || (fg == 99))
         && ((bg == -1) || (bg == 99)))
-        wattron (window, COLOR_PAIR(63));
+        wattron (window, COLOR_PAIR(gui_color_last_pair));
     else
     {
         if ((fg == -1) || (fg == 99))
             fg = COLOR_WHITE;
         if ((bg == -1) || (bg == 99))
             bg = 0;
-        wattron (window, COLOR_PAIR((bg * 8) + fg));
+        wattron (window, COLOR_PAIR((bg * gui_color_num_bg) + fg + 1));
     }
 }
 
@@ -289,11 +312,6 @@ gui_window_set_weechat_color (WINDOW *window, int num_color)
 {
     if ((num_color >= 0) && (num_color < GUI_COLOR_NUM_COLORS))
     {
-        /*
-        wattroff (window, A_BOLD | A_UNDERLINE | A_REVERSE);
-        wattron (window, COLOR_PAIR(gui_color_get_pair (num_color)) |
-                 gui_color[num_color]->attributes);
-        */
         gui_window_reset_style (window, num_color);
         wattron (window, gui_color[num_color]->attributes);
         gui_window_set_color (window,
@@ -317,7 +335,8 @@ gui_window_set_custom_color_fg_bg (WINDOW *window, int fg, int bg)
         wattron (window, gui_weechat_colors[fg].attributes);
         gui_window_set_color (window,
                               gui_weechat_colors[fg].foreground,
-                              gui_weechat_colors[bg].foreground);
+                              (gui_color_num_bg > 8) ?
+                              gui_weechat_colors[bg].background : gui_weechat_colors[bg].foreground);
     }
 }
 
@@ -359,7 +378,8 @@ gui_window_set_custom_color_bg (WINDOW *window, int bg)
         current_fg = window_current_style_fg;
         gui_window_set_color_style (window, current_attr);
         gui_window_set_color (window, current_fg,
-                              gui_weechat_colors[bg].foreground);
+                              (gui_color_num_bg > 8) ?
+                              gui_weechat_colors[bg].background : gui_weechat_colors[bg].foreground);
     }
 }
 
@@ -371,7 +391,8 @@ void
 gui_window_clrtoeol_with_current_bg (WINDOW *window)
 {
     wbkgdset (window,
-              ' ' | COLOR_PAIR ((window_current_style_bg < 0) ? 63 : window_current_style_bg * 8));
+              ' ' | COLOR_PAIR ((window_current_style_bg < 0) ?
+                                gui_color_last_pair : (window_current_style_bg * gui_color_num_bg) + 1));
     wclrtoeol (window);
 }
 
@@ -567,6 +588,12 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
         ptr_bar_window->bar->bar_refresh_needed = 1;
     }
     
+    if (CONFIG_BOOLEAN(config_look_read_marker_always_show)
+        && set_last_read && !window->buffer->lines->last_read_line)
+    {
+        window->buffer->lines->last_read_line = window->buffer->lines->last_line;
+    }
+    
     hook_signal_send ("buffer_switch",
                       WEECHAT_HOOK_SIGNAL_POINTER, buffer);
 }
@@ -746,7 +773,7 @@ gui_window_scroll_up (struct t_gui_window *window)
             break;
         case GUI_BUFFER_NUM_TYPES:
             break;
-    }   
+    }
 }
 
 /*
@@ -1465,6 +1492,21 @@ gui_window_set_title (const char *title)
     }
 }
 
+/*
+ * gui_window_term_display_infos: display some infos about terminal and colors
+ */
+
+void
+gui_window_term_display_infos ()
+{
+    gui_chat_printf (NULL, "");
+    gui_chat_printf (NULL, _("Terminal infos:"));
+    gui_chat_printf (NULL, _("  TERM='%s', size: %dx%d"),
+                     getenv("TERM"), gui_term_cols, gui_term_lines);
+    gui_chat_printf (NULL, _("  %d colors available, %d pairs"),
+                     COLORS, COLOR_PAIRS);
+}
+
 /*
  * gui_window_objects_print_log: print window Curses objects infos in log
  *                               (usually for crash dump)
@@ -1474,6 +1516,6 @@ void
 gui_window_objects_print_log (struct t_gui_window *window)
 {
     log_printf ("  window specific objects for Curses:");
-    log_printf ("    win_chat. . . . . . : 0x%lx", GUI_WINDOW_OBJECTS(window)->win_chat);
-    log_printf ("    win_separator . . . : 0x%lx", GUI_WINDOW_OBJECTS(window)->win_separator);
+    log_printf ("    win_chat. . . . . : 0x%lx", GUI_WINDOW_OBJECTS(window)->win_chat);
+    log_printf ("    win_separator . . : 0x%lx", GUI_WINDOW_OBJECTS(window)->win_separator);
 }
