@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-gtk-main.c: main loop for Gtk GUI */
+/*
+ * gui-gtk-main.c: main loop for Gtk GUI
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -70,8 +72,8 @@ gui_main_pre_init (int *argc, char **argv[])
     /* pre-init colors */
     gui_color_pre_init ();
     
-    /* build empty prefixes (before reading config) */
-    gui_chat_prefix_build_empty ();
+    /* init some variables for chat area */
+    gui_chat_init ();
     
     /* Initialise Gtk */
     gtk_init (argc, argv);
@@ -195,8 +197,10 @@ gui_main_init ()
                 gui_window_set_title (PACKAGE_NAME " " PACKAGE_VERSION);
         }
         
-        /* create bar windows for root bars (they were read from config,
-           but no window was created (GUI was not initialized) */
+        /*
+         * create bar windows for root bars (they were read from config,
+         * but no window was created (GUI was not initialized)
+         */
         for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
         {
             if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_ROOT)
@@ -275,7 +279,7 @@ gui_main_end (int clean_exit)
         /* end color */
         gui_color_end ();
         
-        /* free chat buffer */
-        gui_chat_free_buffer ();
+        /* free some variables used for chat area */
+        gui_chat_end ();
     }
 }
