@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-gtk-window.c: window display functions for Gtk GUI */
+/*
+ * gui-gtk-window.c: window display functions for Gtk GUI
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -490,7 +492,7 @@ gui_window_split_horizontal (struct t_gui_window *window, int percentage)
     
     if ((percentage > 0) && (percentage <= 100))
     {
-        new_window = gui_window_new (window, window->buffer, 
+        new_window = gui_window_new (window, window->buffer,
                                      window->win_x, window->win_y,
                                      window->win_width, height1,
                                      100, percentage);
@@ -801,6 +803,16 @@ gui_window_set_title (const char *title)
     /* TODO: write this function for Gtk */
 }
 
+/*
+ * gui_window_term_display_infos: display some infos about terminal and colors
+ */
+
+void
+gui_window_term_display_infos ()
+{
+    /* No term info for Gtk */
+}
+
 /*
  * gui_window_objects_print_log: print Gtk objects infos in log
  *                               (usually for crash dump)
