@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,8 +41,10 @@ struct t_gui_line;
 #define COLOR_YELLOW  6
 #define COLOR_WHITE   7
 
-/* shift ncurses colors for compatibility with colors
-   in IRC messages (same as other IRC clients) */
+/*
+ * shift ncurses colors for compatibility with colors
+ * in IRC messages (same as other IRC clients)
+ */
 
 #define WEECHAT_COLOR_BLACK   COLOR_BLACK
 #define WEECHAT_COLOR_RED     COLOR_BLUE
