@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-bar-item.c: bar item functions, used by all GUI */
+/*
+ * gui-bar-item.c: bar item functions (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -277,6 +279,7 @@ gui_bar_item_get_value (const char *name, struct t_gui_bar *bar,
     char *result;
     int valid_char, length;
     struct t_gui_bar_item *ptr_item;
+    struct t_weechat_plugin *ptr_plugin;
     
     start = NULL;
     end = NULL;
@@ -312,10 +315,12 @@ gui_bar_item_get_value (const char *name, struct t_gui_bar *bar,
     item_value = NULL;
     if (item_name)
     {
-        ptr_item = gui_bar_item_search_with_plugin ((window && window->buffer) ?
-                                                    window->buffer->plugin : NULL,
-                                                    0,
-                                                    item_name);
+        ptr_plugin = NULL;
+        if (window && window->buffer)
+            ptr_plugin = window->buffer->plugin;
+        else if (gui_current_window && gui_current_window->buffer)
+            ptr_plugin = gui_current_window->buffer->plugin;
+        ptr_item = gui_bar_item_search_with_plugin (ptr_plugin, 0, item_name);
         if (ptr_item && ptr_item->build_callback)
         {
             item_value = (ptr_item->build_callback) (ptr_item->build_callback_data,
@@ -580,12 +585,20 @@ char *
 gui_bar_item_default_input_prompt (void *data, struct t_gui_bar_item *item,
                                    struct t_gui_window *window)
 {
+    struct t_gui_buffer_local_var *local_var_nick;
+    
     /* make C compiler happy */
     (void) data;
     (void) item;
-    (void) window;
     
-    return NULL;
+    if (!window)
+        window = gui_current_window;
+    
+    local_var_nick = gui_buffer_local_var_search (window->buffer, "nick");
+    if (!local_var_nick || !local_var_nick->value)
+        return NULL;
+    
+    return strdup (local_var_nick->value);
 }
 
 /*
@@ -755,7 +768,7 @@ gui_bar_item_default_time (void *data, struct t_gui_bar_item *item,
 {
     time_t date;
     struct tm *local_time;
-    char text_time[128];
+    char text_time[128], text_time2[128];
     
     /* make C compiler happy */
     (void) data;
@@ -769,7 +782,11 @@ gui_bar_item_default_time (void *data, struct t_gui_bar_item *item,
                   local_time) == 0)
         return NULL;
     
-    return strdup (text_time);
+    snprintf (text_time2, sizeof (text_time2), "%s%s",
+              gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_time))),
+              text_time);
+    
+    return strdup (text_time2);
 }
 
 /*
@@ -1172,31 +1189,37 @@ gui_bar_item_default_buffer_nicklist (void *data, struct t_gui_bar_item *item,
                             strcat (buf, " ");
                         }
                     }
-                    if (strchr (ptr_nick->prefix_color, '.'))
+                    if (ptr_nick->prefix_color)
                     {
-                        config_file_search_with_string (ptr_nick->prefix_color,
-                                                        NULL, NULL, &ptr_option,
-                                                        NULL);
-                        if (ptr_option)
-                            strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(ptr_option))));
-                    }
-                    else
-                    {
-                        strcat (buf, gui_color_get_custom (ptr_nick->prefix_color));
+                        if (strchr (ptr_nick->prefix_color, '.'))
+                        {
+                            config_file_search_with_string (ptr_nick->prefix_color,
+                                                            NULL, NULL, &ptr_option,
+                                                            NULL);
+                            if (ptr_option)
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(ptr_option))));
+                        }
+                        else
+                        {
+                            strcat (buf, gui_color_get_custom (ptr_nick->prefix_color));
+                        }
                     }
                     if (ptr_nick->prefix)
                         strcat (buf, ptr_nick->prefix);
-                    if (strchr (ptr_nick->color, '.'))
+                    if (ptr_nick->color)
                     {
-                        config_file_search_with_string (ptr_nick->color,
-                                                        NULL, NULL, &ptr_option,
-                                                        NULL);
-                        if (ptr_option)
-                            strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(ptr_option))));
-                    }
-                    else
-                    {
-                        strcat (buf, gui_color_get_custom (ptr_nick->color));
+                        if (strchr (ptr_nick->color, '.'))
+                        {
+                            config_file_search_with_string (ptr_nick->color,
+                                                            NULL, NULL, &ptr_option,
+                                                            NULL);
+                            if (ptr_option)
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(ptr_option))));
+                        }
+                        else
+                        {
+                            strcat (buf, gui_color_get_custom (ptr_nick->color));
+                        }
                     }
                     strcat (buf, ptr_nick->name);
                 }
@@ -1206,17 +1229,20 @@ gui_bar_item_default_buffer_nicklist (void *data, struct t_gui_bar_item *item,
                     {
                         strcat (buf, " ");
                     }
-                    if (strchr (ptr_group->color, '.'))
+                    if (ptr_group->color)
                     {
-                        config_file_search_with_string (ptr_group->color,
-                                                        NULL, NULL, &ptr_option,
-                                                        NULL);
-                        if (ptr_option)
-                            strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(ptr_option))));
-                    }
-                    else
-                    {
-                        strcat (buf, gui_color_get_custom (ptr_group->color));
+                        if (strchr (ptr_group->color, '.'))
+                        {
+                            config_file_search_with_string (ptr_group->color,
+                                                            NULL, NULL, &ptr_option,
+                                                            NULL);
+                            if (ptr_option)
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(ptr_option))));
+                        }
+                        else
+                        {
+                            strcat (buf, gui_color_get_custom (ptr_group->color));
+                        }
                     }
                     strcat (buf, gui_nicklist_get_group_start (ptr_group->name));
                 }
@@ -1250,9 +1276,11 @@ gui_bar_item_timer_cb (void *data, int remaining_calls)
                   local_time) == 0)
         return WEECHAT_RC_OK;
     
-    /* we update item only if it changed since last time
-       for example if time is only hours:minutes, we'll update
-       only when minute changed */
+    /*
+     * we update item only if it changed since last time
+     * for example if time is only hours:minutes, we'll update
+     * only when minute changed
+     */
     if (strcmp (new_item_time_text, item_time_text) != 0)
     {
         snprintf (item_time_text, sizeof (item_time_text),
@@ -1320,7 +1348,7 @@ gui_bar_item_init ()
     gui_bar_item_new (NULL,
                       gui_bar_item_names[GUI_BAR_ITEM_INPUT_PROMPT],
                       &gui_bar_item_default_input_prompt, NULL);
-    gui_bar_item_hook_signal ("input_prompt_changed",
+    gui_bar_item_hook_signal ("buffer_localvar_*",
                               gui_bar_item_names[GUI_BAR_ITEM_INPUT_PROMPT]);
     
     /* input search */
@@ -1376,6 +1404,8 @@ gui_bar_item_init ()
                               gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NUMBER]);
     gui_bar_item_hook_signal ("buffer_unmerged",
                               gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NUMBER]);
+    gui_bar_item_hook_signal ("buffer_closed",
+                              gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NUMBER]);
     
     /* buffer name */
     gui_bar_item_new (NULL,
@@ -1403,7 +1433,7 @@ gui_bar_item_init ()
                       &gui_bar_item_default_buffer_nicklist_count, NULL);
     gui_bar_item_hook_signal ("buffer_switch",
                               gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NICKLIST_COUNT]);
-    gui_bar_item_hook_signal ("nicklist_changed",
+    gui_bar_item_hook_signal ("nicklist_*",
                               gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NICKLIST_COUNT]);
     
     /* scroll indicator */
@@ -1421,6 +1451,8 @@ gui_bar_item_init ()
                               gui_bar_item_names[GUI_BAR_ITEM_HOTLIST]);
     gui_bar_item_hook_signal ("buffer_moved",
                               gui_bar_item_names[GUI_BAR_ITEM_HOTLIST]);
+    gui_bar_item_hook_signal ("buffer_closed",
+                              gui_bar_item_names[GUI_BAR_ITEM_HOTLIST]);
     
     /* completion (possible words when a partial completion occurs) */
     gui_bar_item_new (NULL,
@@ -1440,7 +1472,9 @@ gui_bar_item_init ()
     gui_bar_item_new (NULL,
                       gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NICKLIST],
                       &gui_bar_item_default_buffer_nicklist, NULL);
-    gui_bar_item_hook_signal ("nicklist_changed",
+    gui_bar_item_hook_signal ("nicklist_*",
+                              gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NICKLIST]);
+    gui_bar_item_hook_signal ("buffer_switch",
                               gui_bar_item_names[GUI_BAR_ITEM_BUFFER_NICKLIST]);
 }
 
