@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-bar-window.c: bar window functions, used by all GUI */
+/*
+ * gui-bar-window.c: bar window functions (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -1134,7 +1136,7 @@ gui_bar_window_print_log (struct t_gui_bar_window *bar_window)
     
     log_printf ("");
     log_printf ("  [window bar (addr:0x%lx)]",     bar_window);
-    log_printf ("    bar. . . . . . . . . : 0x%lx ('%s')",
+    log_printf ("    bar. . . . . . . . . . : 0x%lx ('%s')",
                 bar_window->bar,
                 (bar_window->bar) ? bar_window->bar->name : "");
     log_printf ("    x. . . . . . . . . . . : %d",    bar_window->x);
