@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-buffer.c: buffer functions, used by all GUI */
+/*
+ * gui-buffer.c: buffer functions (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +38,7 @@
 #include "../core/wee-config.h"
 #include "../core/wee-hook.h"
 #include "../core/wee-infolist.h"
+#include "../core/wee-list.h"
 #include "../core/wee-log.h"
 #include "../core/wee-string.h"
 #include "../core/wee-utf8.h"
@@ -68,6 +71,34 @@ int gui_buffers_visited_frozen = 0;             /* 1 to forbid list updates */
 char *gui_buffer_notify_string[GUI_BUFFER_NUM_NOTIFY] =
 { "none", "highlight", "message", "all" };
 
+char *gui_buffer_properties_get_integer[] =
+{ "number", "layout_number", "type", "notify", "num_displayed", "active",
+  "print_hooks_enabled", "lines_hidden", "prefix_max_length",
+  "time_for_each_line", "nicklist", "nicklist_case_sensitive",
+  "nicklist_max_length", "nicklist_display_groups", "nicklist_visible_count",
+  "input", "input_get_unknown_commands", "input_size", "input_length",
+  "input_pos", "input_1st_display", "num_history", "text_search",
+  "text_search_exact", "text_search_found",
+  NULL
+};
+char *gui_buffer_properties_get_string[] =
+{ "plugin", "name", "short_name", "title", "input", "text_search_input",
+  "highlight_words", "highlight_tags",
+  NULL
+};
+char *gui_buffer_properties_get_pointer[] =
+{ "plugin",
+  NULL
+};
+char *gui_buffer_properties_set[] =
+{ "unread", "display", "print_hooks_enabled", "number", "name", "short_name",
+  "type", "notify", "title", "time_for_each_line", "nicklist",
+  "nicklist_case_sensitive", "nicklist_display_groups", "highlight_words",
+  "highlight_words_add", "highlight_words_del", "highlight_tags", "input",
+  "input_pos", "input_get_unknown_commands",
+  NULL
+};
+
 
 /*
  * gui_buffer_find_pos: find position for buffer in list
@@ -401,6 +432,7 @@ gui_buffer_new (struct t_weechat_plugin *plugin,
         new_buffer->layout_number = gui_layout_buffer_get_number (gui_layout_buffers,
                                                                   plugin_get_name (plugin),
                                                                   name);
+        new_buffer->layout_applied = 0;
         new_buffer->name = strdup (name);
         new_buffer->short_name = strdup (name);
         new_buffer->type = GUI_BUFFER_TYPE_FORMATTED;
@@ -474,6 +506,7 @@ gui_buffer_new (struct t_weechat_plugin *plugin,
         /* keys */
         new_buffer->keys = NULL;
         new_buffer->last_key = NULL;
+        new_buffer->keys_count = 0;
         
         /* local variables */
         new_buffer->local_variables = NULL;
@@ -488,8 +521,10 @@ gui_buffer_new (struct t_weechat_plugin *plugin,
         /* set notify level */
         new_buffer->notify = gui_buffer_notify_get (new_buffer);
         
-        /* check if this buffer should be assigned to a window,
-           according to windows layout saved */
+        /*
+         * check if this buffer should be assigned to a window,
+         * according to windows layout saved
+         */
         gui_layout_window_check_buffer (new_buffer);
         
         if (first_buffer_creation)
@@ -548,6 +583,9 @@ gui_buffer_string_replace_local_var (struct t_gui_buffer *buffer,
     const char *pos_end_name;
     struct t_gui_buffer_local_var *ptr_local_var;
     
+    if (!string)
+        return NULL;
+    
     length = strlen (string) + 1;
     result = malloc (length);
     if (result)
@@ -635,6 +673,29 @@ gui_buffer_set_plugin_for_upgrade (char *name, struct t_weechat_plugin *plugin)
     }
 }
 
+/*
+ * gui_buffer_property_in_list: return 1 if buffer property name is in a list
+ *                                     0 if property is not in list
+ */
+
+int
+gui_buffer_property_in_list (char *properties[], char *property)
+{
+    int i;
+    
+    if (!properties || !property)
+        return 0;
+    
+    for (i = 0; properties[i]; i++)
+    {
+        if (strcmp (properties[i], property) == 0)
+            return 1;
+    }
+    
+    /* property not found in list */
+    return 0;
+}
+
 /*
  * gui_buffer_get_integer: get a buffer property as integer
  */
@@ -646,18 +707,48 @@ gui_buffer_get_integer (struct t_gui_buffer *buffer, const char *property)
     {
         if (string_strcasecmp (property, "number") == 0)
             return buffer->number;
-        else if (string_strcasecmp (property, "num_displayed") == 0)
-            return buffer->num_displayed;
-        else if (string_strcasecmp (property, "print_hooks_enabled") == 0)
-            return buffer->print_hooks_enabled;
+        else if (string_strcasecmp (property, "layout_number") == 0)
+            return buffer->layout_number;
+        else if (string_strcasecmp (property, "type") == 0)
+            return buffer->type;
         else if (string_strcasecmp (property, "notify") == 0)
             return buffer->notify;
+        else if (string_strcasecmp (property, "num_displayed") == 0)
+            return buffer->num_displayed;
+        else if (string_strcasecmp (property, "active") == 0)
+            return buffer->active;
+        else if (string_strcasecmp (property, "print_hooks_enabled") == 0)
+            return buffer->print_hooks_enabled;
         else if (string_strcasecmp (property, "lines_hidden") == 0)
             return buffer->lines->lines_hidden;
         else if (string_strcasecmp (property, "prefix_max_length") == 0)
             return buffer->lines->prefix_max_length;
         else if (string_strcasecmp (property, "time_for_each_line") == 0)
             return buffer->time_for_each_line;
+        else if (string_strcasecmp (property, "nicklist") == 0)
+            return buffer->nicklist;
+        else if (string_strcasecmp (property, "nicklist_case_sensitive") == 0)
+            return buffer->nicklist_case_sensitive;
+        else if (string_strcasecmp (property, "nicklist_max_length") == 0)
+            return buffer->nicklist_max_length;
+        else if (string_strcasecmp (property, "nicklist_display_groups") == 0)
+            return buffer->nicklist_display_groups;
+        else if (string_strcasecmp (property, "nicklist_visible_count") == 0)
+            return buffer->nicklist_visible_count;
+        else if (string_strcasecmp (property, "input") == 0)
+            return buffer->input;
+        else if (string_strcasecmp (property, "input_get_unknown_commands") == 0)
+            return buffer->input_get_unknown_commands;
+        else if (string_strcasecmp (property, "input_size") == 0)
+            return buffer->input_buffer_size;
+        else if (string_strcasecmp (property, "input_length") == 0)
+            return buffer->input_buffer_length;
+        else if (string_strcasecmp (property, "input_pos") == 0)
+            return buffer->input_buffer_pos;
+        else if (string_strcasecmp (property, "input_1st_display") == 0)
+            return buffer->input_buffer_1st_display;
+        else if (string_strcasecmp (property, "num_history") == 0)
+            return buffer->num_history;
         else if (string_strcasecmp (property, "text_search") == 0)
             return buffer->text_search;
         else if (string_strcasecmp (property, "text_search_exact") == 0)
@@ -690,6 +781,12 @@ gui_buffer_get_string (struct t_gui_buffer *buffer, const char *property)
             return buffer->title;
         else if (string_strcasecmp (property, "input") == 0)
             return buffer->input_buffer;
+        else if (string_strcasecmp (property, "text_search_input") == 0)
+            return buffer->text_search_input;
+        else if (string_strcasecmp (property, "highlight_words") == 0)
+            return buffer->highlight_words;
+        else if (string_strcasecmp (property, "highlight_tags") == 0)
+            return buffer->highlight_tags;
         else if (string_strncasecmp (property, "localvar_", 9) == 0)
         {
             ptr_local_var = gui_buffer_local_var_search (buffer, property + 9);
@@ -861,6 +958,148 @@ gui_buffer_set_highlight_words (struct t_gui_buffer *buffer,
         strdup (new_highlight_words) : NULL;
 }
 
+/*
+ * gui_buffer_set_highlight_words_list: set highlight words for a buffer with a
+ *                                      list
+ */
+
+void
+gui_buffer_set_highlight_words_list (struct t_gui_buffer *buffer,
+                                     struct t_weelist *list)
+{
+    struct t_weelist_item *ptr_list_item;
+    int length;
+    const char *ptr_string;
+    char *words;
+
+    /* compute length */
+    length = 0;
+    for (ptr_list_item = weelist_get (list, 0); ptr_list_item;
+         ptr_list_item = weelist_next (ptr_list_item))
+    {
+        ptr_string = weelist_string (ptr_list_item);
+        if (ptr_string)
+            length += strlen (ptr_string) + 1;
+    }
+    length += 2; /* '\n' + '\0' */
+    
+    /* allocate memory */
+    words = malloc (length);
+    if (!words)
+        return;
+    
+    /* build string */
+    words[0] = '\0';
+    for (ptr_list_item = weelist_get (list, 0); ptr_list_item;
+         ptr_list_item = weelist_next (ptr_list_item))
+    {
+        ptr_string = weelist_string (ptr_list_item);
+        if (ptr_string)
+        {
+            strcat (words, ptr_string);
+            if (weelist_next (ptr_list_item))
+                strcat (words, ",");
+        }
+    }
+    
+    gui_buffer_set_highlight_words (buffer, words);
+}
+
+/*
+ * gui_buffer_add_highlight_words: add highlight words for a buffer
+ */
+
+void
+gui_buffer_add_highlight_words (struct t_gui_buffer *buffer,
+                                const char *words_to_add)
+{
+    char **current_words, **add_words;
+    int current_count, add_count, i;
+    struct t_weelist *list;
+    
+    if (!words_to_add)
+        return;
+    
+    list = weelist_new ();
+    if (!list)
+        return;
+    
+    current_words = string_split (buffer->highlight_words, ",", 0, 0,
+                                  &current_count);
+    add_words = string_split (words_to_add, ",", 0, 0,
+                              &add_count);
+    
+    for (i = 0; i < current_count; i++)
+    {
+        if (!weelist_search (list, current_words[i]))
+            weelist_add (list, current_words[i], WEECHAT_LIST_POS_END, NULL);
+    }
+    for (i = 0; i < add_count; i++)
+    {
+        if (!weelist_search (list, add_words[i]))
+            weelist_add (list, add_words[i], WEECHAT_LIST_POS_END, NULL);
+    }
+    
+    gui_buffer_set_highlight_words_list (buffer, list);
+    
+    weelist_free (list);
+    
+    if (current_words)
+        string_free_split (current_words);
+    if (add_words)
+        string_free_split (add_words);
+}
+
+/*
+ * gui_buffer_remove_highlight_words: remove highlight words in a buffer
+ */
+
+void
+gui_buffer_remove_highlight_words (struct t_gui_buffer *buffer,
+                                   const char *words_to_remove)
+{
+    char **current_words, **remove_words;
+    int current_count, remove_count, i, j, to_remove;
+    struct t_weelist *list;
+    
+    if (!words_to_remove)
+        return;
+    
+    list = weelist_new ();
+    if (!list)
+        return;
+    
+    current_words = string_split (buffer->highlight_words, ",", 0, 0,
+                                  &current_count);
+    remove_words = string_split (words_to_remove, ",", 0, 0,
+                                 &remove_count);
+    
+    for (i = 0; i < current_count; i++)
+    {
+        /* search if word is to be removed or not */
+        to_remove = 0;
+        for (j = 0; j < remove_count; j++)
+        {
+            if (strcmp (current_words[i], remove_words[j]) == 0)
+            {
+                to_remove = 1;
+                break;
+            }
+        }
+        if (!to_remove)
+            weelist_add (list, current_words[i], WEECHAT_LIST_POS_END, NULL);
+    }
+    
+    gui_buffer_set_highlight_words_list (buffer, list);
+    
+    weelist_free (list);
+    
+    if (current_words)
+        string_free_split (current_words);
+    if (remove_words)
+        string_free_split (remove_words);
+}
+
 /*
  * gui_buffer_set_highlight_tags: set highlight tags for a buffer
  */
@@ -967,9 +1206,11 @@ gui_buffer_set (struct t_gui_buffer *buffer, const char *property,
     }
     else if (string_strcasecmp (property, "display") == 0)
     {
-        /* if it is auto-switch to a buffer, then we don't set read marker,
-           otherwise we reset it (if current buffer is not displayed) after
-           switch */
+        /*
+         * if it is auto-switch to a buffer, then we don't set read marker,
+         * otherwise we reset it (if current buffer is not displayed) after
+         * switch
+         */
         gui_window_switch_to_buffer (gui_current_window, buffer,
                                      (string_strcasecmp (value, "auto") == 0) ?
                                      0 : 1);
@@ -1052,6 +1293,14 @@ gui_buffer_set (struct t_gui_buffer *buffer, const char *property,
     {
         gui_buffer_set_highlight_words (buffer, value);
     }
+    else if (string_strcasecmp (property, "highlight_words_add") == 0)
+    {
+        gui_buffer_add_highlight_words (buffer, value);
+    }
+    else if (string_strcasecmp (property, "highlight_words_del") == 0)
+    {
+        gui_buffer_remove_highlight_words (buffer, value);
+    }
     else if (string_strcasecmp (property, "highlight_tags") == 0)
     {
         gui_buffer_set_highlight_tags (buffer, value);
@@ -1063,7 +1312,10 @@ gui_buffer_set (struct t_gui_buffer *buffer, const char *property,
     else if (string_strncasecmp (property, "key_unbind_", 11) == 0)
     {
         if (strcmp (property + 11, "*") == 0)
-            gui_keyboard_free_all (&buffer->keys, &buffer->last_key);
+        {
+            gui_keyboard_free_all (&buffer->keys, &buffer->last_key,
+                                   &buffer->keys_count);
+        }
         else
             gui_keyboard_unbind (buffer, property + 11, 1);
     }
@@ -1073,6 +1325,13 @@ gui_buffer_set (struct t_gui_buffer *buffer, const char *property,
         gui_input_insert_string (buffer, value, 0);
         gui_input_text_changed_modifier_and_signal (buffer);
     }
+    else if (string_strcasecmp (property, "input_pos") == 0)
+    {
+        error = NULL;
+        number = strtol (value, &error, 10);
+        if (error && !error[0])
+            gui_input_set_pos (buffer, number);
+    }
     else if (string_strcasecmp (property, "input_get_unknown_commands") == 0)
     {
         error = NULL;
@@ -1210,15 +1469,23 @@ gui_buffer_search_by_name (const char *plugin, const char *name)
             plugin_match = 1;
             if (plugin && plugin[0])
             {
-                if (ptr_buffer->plugin)
+                if (ptr_buffer->plugin_name_for_upgrade)
                 {
-                    if (strcmp (plugin, ptr_buffer->plugin->name) != 0)
+                    if (strcmp (plugin, ptr_buffer->plugin_name_for_upgrade) != 0)
                         plugin_match = 0;
                 }
                 else
                 {
-                    if (strcmp (plugin, PLUGIN_CORE) != 0)
-                        plugin_match = 0;
+                    if (ptr_buffer->plugin)
+                    {
+                        if (strcmp (plugin, ptr_buffer->plugin->name) != 0)
+                            plugin_match = 0;
+                    }
+                    else
+                    {
+                        if (strcmp (plugin, PLUGIN_CORE) != 0)
+                            plugin_match = 0;
+                    }
                 }
             }
             if (plugin_match && (strcmp (ptr_buffer->name, name) == 0))
@@ -1325,8 +1592,10 @@ gui_buffer_search_by_partial_name (const char *plugin, const char *name)
     if (buffer_partial_match[0])
         return buffer_partial_match[0];
     
-    /* return buffer partially matching in name
-       (may be NULL if no buffer was found) */
+    /*
+     * return buffer partially matching in name
+     * (may be NULL if no buffer was found)
+     */
     return buffer_partial_match[1];
 }
 
@@ -1434,6 +1703,8 @@ gui_buffer_clear (struct t_gui_buffer *buffer)
         }
     }
     
+    gui_hotlist_remove_buffer (buffer);
+    
     gui_buffer_ask_chat_refresh (buffer, 2);
 }
 
@@ -1462,7 +1733,7 @@ void
 gui_buffer_close (struct t_gui_buffer *buffer)
 {
     struct t_gui_window *ptr_window;
-    struct t_gui_buffer *ptr_buffer;
+    struct t_gui_buffer *ptr_buffer, *ptr_back_to_buffer;
     int index;
     struct t_gui_buffer_visited *ptr_buffer_visited;
     
@@ -1474,14 +1745,21 @@ gui_buffer_close (struct t_gui_buffer *buffer)
         (void)(buffer->close_callback) (buffer->close_callback_data, buffer);
     }
     
+    ptr_back_to_buffer = NULL;
+    
     /* first unmerge buffer if it is merged to at least one other buffer */
     if (gui_buffer_count_merged_buffers (buffer->number) > 1)
+    {
+        ptr_back_to_buffer = gui_buffer_get_next_active_buffer (buffer);
         gui_buffer_unmerge (buffer, -1);
+    }
     
     if (!weechat_quit)
     {
-        /* find other buffer to display: previously visited buffer if current
-           window is displaying buffer, or buffer # - 1 */
+        /*
+         * find other buffer to display: previously visited buffer if current
+         * window is displaying buffer, or buffer # - 1
+         */
         ptr_buffer_visited = NULL;
         if (CONFIG_BOOLEAN(config_look_jump_previous_buffer_when_closing)
             && gui_current_window && (gui_current_window->buffer == buffer))
@@ -1505,26 +1783,29 @@ gui_buffer_close (struct t_gui_buffer *buffer)
                 /* switch to previous buffer */
                 if (gui_buffers != last_gui_buffer)
                 {
-                    if (ptr_buffer_visited)
+                    if (ptr_back_to_buffer)
+                    {
+                        gui_window_switch_to_buffer (ptr_window,
+                                                     ptr_back_to_buffer,
+                                                     1);
+                    }
+                    else if (ptr_buffer_visited)
                     {
                         gui_window_switch_to_buffer (ptr_window,
                                                      ptr_buffer_visited->buffer,
                                                      1);
                     }
+                    else if (ptr_window->buffer->prev_buffer)
+                    {
+                        gui_window_switch_to_buffer (ptr_window,
+                                                     ptr_window->buffer->prev_buffer,
+                                                     1);
+                    }
                     else
                     {
-                        if (ptr_window->buffer->prev_buffer)
-                        {
-                            gui_window_switch_to_buffer (ptr_window,
-                                                         ptr_window->buffer->prev_buffer,
-                                                         1);
-                        }
-                        else
-                        {
-                            gui_window_switch_to_buffer (ptr_window,
-                                                         last_gui_buffer,
-                                                         1);
-                        }
+                        gui_window_switch_to_buffer (ptr_window,
+                                                     last_gui_buffer,
+                                                     1);
                     }
                 }
             }
@@ -1573,7 +1854,8 @@ gui_buffer_close (struct t_gui_buffer *buffer)
         free (buffer->highlight_tags);
     if (buffer->highlight_tags_array)
         string_free_split (buffer->highlight_tags_array);
-    gui_keyboard_free_all (&buffer->keys, &buffer->last_key);
+    gui_keyboard_free_all (&buffer->keys, &buffer->last_key,
+                           &buffer->keys_count);
     gui_buffer_local_var_remove_all (buffer);
     
     /* remove buffer from buffers list */
@@ -1590,7 +1872,7 @@ gui_buffer_close (struct t_gui_buffer *buffer)
          ptr_window = ptr_window->next_window)
     {
         if (ptr_window->buffer == buffer)
-            ptr_window->buffer = NULL;
+            ptr_window->buffer = gui_buffers;
     }
     
     hook_signal_send ("buffer_closed",
@@ -1670,6 +1952,34 @@ gui_buffer_get_next_active_buffer (struct t_gui_buffer *buffer)
     return buffer;
 }
 
+/*
+ * gui_buffer_get_previous_active_buffer: get previous active buffer (when many
+ *                                        buffers are merged)
+ */
+
+struct t_gui_buffer *
+gui_buffer_get_previous_active_buffer (struct t_gui_buffer *buffer)
+{
+    struct t_gui_buffer *ptr_buffer;
+    
+    if (buffer->prev_buffer
+        && (buffer->prev_buffer->number == buffer->number))
+        return buffer->prev_buffer;
+    else
+    {
+        for (ptr_buffer = last_gui_buffer; ptr_buffer;
+             ptr_buffer = ptr_buffer->prev_buffer)
+        {
+            if ((ptr_buffer != buffer)
+                && (ptr_buffer->number == buffer->number))
+            {
+                return ptr_buffer;
+            }
+        }
+    }
+    return buffer;
+}
+
 /*
  * gui_buffer_move_to_number: move a buffer to another number
  */
@@ -1881,12 +2191,14 @@ void
 gui_buffer_unmerge (struct t_gui_buffer *buffer, int number)
 {
     int num_merged;
-    struct t_gui_buffer *ptr_buffer;
+    struct t_gui_buffer *ptr_buffer, *ptr_new_active_buffer;
     
     /* if only one buffer then return */
     if (gui_buffers == last_gui_buffer)
         return;
     
+    ptr_new_active_buffer = NULL;
+    
     num_merged = gui_buffer_count_merged_buffers (buffer->number);
     
     /* can't unmerge if buffer is not merged to at least one buffer */
@@ -1923,9 +2235,9 @@ gui_buffer_unmerge (struct t_gui_buffer *buffer, int number)
     else
     {
         /* remove this buffer from mixed_lines, but keep other buffers merged */
-        ptr_buffer = gui_buffer_get_next_active_buffer (buffer);
-        if (ptr_buffer)
-            gui_buffer_set_active_buffer (ptr_buffer);
+        ptr_new_active_buffer = gui_buffer_get_next_active_buffer (buffer);
+        if (ptr_new_active_buffer)
+            gui_buffer_set_active_buffer (ptr_new_active_buffer);
         gui_line_mixed_free_buffer (buffer);
         buffer->mixed_lines = NULL;
         buffer->lines = buffer->own_lines;
@@ -1976,6 +2288,13 @@ gui_buffer_unmerge (struct t_gui_buffer *buffer, int number)
     }
     
     gui_buffer_compute_num_displayed ();
+
+    if (ptr_new_active_buffer)
+    {
+        gui_line_compute_prefix_max_length (ptr_new_active_buffer->mixed_lines);
+        gui_line_compute_buffer_max_length (ptr_new_active_buffer,
+                                            ptr_new_active_buffer->mixed_lines);
+    }
     
     gui_window_ask_refresh (1);
     
@@ -2229,6 +2548,10 @@ gui_buffer_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!infolist_new_var_integer (ptr_item, "nicklist_display_groups", buffer->nicklist_display_groups))
         return 0;
+    if (!infolist_new_var_integer (ptr_item, "nicklist_max_length", buffer->nicklist_max_length))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "nicklist_visible_count", buffer->nicklist_visible_count))
+        return 0;
     if (!infolist_new_var_string (ptr_item, "title", buffer->title))
         return 0;
     if (!infolist_new_var_integer (ptr_item, "input", buffer->input))
@@ -2247,6 +2570,8 @@ gui_buffer_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!infolist_new_var_integer (ptr_item, "input_buffer_1st_display", buffer->input_buffer_1st_display))
         return 0;
+    if (!infolist_new_var_integer (ptr_item, "num_history", buffer->num_history))
+        return 0;
     if (!infolist_new_var_integer (ptr_item, "text_search", buffer->text_search))
         return 0;
     if (!infolist_new_var_integer (ptr_item, "text_search_exact", buffer->text_search_exact))
@@ -2270,6 +2595,8 @@ gui_buffer_add_to_infolist (struct t_infolist *infolist,
             return 0;
         i++;
     }
+    if (!infolist_new_var_integer (ptr_item, "keys_count", buffer->keys_count))
+        return 0;
     i = 0;
     for (ptr_local_var = buffer->local_variables; ptr_local_var;
          ptr_local_var = ptr_local_var->next_var)
@@ -2381,6 +2708,7 @@ gui_buffer_print_log ()
         log_printf ("  plugin_name_for_upgrade: '%s'",  ptr_buffer->plugin_name_for_upgrade);
         log_printf ("  number . . . . . . . . : %d",    ptr_buffer->number);
         log_printf ("  layout_number. . . . . : %d",    ptr_buffer->layout_number);
+        log_printf ("  layout_applied . . . . : %d",    ptr_buffer->layout_applied);
         log_printf ("  name . . . . . . . . . : '%s'",  ptr_buffer->name);
         log_printf ("  short_name . . . . . . : '%s'",  ptr_buffer->short_name);
         log_printf ("  type . . . . . . . . . : %d",    ptr_buffer->type);
@@ -2429,6 +2757,7 @@ gui_buffer_print_log ()
         log_printf ("  highlight_tags_array . : 0x%lx", ptr_buffer->highlight_tags_array);
         log_printf ("  keys . . . . . . . . . : 0x%lx", ptr_buffer->keys);
         log_printf ("  last_key . . . . . . . : 0x%lx", ptr_buffer->last_key);
+        log_printf ("  keys_count . . . . . . : %d",    ptr_buffer->keys_count);
         log_printf ("  local_variables. . . . : 0x%lx", ptr_buffer->local_variables);
         log_printf ("  last_local_var . . . . : 0x%lx", ptr_buffer->last_local_var);
         log_printf ("  prev_buffer. . . . . . : 0x%lx", ptr_buffer->prev_buffer);
@@ -2437,16 +2766,14 @@ gui_buffer_print_log ()
         if (ptr_buffer->keys)
         {
             log_printf ("");
-            log_printf ("  => keys = 0x%lx, last_key = 0x%lx:",
-                        ptr_buffer->keys, ptr_buffer->last_key);
+            log_printf ("  => keys:");
             gui_keyboard_print_log (ptr_buffer);
         }
 
         if (ptr_buffer->local_variables)
         {
             log_printf ("");
-            log_printf ("  => local_variables = 0x%lx, last_local_var = 0x%lx:",
-                        ptr_buffer->local_variables, ptr_buffer->last_local_var);
+            log_printf ("  => local_variables:");
             for (ptr_local_var = ptr_buffer->local_variables; ptr_local_var;
                  ptr_local_var = ptr_local_var->next_var)
             {
@@ -2459,20 +2786,20 @@ gui_buffer_print_log ()
         }
         
         log_printf ("");
-        log_printf ("  => nicklist_root (addr:0x%lx):", ptr_buffer->nicklist_root);
+        log_printf ("  => nicklist:");
         gui_nicklist_print_log (ptr_buffer->nicklist_root, 0);
         
         log_printf ("");
         log_printf ("  => last 100 lines:");
         num = 0;
-        ptr_line = ptr_buffer->lines->last_line;
+        ptr_line = ptr_buffer->own_lines->last_line;
         while (ptr_line && (num < 100))
         {
             num++;
             ptr_line = ptr_line->prev_line;
         }
         if (!ptr_line)
-            ptr_line = ptr_buffer->lines->first_line;
+            ptr_line = ptr_buffer->own_lines->first_line;
         else
             ptr_line = ptr_line->next_line;
         
