@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-chat.c: chat functions, used by all GUI */
+/*
+ * gui-chat.c: chat functions (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -46,20 +48,23 @@
 #include "gui-window.h"
 
 
-char *gui_chat_buffer = NULL;                 /* buffer for printf          */
-char *gui_chat_prefix[GUI_CHAT_NUM_PREFIXES]; /* prefixes                   */
-char gui_chat_prefix_empty[] = "";            /* empty prefix               */
+char *gui_chat_buffer = NULL;                   /* buffer for printf        */
+char *gui_chat_prefix[GUI_CHAT_NUM_PREFIXES];   /* prefixes                 */
+char gui_chat_prefix_empty[] = "";              /* empty prefix             */
 int gui_chat_time_length = 0;    /* length of time for each line (in chars) */
+int gui_chat_mute = GUI_CHAT_MUTE_DISABLED;     /* mute mode                */
+struct t_gui_buffer *gui_chat_mute_buffer = NULL; /* mute buffer            */
 
 
 /*
- * gui_chat_prefix_build_empty: build empty prefixes
- *                              (called before reading WeeChat config file)
+ * gui_chat_init: init some variables for chat area
+ *                (called before reading WeeChat config file)
  */
 
 void
-gui_chat_prefix_build_empty ()
+gui_chat_init ()
 {
+    /* build empty prefixes */
     gui_chat_prefix[GUI_CHAT_PREFIX_ERROR] = strdup (gui_chat_prefix_empty);
     gui_chat_prefix[GUI_CHAT_PREFIX_NETWORK] = strdup (gui_chat_prefix_empty);
     gui_chat_prefix[GUI_CHAT_PREFIX_ACTION] = strdup (gui_chat_prefix_empty);
@@ -452,6 +457,12 @@ gui_chat_printf_date_tags (struct t_gui_buffer *buffer, time_t date,
         if (buffer->type != GUI_BUFFER_TYPE_FORMATTED)
             return;
     }
+
+    /* if mute is enabled for buffer (or all buffers), then just return */
+    if ((gui_chat_mute == GUI_CHAT_MUTE_ALL_BUFFERS)
+        || ((gui_chat_mute == GUI_CHAT_MUTE_BUFFER)
+            && (gui_chat_mute_buffer == buffer)))
+        return;
     
     if (!gui_chat_buffer)
         gui_chat_buffer = malloc (GUI_CHAT_BUFFER_PRINTF_SIZE);
@@ -495,13 +506,25 @@ gui_chat_printf_date_tags (struct t_gui_buffer *buffer, time_t date,
                                               "weechat_print",
                                               modifier_data,
                                               pos);
-                /* no changes in new message */
-                if (new_msg && (strcmp (message, new_msg) == 0))
-                {
-                    free (new_msg);
-                    new_msg = NULL;
-                }
                 free (modifier_data);
+                if (new_msg)
+                {
+                    if (!new_msg[0] && pos[0])
+                    {
+                        /*
+                         * modifier returned empty message, then we'll not
+                         * print anything
+                         */
+                        free (new_msg);
+                        return;
+                    }
+                    if (strcmp (message, new_msg) == 0)
+                    {
+                        /* no changes in new message */
+                        free (new_msg);
+                        new_msg = NULL;
+                    }
+                }
             }
         }
         
@@ -631,15 +654,28 @@ gui_chat_printf_y (struct t_gui_buffer *buffer, int y, const char *message, ...)
 }
 
 /*
- * gui_chat_free_buffer: free buffer used by chat functions
+ * gui_chat_end: free some variables allocated for chat area
  */
 
 void
-gui_chat_free_buffer ()
+gui_chat_end ()
 {
+    int i;
+    
+    /* free buffer used by chat functions */
     if (gui_chat_buffer)
     {
         free (gui_chat_buffer);
         gui_chat_buffer = NULL;
     }
+    
+    /* free prefixes */
+    for (i = 0; i < GUI_CHAT_NUM_PREFIXES; i++)
+    {
+        if (gui_chat_prefix[i])
+        {
+            free (gui_chat_prefix[i]);
+            gui_chat_prefix[i] = NULL;
+        }
+    }
 }
