@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -91,24 +91,26 @@ enum t_gui_color_enum
 #define GUI_COLOR_CUSTOM_BAR_DELIM (gui_color_get_custom ("bar_delim"))
 #define GUI_COLOR_CUSTOM_BAR_BG    (gui_color_get_custom ("bar_bg"))
 
-#define GUI_COLOR_FG_CHAR              'F'
-#define GUI_COLOR_FG_STR               "F"
-#define GUI_COLOR_BG_CHAR              'B'
-#define GUI_COLOR_BG_STR               "B"
-#define GUI_COLOR_FG_BG_CHAR           '*'
-#define GUI_COLOR_FG_BG_STR            "*"
-#define GUI_COLOR_BAR_CHAR             'b'
-#define GUI_COLOR_BAR_STR              "b"
-#define GUI_COLOR_BAR_FG_CHAR          'F'
-#define GUI_COLOR_BAR_FG_STR           "F"
-#define GUI_COLOR_BAR_DELIM_CHAR       'D'
-#define GUI_COLOR_BAR_DELIM_STR        "D"
-#define GUI_COLOR_BAR_BG_CHAR          'B'
-#define GUI_COLOR_BAR_BG_STR           "B"
-#define GUI_COLOR_BAR_START_INPUT_CHAR '_'
-#define GUI_COLOR_BAR_START_INPUT_STR  "_"
-#define GUI_COLOR_BAR_MOVE_CURSOR_CHAR '#'
-#define GUI_COLOR_BAR_MOVE_CURSOR_STR  "#"
+#define GUI_COLOR_FG_CHAR                     'F'
+#define GUI_COLOR_FG_STR                      "F"
+#define GUI_COLOR_BG_CHAR                     'B'
+#define GUI_COLOR_BG_STR                      "B"
+#define GUI_COLOR_FG_BG_CHAR                  '*'
+#define GUI_COLOR_FG_BG_STR                   "*"
+#define GUI_COLOR_BAR_CHAR                    'b'
+#define GUI_COLOR_BAR_STR                     "b"
+#define GUI_COLOR_BAR_FG_CHAR                 'F'
+#define GUI_COLOR_BAR_FG_STR                  "F"
+#define GUI_COLOR_BAR_DELIM_CHAR              'D'
+#define GUI_COLOR_BAR_DELIM_STR               "D"
+#define GUI_COLOR_BAR_BG_CHAR                 'B'
+#define GUI_COLOR_BAR_BG_STR                  "B"
+#define GUI_COLOR_BAR_START_INPUT_CHAR        '_'
+#define GUI_COLOR_BAR_START_INPUT_STR         "_"
+#define GUI_COLOR_BAR_START_INPUT_HIDDEN_CHAR '-'
+#define GUI_COLOR_BAR_START_INPUT_HIDDEN_STR  "-"
+#define GUI_COLOR_BAR_MOVE_CURSOR_CHAR        '#'
+#define GUI_COLOR_BAR_MOVE_CURSOR_STR         "#"
 
 /* color structure */
 
@@ -126,7 +128,7 @@ extern struct t_gui_color *gui_color[];
 
 /* color functions */
 
-extern int gui_color_search_config_int (const char *color_name);
+extern const char *gui_color_search_config (const char *color_name);
 extern const char *gui_color_get_custom (const char *color_name);
 extern char *gui_color_decode (const char *string, const char *replacement);
 extern void gui_color_free (struct t_gui_color *color);
