@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-completion.c: word completion according to context */
+/*
+ * gui-completion.c: word completion according to context (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -25,9 +27,11 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <limits.h>
 #include <unistd.h>
 
 #include "../core/weechat.h"
@@ -63,7 +67,7 @@ gui_completion_buffer_init (struct t_gui_completion *completion,
     completion->base_command_arg_index = 0;
     completion->base_word = NULL;
     completion->base_word_pos = 0;
-    completion->position = -1; 
+    completion->position = -1;
     completion->args = NULL;
     completion->direction = 0;
     completion->add_space = 1;
@@ -335,9 +339,9 @@ gui_completion_list_add (struct t_gui_completion *completion, const char *word,
     
     if (!completion->base_word || !completion->base_word[0]
         || (nick_completion && (gui_completion_nickncmp (completion->base_word, word,
-                                                         strlen (completion->base_word)) == 0))
+                                                         utf8_strlen (completion->base_word)) == 0))
         || (!nick_completion && (string_strncasecmp (completion->base_word, word,
-                                                     strlen (completion->base_word)) == 0)))
+                                                     utf8_strlen (completion->base_word)) == 0)))
     {
         if (nick_completion && (completion->base_word_pos == 0))
         {
@@ -496,6 +500,76 @@ gui_completion_list_add_buffers_plugins_names_cb (void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * gui_completion_list_add_buffer_properties_set_cb: add buffer properties
+ *                                                   (that can be set) to
+ *                                                   completion list
+ */
+
+int
+gui_completion_list_add_buffer_properties_set_cb (void *data,
+                                                  const char *completion_item,
+                                                  struct t_gui_buffer *buffer,
+                                                  struct t_gui_completion *completion)
+{
+    int i;
+    
+    /* make C compiler happy */
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+    
+    for (i = 0; gui_buffer_properties_set[i]; i++)
+    {
+        gui_completion_list_add (completion,
+                                 gui_buffer_properties_set[i],
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+    
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * gui_completion_list_add_buffer_properties_get_cb: add buffer properties
+ *                                                   (that can be read) to
+ *                                                   completion list
+ */
+
+int
+gui_completion_list_add_buffer_properties_get_cb (void *data,
+                                                  const char *completion_item,
+                                                  struct t_gui_buffer *buffer,
+                                                  struct t_gui_completion *completion)
+{
+    int i;
+    
+    /* make C compiler happy */
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+    
+    for (i = 0; gui_buffer_properties_get_integer[i]; i++)
+    {
+        gui_completion_list_add (completion,
+                                 gui_buffer_properties_get_integer[i],
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+    for (i = 0; gui_buffer_properties_get_string[i]; i++)
+    {
+        gui_completion_list_add (completion,
+                                 gui_buffer_properties_get_string[i],
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+    for (i = 0; gui_buffer_properties_get_pointer[i]; i++)
+    {
+        gui_completion_list_add (completion,
+                                 gui_buffer_properties_get_pointer[i],
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+    
+    return WEECHAT_RC_OK;
+}
+
 /*
  * gui_completion_list_add_config_files_cb: add config files to completion list
  */
@@ -550,39 +624,43 @@ gui_completion_list_add_filename_cb (void *data,
     buf_len = PATH_MAX;
     buf = malloc (buf_len);
     if (!buf)
-	return WEECHAT_RC_OK;
+        return WEECHAT_RC_OK;
     
     completion->add_space = 0;
     
     if ((strncmp (completion->base_word, home, 2) == 0) && getenv("HOME"))
     {
-	real_prefix = strdup (getenv("HOME"));
-	prefix = strdup (home);
-    }
-    else if ((strncmp (completion->base_word, DIR_SEPARATOR, 1) != 0)
-	     || (strcmp (completion->base_word, "") == 0))
-    {
-	real_prefix = strdup (weechat_home);
-	prefix = strdup ("");
+        real_prefix = strdup (getenv("HOME"));
+        prefix = strdup (home);
     }
     else
     {
-	real_prefix = strdup (DIR_SEPARATOR);
-	prefix = strdup (DIR_SEPARATOR);
+        if ((strncmp (completion->base_word, DIR_SEPARATOR, 1) != 0)
+            || (strcmp (completion->base_word, "") == 0))
+        {
+            real_prefix = strdup (weechat_home);
+            prefix = strdup ("");
+        }
+        else
+        {
+            real_prefix = strdup (DIR_SEPARATOR);
+            prefix = strdup (DIR_SEPARATOR);
+        }
     }
-	
+    
     snprintf (buf, buf_len, "%s", completion->base_word + strlen (prefix));
     p = strrchr (buf, DIR_SEPARATOR_CHAR);
     if (p)
     {
-	*p = '\0';
-	path_d = strdup (buf);
-	p++;
-	path_b = strdup (p);
+        p[0] = '\0';
+        path_d = strdup (buf);
+        p++;
+        path_b = strdup (p);
     }
-    else {
-	path_d = strdup ("");
-	path_b = strdup (buf);
+    else
+    {
+        path_d = strdup ("");
+        path_b = strdup (buf);
     }
     
     sprintf (buf, "%s%s%s", real_prefix, DIR_SEPARATOR, path_d);
@@ -590,31 +668,31 @@ gui_completion_list_add_filename_cb (void *data,
     dp = opendir (d_name);
     if (dp != NULL)
     {
-	while ((entry = readdir (dp)) != NULL) 
-	{	
-	    if (strncmp (entry->d_name, path_b, strlen (path_b)) == 0) {
-		
-		if (strcmp (entry->d_name, ".") == 0 || strcmp (entry->d_name, "..") == 0)
-		    continue;
-		
-		snprintf(buf, buf_len, "%s%s%s", 
-			 d_name, DIR_SEPARATOR, entry->d_name);
-		if (stat(buf, &statbuf) == -1)
-		    continue;
-		
-		snprintf(buf, buf_len, "%s%s%s%s%s%s",
-			 prefix, 
-			 ((strcmp(prefix, "") == 0)
-			  || strchr(prefix, DIR_SEPARATOR_CHAR)) ? "" : DIR_SEPARATOR, 
-			 path_d, 
-			 strcmp(path_d, "") == 0 ? "" : DIR_SEPARATOR, 
-			 entry->d_name, 
-			 S_ISDIR(statbuf.st_mode) ? DIR_SEPARATOR : "");
-		
-		gui_completion_list_add (completion, buf,
+        while ((entry = readdir (dp)) != NULL)
+        {
+            if (strncmp (entry->d_name, path_b, strlen (path_b)) == 0)
+            {
+                if (strcmp (entry->d_name, ".") == 0 || strcmp (entry->d_name, "..") == 0)
+                    continue;
+                
+                snprintf (buf, buf_len, "%s%s%s",
+                          d_name, DIR_SEPARATOR, entry->d_name);
+                if (stat (buf, &statbuf) == -1)
+                    continue;
+                
+                snprintf (buf, buf_len, "%s%s%s%s%s%s",
+                          prefix,
+                          ((strcmp(prefix, "") == 0)
+                           || strchr(prefix, DIR_SEPARATOR_CHAR)) ? "" : DIR_SEPARATOR,
+                          path_d,
+                          strcmp(path_d, "") == 0 ? "" : DIR_SEPARATOR,
+                          entry->d_name,
+                          S_ISDIR(statbuf.st_mode) ? DIR_SEPARATOR : "");
+                
+                gui_completion_list_add (completion, buf,
                                          0, WEECHAT_LIST_POS_SORT);
-	    }
-	}
+            }
+        }
         closedir (dp);
     }
     
@@ -910,8 +988,10 @@ gui_completion_list_add_plugins_commands_cb (void *data,
             ptr_plugin = NULL;
             if (string_strcasecmp (plugin_name, PLUGIN_CORE) != 0)
             {
-                /* plugin name is different from "core", then search it in
-                   plugin list */
+                /*
+                 * plugin name is different from "core", then search it in
+                 * plugin list
+                 */
                 ptr_plugin = plugin_search (plugin_name);
                 if (!ptr_plugin)
                     return WEECHAT_RC_OK;
@@ -1286,6 +1366,61 @@ gui_completion_list_add_keys_codes_cb (void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * gui_completion_list_add_keys_codes_for_reset_cb: add keys that can be reset
+ *                                                  (keys added, redefined or
+ *                                                  removed) to completion list
+ */
+
+int
+gui_completion_list_add_keys_codes_for_reset_cb (void *data,
+                                                 const char *completion_item,
+                                                 struct t_gui_buffer *buffer,
+                                                 struct t_gui_completion *completion)
+{
+    struct t_gui_key *ptr_key, *ptr_default_key;
+    char *expanded_name;
+    
+    /* make C compiler happy */
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+    
+    /* keys added or redefined */
+    for (ptr_key = gui_keys; ptr_key; ptr_key = ptr_key->next_key)
+    {
+        ptr_default_key = gui_keyboard_search (gui_default_keys, ptr_key->key);
+        if (!ptr_default_key
+            || (strcmp (ptr_default_key->command, ptr_key->command) != 0))
+        {
+            expanded_name = gui_keyboard_get_expanded_name (ptr_key->key);
+            gui_completion_list_add (completion,
+                                     (expanded_name) ? expanded_name : ptr_key->key,
+                                     0, WEECHAT_LIST_POS_SORT);
+            if (expanded_name)
+                free (expanded_name);
+        }
+    }
+    
+    /* keys deleted */
+    for (ptr_default_key = gui_default_keys; ptr_default_key;
+         ptr_default_key = ptr_default_key->next_key)
+    {
+        ptr_key = gui_keyboard_search (gui_keys, ptr_default_key->key);
+        if (!ptr_key)
+        {
+            expanded_name = gui_keyboard_get_expanded_name (ptr_default_key->key);
+            gui_completion_list_add (completion,
+                                     (expanded_name) ? expanded_name : ptr_default_key->key,
+                                     0, WEECHAT_LIST_POS_SORT);
+            if (expanded_name)
+                free (expanded_name);
+        }
+    }
+    
+    return WEECHAT_RC_OK;
+}
+
 /*
  * gui_completion_custom: custom completion by a plugin
  */
@@ -1419,8 +1554,10 @@ gui_completion_get_template_for_args (struct t_gui_completion *completion,
 {
     int matching_template;
     
-    /* if template refers to another command, search this command and use its
-       template */
+    /*
+     * if template refers to another command, search this command and use its
+     * template
+     */
     if ((HOOK_COMMAND(hook_command, cplt_templates)[0][0] == '%')
         && (HOOK_COMMAND(hook_command, cplt_templates)[0][1] == '%')
         && (HOOK_COMMAND(hook_command, cplt_templates)[0][1]))
@@ -1469,8 +1606,16 @@ gui_completion_build_list (struct t_gui_completion *completion)
     repeat_last = 0;
     
     ptr_hook = gui_completion_search_command (completion->base_command);
-    if (!ptr_hook || !HOOK_COMMAND(ptr_hook, completion)
-        || (strcmp (HOOK_COMMAND(ptr_hook, completion), "-") == 0))
+    if (!ptr_hook || !HOOK_COMMAND(ptr_hook, completion))
+    {
+        completion->context = GUI_COMPLETION_AUTO;
+        completion->base_command_arg_index = 0;
+        free (completion->base_command);
+        completion->base_command = NULL;
+        return;
+    }
+    
+    if (strcmp (HOOK_COMMAND(ptr_hook, completion), "-") == 0)
     {
         gui_completion_stop (completion, 1);
         return;
@@ -1532,11 +1677,12 @@ gui_completion_find_context (struct t_gui_completion *completion,
                              const char *data, int size, int pos)
 {
     int i, command, command_arg, pos_start, pos_end;
+    char *prev_char;
     
     /* look for context */
     gui_completion_free_data (completion);
     gui_completion_buffer_init (completion, completion->buffer);
-    command = (data[0] == '/') ? 1 : 0;
+    command = (string_is_command_char (data)) ? 1 : 0;
     command_arg = 0;
     i = 0;
     while (i < pos)
@@ -1609,8 +1755,8 @@ gui_completion_find_context (struct t_gui_completion *completion,
         if (completion->context == GUI_COMPLETION_COMMAND)
         {
             pos_start++;
-            if (data[pos_start] == '/')
-                pos_start++;
+            if (string_is_command_char (data + pos_start))
+                pos_start += utf8_char_size (data + pos_start);
         }
         
         completion->base_word_pos = pos_start;
@@ -1634,22 +1780,25 @@ gui_completion_find_context (struct t_gui_completion *completion,
     if (completion->context == GUI_COMPLETION_COMMAND_ARG)
     {
         pos_start = 0;
-        while ((pos_start < size) && (data[pos_start] != '/'))
+        while ((pos_start < size) && !string_is_command_char (data + pos_start))
         {
-            pos_start++;
+            pos_start += utf8_char_size (data + pos_start);
         }
-        if (data[pos_start] == '/')
+        if (string_is_command_char (data + pos_start))
         {
-            pos_start++;
-            if (data[pos_start] == '/')
-                pos_start++;
+            pos_start += utf8_char_size (data + pos_start);
+            if (string_is_command_char (data + pos_start))
+                pos_start += utf8_char_size (data + pos_start);
             pos_end = pos_start;
             while ((pos_end < size) && (data[pos_end] != ' '))
             {
-                pos_end++;
+                pos_end += utf8_char_size (data + pos_end);
             }
             if (data[pos_end] == ' ')
-                pos_end--;
+            {
+                prev_char = utf8_prev_char (data, data + pos_end);
+                pos_end -= utf8_char_size (prev_char);
+            }
             
             completion->base_command = malloc (pos_end - pos_start + 2);
             for (i = pos_start; i <= pos_end; i++)
@@ -1661,8 +1810,10 @@ gui_completion_find_context (struct t_gui_completion *completion,
         }
     }
     
-    /* auto completion with nothing as base word is disabled,
-       in order to prevent completion when pasting messages with [tab] inside */
+    /*
+     * auto completion with nothing as base word is disabled,
+     * in order to prevent completion when pasting messages with [tab] inside
+     */
     if ((completion->context == GUI_COMPLETION_AUTO)
         && ((!completion->base_word) || (!completion->base_word[0])))
     {
@@ -1797,7 +1948,7 @@ gui_completion_complete (struct t_gui_completion *completion)
     int common_prefix_size, item_is_nick;
     struct t_weelist_item *ptr_item, *ptr_item2;
     
-    length = strlen (completion->base_word);
+    length = utf8_strlen (completion->base_word);
     word_found_seen = 0;
     other_completion = 0;
     
@@ -1832,14 +1983,14 @@ gui_completion_complete (struct t_gui_completion *completion)
     
     if (partial_completion
         && completion->word_found
-        && ((int)strlen (completion->word_found) >= common_prefix_size))
+        && (utf8_strlen (completion->word_found) >= common_prefix_size))
     {
         return;
     }
     
     while (ptr_item)
     {
-        item_is_nick = ((int)(ptr_item->user_data) == 1);
+        item_is_nick = ((long)(ptr_item->user_data) == 1);
         if ((item_is_nick
              && (gui_completion_nickncmp (completion->base_word, ptr_item->data,
                                           length) == 0))
@@ -1900,7 +2051,7 @@ gui_completion_complete (struct t_gui_completion *completion)
                 
                 /* stop after common prefix, if asked by user */
                 if (partial_completion
-                    && (((int)strlen (completion->word_found) >= common_prefix_size))
+                    && ((utf8_strlen (completion->word_found) >= common_prefix_size))
                     && (other_completion > 0))
                 {
                     completion->word_found[common_prefix_size] = '\0';
@@ -1913,8 +2064,10 @@ gui_completion_complete (struct t_gui_completion *completion)
                     if (CONFIG_BOOLEAN(config_completion_partial_completion_alert))
                         printf ("\a");
                     
-                    /* send "partial_completion" signal, to display possible
-                       completions in bar item */
+                    /*
+                     * send "partial_completion" signal, to display possible
+                     * completions in bar item
+                     */
                     gui_completion_partial_build_list (completion,
                                                        common_prefix_size);
                     hook_signal_send ("partial_completion",
@@ -1938,8 +2091,10 @@ gui_completion_complete (struct t_gui_completion *completion)
             ptr_item = ptr_item->next_item;
     }
     
-    /* if we was on last completion in list, then recomplete, starting from
-       first matching item */
+    /*
+     * if we was on last completion in list, then recomplete, starting from
+     * first matching item
+     */
     if (completion->word_found && (completion->position >= 0))
     {
         free (completion->word_found);
@@ -2127,55 +2282,61 @@ gui_completion_print_log (struct t_gui_completion *completion)
 void
 gui_completion_init ()
 {
-    hook_completion (NULL, "buffers_names", /* it was %b */
+    hook_completion (NULL, "buffers_names", /* formerly "%b" */
                      N_("names of buffers"),
                      &gui_completion_list_add_buffers_names_cb, NULL);
     hook_completion (NULL, "buffers_numbers",
                      N_("numbers of buffers"),
                      &gui_completion_list_add_buffers_numbers_cb, NULL);
-    hook_completion (NULL, "buffers_plugins_names", /* it was %B */
+    hook_completion (NULL, "buffers_plugins_names", /* formerly "%B" */
                      N_("names of buffers (including plugins names)"),
                      &gui_completion_list_add_buffers_plugins_names_cb, NULL);
-    hook_completion (NULL, "config_files", /* it was %c */
+    hook_completion (NULL, "buffer_properties_set",
+                     N_("properties that can be set on a buffer"),
+                     &gui_completion_list_add_buffer_properties_set_cb, NULL);
+    hook_completion (NULL, "buffer_properties_get",
+                     N_("properties that can be read on a buffer"),
+                     &gui_completion_list_add_buffer_properties_get_cb, NULL);
+    hook_completion (NULL, "config_files", /* formerly "%c" */
                      N_("configuration files"),
                      &gui_completion_list_add_config_files_cb, NULL);
-    hook_completion (NULL, "filename", /* it was %f */
+    hook_completion (NULL, "filename", /* formerly "%f" */
                      N_("filename"),
                      &gui_completion_list_add_filename_cb, NULL);
-    hook_completion (NULL, "filters_names", /* it was %F */
+    hook_completion (NULL, "filters_names", /* formerly "%F" */
                      N_("names of filters"),
                      &gui_completion_list_add_filters_cb, NULL);
-    hook_completion (NULL, "commands", /* it was %h */
+    hook_completion (NULL, "commands", /* formerly "%h" */
                      N_("commands (weechat and plugins)"),
                      &gui_completion_list_add_commands_cb, NULL);
-    hook_completion (NULL, "infos", /* it was %i */
+    hook_completion (NULL, "infos", /* formerly "%i" */
                      N_("names of infos hooked"),
                      &gui_completion_list_add_infos_cb, NULL);
-    hook_completion (NULL, "infolists", /* it was %I */
+    hook_completion (NULL, "infolists", /* formerly "%I" */
                      N_("names of infolists hooked"),
                      &gui_completion_list_add_infolists_cb, NULL);
-    hook_completion (NULL, "nicks", /* it was %n */
+    hook_completion (NULL, "nicks", /* formerly "%n" */
                      N_("nicks in nicklist of current buffer"),
                      &gui_completion_list_add_nicks_cb, NULL);
-    hook_completion (NULL, "config_options", /* it was %o */
+    hook_completion (NULL, "config_options", /* formerly "%o" */
                      N_("configuration options"),
                      &gui_completion_list_add_config_options_cb, NULL);
-    hook_completion (NULL, "plugins_names", /* it was %p */
+    hook_completion (NULL, "plugins_names", /* formerly "%p" */
                      N_("names of plugins"),
                      &gui_completion_list_add_plugins_cb, NULL);
-    hook_completion (NULL, "plugins_commands", /* it was %P */
+    hook_completion (NULL, "plugins_commands", /* formerly "%P" */
                      N_("commands defined by plugins"),
                      &gui_completion_list_add_plugins_commands_cb, NULL);
-    hook_completion (NULL, "bars_names", /* it was %r */
+    hook_completion (NULL, "bars_names", /* formerly "%r" */
                      N_("names of bars"),
                      &gui_completion_list_add_bars_names_cb, NULL);
-    hook_completion (NULL, "config_option_values", /* it was %v */
+    hook_completion (NULL, "config_option_values", /* formerly "%v" */
                      N_("values for a configuration option"),
                      &gui_completion_list_add_config_option_values_cb, NULL);
-    hook_completion (NULL, "weechat_commands", /* it was %w */
+    hook_completion (NULL, "weechat_commands", /* formerly "%w" */
                      N_("weechat commands"),
                      &gui_completion_list_add_weechat_commands_cb, NULL);
-    hook_completion (NULL, "proxies_names", /* it was %y */
+    hook_completion (NULL, "proxies_names", /* formerly "%y" */
                      N_("names of proxies"),
                      &gui_completion_list_add_proxies_names_cb, NULL);
     hook_completion (NULL, "proxies_options",
@@ -2187,4 +2348,8 @@ gui_completion_init ()
     hook_completion (NULL, "keys_codes",
                      N_("key codes"),
                      &gui_completion_list_add_keys_codes_cb, NULL);
+    hook_completion (NULL, "keys_codes_for_reset",
+                     N_("key codes that can be reset (keys added, redefined "
+                        "or removed)"),
+                     &gui_completion_list_add_keys_codes_for_reset_cb, NULL);
 }
