@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-line.c: line functions, used by all GUI */
+/*
+ * gui-line.c: line functions (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -88,7 +90,12 @@ gui_line_get_align (struct t_gui_buffer *buffer, struct t_gui_line *line,
     int length_time, length_buffer, length_suffix;
     
     /* length of time */
-    length_time = (buffer->time_for_each_line) ? gui_chat_time_length : 0;
+    if (buffer->time_for_each_line)
+    {
+        length_time = (gui_chat_time_length == 0) ? 0 : gui_chat_time_length + 1;
+    }
+    else
+        length_time = 0;
     
     /* length of buffer name (when many buffers are merged) */
     if (buffer->mixed_lines)
@@ -110,7 +117,7 @@ gui_line_get_align (struct t_gui_buffer *buffer, struct t_gui_line *line,
         length_buffer = 0;
     
     if (CONFIG_INTEGER(config_look_prefix_align) == CONFIG_LOOK_PREFIX_ALIGN_NONE)
-        return length_time + 1 + length_buffer + line->data->prefix_length
+        return length_time + length_buffer + line->data->prefix_length
             + ((line->data->prefix_length > 0) ? 1 : 0);
     
     length_suffix = 0;
@@ -126,7 +133,7 @@ gui_line_get_align (struct t_gui_buffer *buffer, struct t_gui_line *line,
         + (((CONFIG_INTEGER(config_look_prefix_align_max) > 0)
             && (buffer->lines->prefix_max_length > CONFIG_INTEGER(config_look_prefix_align_max))) ?
            CONFIG_INTEGER(config_look_prefix_align_max) : buffer->lines->prefix_max_length)
-        + length_suffix + 1;
+        + length_suffix;
 }
 
 /*
@@ -356,10 +363,12 @@ int
 gui_line_has_highlight (struct t_gui_line *line)
 {
     int rc, i;
-    char *msg_no_color;
+    char *msg_no_color, *highlight_words;
     
-    /* highlights are disabled on this buffer? (special value "-" means that
-       buffer does not want any highlight) */
+    /*
+     * highlights are disabled on this buffer? (special value "-" means that
+     * buffer does not want any highlight)
+     */
     if (line->data->buffer->highlight_words
         && (strcmp (line->data->buffer->highlight_words, "-") == 0))
         return 0;
@@ -371,8 +380,10 @@ gui_line_has_highlight (struct t_gui_line *line)
             return 0;
     }
     
-    /* check that line matches highlight tags, if any (if no tag is specified,
-       then any tag is allowed) */
+    /*
+     * check that line matches highlight tags, if any (if no tag is specified,
+     * then any tag is allowed)
+     */
     if (line->data->buffer->highlight_tags_count > 0)
     {
         if (!gui_line_match_tags (line,
@@ -386,12 +397,28 @@ gui_line_has_highlight (struct t_gui_line *line)
     if (!msg_no_color)
         return 0;
     
-    /* there is highlight on line if one of global highlight words matches line
-       or one of buffer highlight words matches line */
-    rc = (string_has_highlight (msg_no_color,
-                                CONFIG_STRING(config_look_highlight)) ||
-          string_has_highlight (msg_no_color,
-                                line->data->buffer->highlight_words));
+    /*
+     * there is highlight on line if one of buffer highlight words matches line
+     * or one of global highlight words matches line
+     */
+    highlight_words = gui_buffer_string_replace_local_var (line->data->buffer,
+                                                           line->data->buffer->highlight_words);
+    rc = string_has_highlight (msg_no_color,
+                               (highlight_words) ?
+                               highlight_words : line->data->buffer->highlight_words);
+    if (highlight_words)
+        free (highlight_words);
+    
+    if (!rc)
+    {
+        highlight_words = gui_buffer_string_replace_local_var (line->data->buffer,
+                                                               CONFIG_STRING(config_look_highlight));
+        rc = string_has_highlight (msg_no_color,
+                                   (highlight_words) ?
+                                   highlight_words : CONFIG_STRING(config_look_highlight));
+        if (highlight_words)
+            free (highlight_words);
+    }
     
     free (msg_no_color);
     
@@ -474,8 +501,20 @@ gui_line_remove_from_list (struct t_gui_buffer *buffer,
                            struct t_gui_line *line,
                            int free_data)
 {
+    struct t_gui_window *ptr_win;
     int update_prefix_max_length;
     
+    /* reset scroll for any window starting with this line */
+    for (ptr_win = gui_windows; ptr_win; ptr_win = ptr_win->next_window)
+    {
+        if (ptr_win->start_line == line)
+        {
+            ptr_win->start_line = ptr_win->start_line->next_line;
+            ptr_win->start_line_pos = 0;
+            gui_buffer_ask_chat_refresh (buffer, 2);
+        }
+    }
+    
     update_prefix_max_length =
         (line->data->prefix_length == lines->prefix_max_length);
     
@@ -594,7 +633,6 @@ void
 gui_line_free (struct t_gui_buffer *buffer, struct t_gui_line *line)
 {
     struct t_gui_line *ptr_line;
-    struct t_gui_window *ptr_win;
     
     /* first remove mixed line if it exists */
     if (buffer->mixed_lines)
@@ -613,17 +651,6 @@ gui_line_free (struct t_gui_buffer *buffer, struct t_gui_line *line)
         }
     }
     
-    /* reset scroll for any window starting with this line */
-    for (ptr_win = gui_windows; ptr_win; ptr_win = ptr_win->next_window)
-    {
-        if (ptr_win->start_line == line)
-        {
-            ptr_win->start_line = ptr_win->start_line->next_line;
-            ptr_win->start_line_pos = 0;
-            gui_buffer_ask_chat_refresh (buffer, 2);
-        }
-    }
-    
     /* remove line from lines list */
     gui_line_remove_from_list (buffer, buffer->own_lines, line, 1);
 }
