@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-nicklist.c: nicklist functions, used by all GUI */
+/*
+ * gui-nicklist.c: nicklist functions (used by all GUI)
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -45,13 +47,36 @@
 
 
 /*
- * gui_nicklist_changed_signal: send "nicklist_changed" signal
+ * gui_nicklist_send_signal: send a signal when something has changed in
+ *                           nicklist
  */
 
 void
-gui_nicklist_changed_signal ()
+gui_nicklist_send_signal (const char *signal, struct t_gui_buffer *buffer,
+                          const char *arguments)
 {
-    hook_signal_send ("nicklist_changed", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    char *str_args;
+    int length;
+    
+    if (buffer)
+    {
+        length = 128 + ((arguments) ? strlen (arguments) : 0) + 1 + 1;
+        str_args = malloc (length);
+        if (str_args)
+        {
+            snprintf (str_args, length,
+                      "0x%lx,%s",
+                      (long unsigned int)(buffer),
+                      (arguments) ? arguments : "");
+            hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING, str_args);
+            free (str_args);
+        }
+    }
+    else
+    {
+        hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING,
+                          (char *)arguments);
+    }
 }
 
 /*
@@ -201,7 +226,7 @@ gui_nicklist_add_group (struct t_gui_buffer *buffer,
     if (buffer->nicklist_display_groups && visible)
         buffer->nicklist_visible_count++;
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_group_added", buffer, name);
     
     return new_group;
 }
@@ -287,6 +312,9 @@ gui_nicklist_search_nick (struct t_gui_buffer *buffer,
     if (!buffer && !from_group)
         return NULL;
     
+    if (!from_group && !buffer->nicklist_root)
+        return NULL;
+    
     for (ptr_nick = (from_group) ? from_group->nicks : buffer->nicklist_root->nicks;
          ptr_nick; ptr_nick = ptr_nick->next_nick)
     {
@@ -339,7 +367,7 @@ gui_nicklist_add_nick (struct t_gui_buffer *buffer,
     if (visible)
         buffer->nicklist_visible_count++;
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_nick_added", buffer, name);
     
     return new_nick;
 }
@@ -352,9 +380,13 @@ void
 gui_nicklist_remove_nick (struct t_gui_buffer *buffer,
                           struct t_gui_nick *nick)
 {
+    char *nick_removed;
+    
     if (!buffer || !nick)
         return;
     
+    nick_removed = (nick->name) ? strdup (nick->name) : NULL;
+    
     /* remove nick from list */
     if (nick->prev_nick)
         (nick->prev_nick)->next_nick = nick->next_nick;
@@ -383,7 +415,10 @@ gui_nicklist_remove_nick (struct t_gui_buffer *buffer,
     
     free (nick);
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_nick_removed", buffer, nick_removed);
+    
+    if (nick_removed)
+        free (nick_removed);
 }
 
 /*
@@ -394,9 +429,13 @@ void
 gui_nicklist_remove_group (struct t_gui_buffer *buffer,
                            struct t_gui_nick_group *group)
 {
+    char *group_removed;
+    
     if (!buffer || !group)
         return;
     
+    group_removed = (group->name) ? strdup (group->name) : NULL;
+    
     /* remove childs first */
     while (group->childs)
     {
@@ -441,7 +480,10 @@ gui_nicklist_remove_group (struct t_gui_buffer *buffer,
     
     free (group);
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_group_removed", buffer, group_removed);
+    
+    if (group_removed)
+        free (group_removed);
 }
 
 /*
@@ -628,6 +670,82 @@ gui_nicklist_compute_visible_count (struct t_gui_buffer *buffer,
         buffer->nicklist_visible_count++;
 }
 
+/*
+ * gui_nicklist_add_group_to_infolist: add a group in an infolist
+ *                                     return 1 if ok, 0 if error
+ */
+
+int
+gui_nicklist_add_group_to_infolist (struct t_infolist *infolist,
+                                    struct t_gui_nick_group *group)
+{
+    struct t_infolist_item *ptr_item;
+    
+    if (!infolist || !group)
+        return 0;
+    
+    ptr_item = infolist_new_item (infolist);
+    if (!ptr_item)
+        return 0;
+    
+    if (!infolist_new_var_string (ptr_item, "type", "group"))
+        return 0;
+    if (group->parent)
+    {
+        if (!infolist_new_var_string (ptr_item, "parent_name", group->parent->name))
+            return 0;
+    }
+    if (!infolist_new_var_string (ptr_item, "name", group->name))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "color", group->color))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "visible", group->visible))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "level", group->level))
+        return 0;
+    
+    return 1;
+}
+
+/*
+ * gui_nicklist_add_nick_to_infolist: add a nick in an infolist
+ *                                    return 1 if ok, 0 if error
+ */
+
+int
+gui_nicklist_add_nick_to_infolist (struct t_infolist *infolist,
+                                   struct t_gui_nick *nick)
+{
+    struct t_infolist_item *ptr_item;
+    
+    if (!infolist || !nick)
+        return 0;
+    
+    ptr_item = infolist_new_item (infolist);
+    if (!ptr_item)
+        return 0;
+    
+    if (!infolist_new_var_string (ptr_item, "type", "nick"))
+        return 0;
+    if (nick->group)
+    {
+        if (!infolist_new_var_string (ptr_item, "group_name", nick->group->name))
+            return 0;
+    }
+    if (!infolist_new_var_string (ptr_item, "name", nick->name))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "color", nick->color))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "prefix", nick->prefix))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "prefix_color", nick->prefix_color))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "visible", nick->visible))
+        return 0;
+    
+    return 1;
+}
+
 /*
  * gui_nicklist_add_to_infolist: add a nicklist in an infolist
  *                               return 1 if ok, 0 if error
@@ -635,62 +753,43 @@ gui_nicklist_compute_visible_count (struct t_gui_buffer *buffer,
 
 int
 gui_nicklist_add_to_infolist (struct t_infolist *infolist,
-                              struct t_gui_buffer *buffer)
+                              struct t_gui_buffer *buffer,
+                              const char *name)
 {
-    struct t_infolist_item *ptr_item;
     struct t_gui_nick_group *ptr_group;
     struct t_gui_nick *ptr_nick;
     
     if (!infolist || !buffer)
         return 0;
     
+    /* add only one nick if asked */
+    if (name && (strncmp (name, "nick_", 5) == 0))
+    {
+        ptr_nick = gui_nicklist_search_nick (buffer, NULL, name + 5);
+        if (!ptr_nick)
+            return 0;
+        return gui_nicklist_add_nick_to_infolist (infolist, ptr_nick);
+    }
+    
+    /* add only one group if asked */
+    if (name && (strncmp (name, "group_", 6) == 0))
+    {
+        ptr_group = gui_nicklist_search_group (buffer, NULL, name + 6);
+        if (!ptr_group)
+            return 0;
+        return gui_nicklist_add_group_to_infolist (infolist, ptr_group);
+    }
+    
     ptr_group = NULL;
     ptr_nick = NULL;
     gui_nicklist_get_next_item (buffer, &ptr_group, &ptr_nick);
     while (ptr_group || ptr_nick)
     {
-        ptr_item = infolist_new_item (infolist);
-        if (!ptr_item)
-            return 0;
-        
         if (ptr_nick)
-        {
-            if (!infolist_new_var_string (ptr_item, "type", "nick"))
-                return 0;
-            if (ptr_nick->group)
-            {
-                if (!infolist_new_var_string (ptr_item, "group_name", ptr_nick->group->name))
-                    return 0;
-            }
-            if (!infolist_new_var_string (ptr_item, "name", ptr_nick->name))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "color", ptr_nick->color))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "prefix", ptr_nick->prefix))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "prefix_color", ptr_nick->prefix_color))
-                return 0;
-            if (!infolist_new_var_integer (ptr_item, "visible", ptr_nick->visible))
-                return 0;
-        }
+            gui_nicklist_add_nick_to_infolist (infolist, ptr_nick);
         else
-        {
-            if (!infolist_new_var_string (ptr_item, "type", "group"))
-                return 0;
-            if (ptr_group->parent)
-            {
-                if (!infolist_new_var_string (ptr_item, "parent_name", ptr_group->parent->name))
-                    return 0;
-            }
-            if (!infolist_new_var_string (ptr_item, "name", ptr_group->name))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "color", ptr_group->color))
-                return 0;
-            if (!infolist_new_var_integer (ptr_item, "visible", ptr_group->visible))
-                return 0;
-            if (!infolist_new_var_integer (ptr_item, "level", ptr_group->level))
-                return 0;
-        }
+            gui_nicklist_add_group_to_infolist (infolist, ptr_group);
+        
         gui_nicklist_get_next_item (buffer, &ptr_group, &ptr_nick);
     }
     
@@ -784,7 +883,7 @@ gui_nicklist_print_log (struct t_gui_nick_group *group, int indent)
                   (indent * 2) + 6);
         log_printf (format, " ", ptr_nick->color);
         snprintf (format, sizeof (format),
-                  "%%-%dsprefix. . . . . : '%%c'",
+                  "%%-%dsprefix. . . . . : '%%s'",
                   (indent * 2) + 6);
         log_printf (format, " ", ptr_nick->prefix);
         snprintf (format, sizeof (format),
