@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-aspell.c: aspell plugin for WeeChat */
+/*
+ * weechat-aspell.c: aspell plugin for WeeChat: use color to show mispelled
+ *                   words in input line
+ */
 
 
 #include <stdio.h>
@@ -35,7 +38,7 @@ WEECHAT_PLUGIN_NAME(ASPELL_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Aspell plugin for WeeChat");
 WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_aspell_plugin = NULL;
 
@@ -134,7 +137,7 @@ struct t_aspell_code langs_avail[] =
     { NULL,     NULL}
 };
 
-struct t_aspell_code countries_avail[] = 
+struct t_aspell_code countries_avail[] =
 {
     { "AT", "Austria" },
     { "BR", "Brazil" },
@@ -406,7 +409,7 @@ weechat_aspell_speller_list_dicts ()
     struct AspellConfig *config;
     AspellDictInfoList *list;
     AspellDictInfoEnumeration *el;
-    const AspellDictInfo *dict;    
+    const AspellDictInfo *dict;
     
     config = new_aspell_config();
     list = get_aspell_dict_info_list (config);
@@ -692,9 +695,11 @@ weechat_aspell_modifier_cb (void *data, const char *modifier,
         && !weechat_config_boolean (weechat_aspell_config_check_during_search))
         return NULL;
     
-    /* for performance: return last stirng built if input string is the
-       same (for example user just change cursor position, or input text is
-       refreshed with same content) */
+    /*
+     * for performance: return last stirng built if input string is the
+     * same (for example user just change cursor position, or input text is
+     * refreshed with same content)
+     */
     if (aspell_last_modifier_string
         && (strcmp (string, aspell_last_modifier_string) == 0))
     {
@@ -733,10 +738,10 @@ weechat_aspell_modifier_cb (void *data, const char *modifier,
         index_result = 0;
         
         /* check if string is a command */
-        if ((ptr_string[0] == '/') && ptr_string[1] && (ptr_string[1] != '/')
-            && (ptr_string[1] != ' '))
+        if (!weechat_string_input_for_buffer (ptr_string))
         {
-            ptr_string++;
+            char_size = weechat_utf8_char_size (ptr_string);
+            ptr_string += char_size;
             pos_space = ptr_string;
             while (pos_space && pos_space[0] && (pos_space[0] != ' '))
             {
@@ -756,10 +761,11 @@ weechat_aspell_modifier_cb (void *data, const char *modifier,
                 free (result);
                 return NULL;
             }
-            result[index_result++] = '/';
+            memcpy (result + index_result, aspell_last_modifier_string, char_size);
+            index_result += char_size;
             strcpy (result + index_result, ptr_string);
             index_result += strlen (ptr_string);
-
+            
             pos_space[0] = ' ';
             ptr_string = pos_space;
         }
@@ -854,11 +860,11 @@ weechat_aspell_command_cb (void *data, struct t_gui_buffer *buffer,
     if (argc > 1)
     {
         if (weechat_strcasecmp (argv[1], "dictlist") == 0)
-        { 
+        {
             weechat_aspell_speller_list_dicts ();
             return WEECHAT_RC_OK;
         }
-        if (weechat_strcasecmp (argv[1], "addword") == 0) 
+        if (weechat_strcasecmp (argv[1], "addword") == 0)
         {
             if (argc > 3)
                 weechat_aspell_add_word (argv[2], argv_eol[3]);
@@ -933,7 +939,7 @@ weechat_aspell_completion_langs_cb (void *data, const char *completion_item,
     return WEECHAT_RC_OK;
 }
 
-/* 
+/*
  * weechat_plugin_init : init aspell plugin
  */
 
@@ -987,7 +993,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     return WEECHAT_RC_OK;
 }
 
-/* 
+/*
  * weechat_plugin_end : end aspell plugin
  */
 
