@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* irc-channel.c: manages a chat (channel or private chat) for IRC plugin */
+/*
+ * irc-channel.c: channel and private chat management for IRC plugin
+ */
 
 
 #include <stdlib.h>
@@ -29,6 +31,7 @@
 #include "irc.h"
 #include "irc-channel.h"
 #include "irc-buffer.h"
+#include "irc-command.h"
 #include "irc-config.h"
 #include "irc-nick.h"
 #include "irc-server.h"
@@ -184,26 +187,36 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
         buffer_created = 1;
     }
     
-    weechat_buffer_set (new_buffer, "short_name", channel_name);
-    weechat_buffer_set (new_buffer, "localvar_set_type",
-                        (channel_type == IRC_CHANNEL_TYPE_CHANNEL) ? "channel" : "private");
-    weechat_buffer_set (new_buffer, "localvar_set_nick", server->nick);
-    weechat_buffer_set (new_buffer, "localvar_set_server", server->name);
-    weechat_buffer_set (new_buffer, "localvar_set_channel", channel_name);
-    
     if (buffer_created)
     {
+        weechat_buffer_set (new_buffer, "short_name", channel_name);
+        weechat_buffer_set (new_buffer, "localvar_set_type",
+                            (channel_type == IRC_CHANNEL_TYPE_CHANNEL) ? "channel" : "private");
+        weechat_buffer_set (new_buffer, "localvar_set_nick", server->nick);
+        weechat_buffer_set (new_buffer, "localvar_set_server", server->name);
+        weechat_buffer_set (new_buffer, "localvar_set_channel", channel_name);
         weechat_hook_signal_send ("logger_backlog",
                                   WEECHAT_HOOK_SIGNAL_POINTER, new_buffer);
+        if (weechat_config_boolean (irc_config_network_send_unknown_commands))
+            weechat_buffer_set (new_buffer, "input_get_unknown_commands", "1");
+        if (channel_type == IRC_CHANNEL_TYPE_CHANNEL)
+        {
+            weechat_buffer_set (new_buffer, "nicklist", "1");
+            weechat_buffer_set (new_buffer, "nicklist_display_groups", "0");
+        }
+        
+        /* set highlights settings on channel buffer */
+        weechat_buffer_set(new_buffer, "highlight_words_add", "$nick");
+        if (weechat_config_string (irc_config_look_highlight_tags)
+            && weechat_config_string (irc_config_look_highlight_tags)[0])
+        {
+            weechat_buffer_set (new_buffer, "highlight_tags",
+                                weechat_config_string (irc_config_look_highlight_tags));
+        }
     }
     
-    if (weechat_config_boolean (irc_config_network_send_unknown_commands))
-        weechat_buffer_set (new_buffer, "input_get_unknown_commands", "1");
-    
     if (channel_type == IRC_CHANNEL_TYPE_CHANNEL)
     {
-        weechat_buffer_set (new_buffer, "nicklist", "1");
-        weechat_buffer_set (new_buffer, "nicklist_display_groups", "0");
         weechat_nicklist_add_group (new_buffer, NULL, IRC_NICK_GROUP_CHANOWNER,
                                     "weechat.color.nicklist_group", 1);
         weechat_nicklist_add_group (new_buffer, NULL, IRC_NICK_GROUP_CHANADMIN,
@@ -222,15 +235,6 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
                                     "weechat.color.nicklist_group", 1);
     }
     
-    /* set highlights settings on channel buffer */
-    weechat_buffer_set (new_buffer, "highlight_words", server->nick);
-    if (weechat_config_string (irc_config_look_highlight_tags)
-        && weechat_config_string (irc_config_look_highlight_tags)[0])
-    {
-        weechat_buffer_set (new_buffer, "highlight_tags",
-                            weechat_config_string (irc_config_look_highlight_tags));
-    }
-    
     /* initialize new channel */
     new_channel->type = channel_type;
     new_channel->name = strdup (channel_name);
@@ -243,6 +247,8 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
     new_channel->cycle = 0;
     new_channel->display_creation_date = 0;
     new_channel->nick_completion_reset = 0;
+    new_channel->pv_remote_nick_color = NULL;
+    new_channel->hook_autorejoin = NULL;
     new_channel->nicks_count = 0;
     new_channel->nicks = NULL;
     new_channel->last_nick = NULL;
@@ -326,7 +332,7 @@ irc_channel_is_channel (const char *string)
     
     first_char[0] = string[0];
     first_char[1] = '\0';
-    return (strpbrk (first_char, IRC_CHANNEL_PREFIX)) ? 1 : 0;    
+    return (strpbrk (first_char, IRC_CHANNEL_PREFIX)) ? 1 : 0;
 }
 
 /*
@@ -334,7 +340,8 @@ irc_channel_is_channel (const char *string)
  */
 
 void
-irc_channel_remove_away (struct t_irc_channel *channel)
+irc_channel_remove_away (struct t_irc_server *server,
+                         struct t_irc_channel *channel)
 {
     struct t_irc_nick *ptr_nick;
     
@@ -342,7 +349,7 @@ irc_channel_remove_away (struct t_irc_channel *channel)
     {
         for (ptr_nick = channel->nicks; ptr_nick; ptr_nick = ptr_nick->next_nick)
         {
-            irc_nick_set (channel, ptr_nick, 0, IRC_NICK_AWAY);
+            irc_nick_set (server, channel, ptr_nick, 0, IRC_NICK_AWAY);
         }
     }
 }
@@ -362,10 +369,11 @@ irc_channel_check_away (struct t_irc_server *server,
             || (channel->nicks_count <= weechat_config_integer (irc_config_network_away_check_max_nicks)))
         {
             channel->checking_away++;
-            irc_server_sendf (server, 1, "WHO %s", channel->name);
+            irc_server_sendf (server, IRC_SERVER_OUTQUEUE_PRIO_LOW,
+                              "WHO %s", channel->name);
         }
         else
-            irc_channel_remove_away (channel);
+            irc_channel_remove_away (server, channel);
     }
 }
 
@@ -374,7 +382,8 @@ irc_channel_check_away (struct t_irc_server *server,
  */
 
 void
-irc_channel_set_away (struct t_irc_channel *channel, const char *nick_name,
+irc_channel_set_away (struct t_irc_server *server,
+                      struct t_irc_channel *channel, const char *nick_name,
                       int is_away)
 {
     struct t_irc_nick *ptr_nick;
@@ -383,7 +392,7 @@ irc_channel_set_away (struct t_irc_channel *channel, const char *nick_name,
     {
         ptr_nick = irc_nick_search (channel, nick_name);
         if (ptr_nick)
-            irc_nick_set_away (channel, ptr_nick, is_away);
+            irc_nick_set_away (server, channel, ptr_nick, is_away);
     }
 }
 
@@ -614,6 +623,62 @@ irc_channel_nick_speaking_time_rename (struct t_irc_channel *channel,
     }
 }
 
+/*
+ * irc_channel_rejoin: rejoin a channel (for example after kick)
+ */
+
+void
+irc_channel_rejoin (struct t_irc_server *server, struct t_irc_channel *channel)
+{
+    char join_args[256];
+    
+    snprintf (join_args, sizeof (join_args), "%s%s%s",
+              channel->name,
+              (channel->key) ? " " : "",
+              (channel->key) ? channel->key : "");
+    
+    irc_command_join_server (server, join_args);
+}
+
+/*
+ * irc_channel_autorejoin_cb: callback for autorejoin on a channel
+ */
+
+int
+irc_channel_autorejoin_cb (void *data, int remaining_calls)
+{
+    struct t_irc_server *ptr_server, *ptr_server_found;
+    struct t_irc_channel *ptr_channel_arg, *ptr_channel;
+    
+    /* make C compiler happy */
+    (void) remaining_calls;
+    
+    ptr_channel_arg = (struct t_irc_channel *)data;
+    
+    ptr_server_found = NULL;
+    for (ptr_server = irc_servers; ptr_server;
+         ptr_server = ptr_server->next_server)
+    {
+        for (ptr_channel = ptr_server->channels; ptr_channel;
+             ptr_channel = ptr_channel->next_channel)
+        {
+            if (ptr_channel == ptr_channel_arg)
+            {
+                ptr_server_found = ptr_server;
+                break;
+            }
+        }
+    }
+    
+    if (ptr_server_found && (ptr_channel_arg->hook_autorejoin))
+    {
+        irc_channel_rejoin (ptr_server_found, ptr_channel_arg);
+        ptr_channel_arg->hook_autorejoin = NULL;
+    }
+    
+    return WEECHAT_RC_OK;
+}
+
 /*
  * irc_channel_free: free a channel and remove it from channels list
  */
@@ -652,6 +717,10 @@ irc_channel_free (struct t_irc_server *server, struct t_irc_channel *channel)
     irc_nick_free_all (channel);
     if (channel->away_message)
         free (channel->away_message);
+    if (channel->pv_remote_nick_color)
+        free (channel->pv_remote_nick_color);
+    if (channel->hook_autorejoin)
+        weechat_unhook(channel->hook_autorejoin);
     if (channel->nicks_speaking[0])
         weechat_list_free (channel->nicks_speaking[0]);
     if (channel->nicks_speaking[1])
@@ -722,6 +791,8 @@ irc_channel_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "key", channel->key))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "pv_remote_nick_color", channel->pv_remote_nick_color))
+        return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "nicks_count", channel->nicks_count))
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "checking_away", channel->checking_away))
@@ -799,6 +870,8 @@ irc_channel_print_log (struct t_irc_channel *channel)
     weechat_log_printf ("       cycle. . . . . . . . . . : %d",    channel->cycle);
     weechat_log_printf ("       display_creation_date. . : %d",    channel->display_creation_date);
     weechat_log_printf ("       nick_completion_reset. . : %d",    channel->nick_completion_reset);
+    weechat_log_printf ("       pv_remote_nick_color . . : '%s'",  channel->pv_remote_nick_color);
+    weechat_log_printf ("       hook_autorejoin. . . . . : 0x%lx", channel->hook_autorejoin);
     weechat_log_printf ("       nicks_count. . . . . . . : %d",    channel->nicks_count);
     weechat_log_printf ("       nicks. . . . . . . . . . : 0x%lx", channel->nicks);
     weechat_log_printf ("       last_nick. . . . . . . . : 0x%lx", channel->last_nick);
