@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* irc-completion.c: completion for IRC commands */
+/*
+ * irc-completion.c: completion for IRC commands
+ */
 
 
 #include <stdlib.h>
@@ -44,7 +46,7 @@ irc_completion_server_cb (void *data, const char *completion_item,
                           struct t_gui_buffer *buffer,
                           struct t_gui_completion *completion)
 {
-    IRC_GET_SERVER(buffer);
+    IRC_BUFFER_GET_SERVER(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -69,7 +71,7 @@ irc_completion_server_nick_cb (void *data, const char *completion_item,
                                struct t_gui_buffer *buffer,
                                struct t_gui_completion *completion)
 {
-    IRC_GET_SERVER(buffer);
+    IRC_BUFFER_GET_SERVER(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -96,7 +98,7 @@ irc_completion_server_channels_cb (void *data, const char *completion_item,
 {
     struct t_irc_channel *ptr_channel;
     
-    IRC_GET_SERVER(buffer);
+    IRC_BUFFER_GET_SERVER(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -131,7 +133,7 @@ irc_completion_server_privates_cb (void *data, const char *completion_item,
 {
     struct t_irc_channel *ptr_channel;
     
-    IRC_GET_SERVER(buffer);
+    IRC_BUFFER_GET_SERVER(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -167,7 +169,7 @@ irc_completion_server_nicks_cb (void *data, const char *completion_item,
     struct t_irc_channel *ptr_channel2;
     struct t_irc_nick *ptr_nick;
     
-    IRC_GET_SERVER_CHANNEL(buffer);
+    IRC_BUFFER_GET_SERVER_CHANNEL(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -232,7 +234,7 @@ irc_completion_channel_cb (void *data, const char *completion_item,
                            struct t_gui_buffer *buffer,
                            struct t_gui_completion *completion)
 {
-    IRC_GET_SERVER_CHANNEL(buffer);
+    IRC_BUFFER_GET_SERVER_CHANNEL(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -289,7 +291,7 @@ irc_completion_channel_nicks_cb (void *data, const char *completion_item,
 {
     struct t_irc_nick *ptr_nick;
     
-    IRC_GET_SERVER_CHANNEL(buffer);
+    IRC_BUFFER_GET_SERVER_CHANNEL(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -357,7 +359,7 @@ irc_completion_channel_nicks_hosts_cb (void *data, const char *completion_item,
     char *buf;
     int length;
     
-    IRC_GET_SERVER_CHANNEL(buffer);
+    IRC_BUFFER_GET_SERVER_CHANNEL(buffer);
     
     /* make C compiler happy */
     (void) data;
@@ -417,7 +419,7 @@ irc_completion_channel_topic_cb (void *data, const char *completion_item,
 {
     char *topic_color;
     
-    IRC_GET_SERVER_CHANNEL(buffer);
+    IRC_BUFFER_GET_SERVER_CHANNEL(buffer);
     
     /* make C compiler happy */
     (void) data;
