@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* irc-config.c: IRC configuration options */
+/*
+ * irc-config.c: IRC configuration options (file irc.conf)
+ */
 
 
 #include <stdlib.h>
@@ -29,20 +31,25 @@
 #include "../weechat-plugin.h"
 #include "irc.h"
 #include "irc-config.h"
+#include "irc-ctcp.h"
 #include "irc-buffer.h"
 #include "irc-ignore.h"
+#include "irc-msgbuffer.h"
 #include "irc-nick.h"
 #include "irc-server.h"
 #include "irc-channel.h"
 
 
 struct t_config_file *irc_config_file = NULL;
+struct t_config_section *irc_config_section_msgbuffer = NULL;
+struct t_config_section *irc_config_section_ctcp = NULL;
 struct t_config_section *irc_config_section_server_default = NULL;
 struct t_config_section *irc_config_section_server = NULL;
 
 /* IRC config, look section */
 
 struct t_config_option *irc_config_look_color_nicks_in_server_messages;
+struct t_config_option *irc_config_look_color_pv_nick_like_channel;
 struct t_config_option *irc_config_look_server_buffer;
 struct t_config_option *irc_config_look_open_channel_near_server;
 struct t_config_option *irc_config_look_open_pv_near_server;
@@ -50,17 +57,26 @@ struct t_config_option *irc_config_look_nick_prefix;
 struct t_config_option *irc_config_look_nick_suffix;
 struct t_config_option *irc_config_look_nick_completion_smart;
 struct t_config_option *irc_config_look_display_away;
-struct t_config_option *irc_config_look_display_channel_modes;
-struct t_config_option *irc_config_look_display_nick_modes;
+struct t_config_option *irc_config_look_display_ctcp_blocked;
+struct t_config_option *irc_config_look_display_ctcp_reply;
+struct t_config_option *irc_config_look_display_ctcp_unknown;
 struct t_config_option *irc_config_look_display_old_topic;
+struct t_config_option *irc_config_look_item_channel_modes;
+struct t_config_option *irc_config_look_item_channel_modes_hide_key;
+struct t_config_option *irc_config_look_item_nick_modes;
+struct t_config_option *irc_config_look_item_nick_prefix;
 struct t_config_option *irc_config_look_hide_nickserv_pwd;
 struct t_config_option *irc_config_look_highlight_tags;
 struct t_config_option *irc_config_look_item_display_server;
+struct t_config_option *irc_config_look_msgbuffer_fallback;
 struct t_config_option *irc_config_look_notice_as_pv;
+struct t_config_option *irc_config_look_part_closes_buffer;
 struct t_config_option *irc_config_look_raw_messages;
 struct t_config_option *irc_config_look_show_away_once;
 struct t_config_option *irc_config_look_smart_filter;
 struct t_config_option *irc_config_look_smart_filter_delay;
+struct t_config_option *irc_config_look_smart_filter_join;
+struct t_config_option *irc_config_look_smart_filter_quit;
 struct t_config_option *irc_config_look_topic_strip_colors;
 
 /* IRC config, color section */
@@ -74,6 +90,7 @@ struct t_config_option *irc_config_color_item_channel_modes;
 
 /* IRC config, network section */
 
+struct t_config_option *irc_config_network_connection_timeout;
 struct t_config_option *irc_config_network_default_msg_part;
 struct t_config_option *irc_config_network_default_msg_quit;
 struct t_config_option *irc_config_network_away_check;
@@ -81,7 +98,7 @@ struct t_config_option *irc_config_network_away_check_max_nicks;
 struct t_config_option *irc_config_network_lag_check;
 struct t_config_option *irc_config_network_lag_min_show;
 struct t_config_option *irc_config_network_lag_disconnect;
-struct t_config_option *irc_config_network_anti_flood;
+struct t_config_option *irc_config_network_anti_flood[2];
 struct t_config_option *irc_config_network_colors_receive;
 struct t_config_option *irc_config_network_colors_send;
 struct t_config_option *irc_config_network_send_unknown_commands;
@@ -148,7 +165,7 @@ irc_config_change_look_color_nicks_number (void *data, const char *option,
             for (ptr_nick = ptr_channel->nicks; ptr_nick;
                  ptr_nick = ptr_nick->next_nick)
             {
-                ptr_nick->color = irc_nick_find_color (ptr_nick);
+                ptr_nick->color = irc_nick_find_color (ptr_nick->name);
             }
         }
     }
@@ -185,7 +202,7 @@ irc_config_change_look_server_buffer (void *data,
          IRC_CONFIG_LOOK_SERVER_BUFFER_MERGE_WITH_CORE)
         || (weechat_config_integer (irc_config_look_server_buffer) ==
             IRC_CONFIG_LOOK_SERVER_BUFFER_MERGE_WITHOUT_CORE))
-    {    
+    {
         ptr_buffer =
             (weechat_config_integer (irc_config_look_server_buffer) ==
              IRC_CONFIG_LOOK_SERVER_BUFFER_MERGE_WITH_CORE) ?
@@ -204,14 +221,13 @@ irc_config_change_look_server_buffer (void *data,
 }
 
 /*
- * irc_config_change_look_display_channel_modes: called when the "display
- *                                               channel modes" option is
- *                                               changed
+ * irc_config_change_look_item_channel_modes: called when the "display
+ *                                            channel modes" option is changed
  */
 
 void
-irc_config_change_look_display_channel_modes (void *data,
-                                              struct t_config_option *option)
+irc_config_change_look_item_channel_modes (void *data,
+                                           struct t_config_option *option)
 {
     /* make C compiler happy */
     (void) data;
@@ -221,13 +237,47 @@ irc_config_change_look_display_channel_modes (void *data,
 }
 
 /*
- * irc_config_change_look_display_nick_modes: called when the "display
- *                                            nick modes" option is changed
+ * irc_config_change_look_item_channel_modes_hide_key: called when the
+ *                                                     "display channel modes
+ *                                                     hide key" option is
+ *                                                     changed
  */
 
 void
-irc_config_change_look_display_nick_modes (void *data,
-                                           struct t_config_option *option)
+irc_config_change_look_item_channel_modes_hide_key (void *data,
+                                                    struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) option;
+    
+    weechat_bar_item_update ("buffer_name");
+}
+
+/*
+ * irc_config_change_look_item_nick_modes: called when the "display nick modes"
+ *                                         option is changed
+ */
+
+void
+irc_config_change_look_item_nick_modes (void *data,
+                                        struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) option;
+    
+    weechat_bar_item_update ("input_prompt");
+}
+
+/*
+ * irc_config_change_look_item_nick_prefix: called when the "display nick
+ *                                          prefix" option is changed
+ */
+
+void
+irc_config_change_look_item_nick_prefix (void *data,
+                                         struct t_config_option *option)
 {
     /* make C compiler happy */
     (void) data;
@@ -552,15 +602,160 @@ irc_config_reload (void *data, struct t_config_file *config_file)
 }
 
 /*
- * irc_config_ignore_read: read ignore option from configuration file
- *                         return 1 if ok, 0 if error
+ * irc_config_msgbuffer_create_option: set a message target buffer
  */
 
 int
-irc_config_ignore_read (void *data,
-                        struct t_config_file *config_file,
-                        struct t_config_section *section,
-                        const char *option_name, const char *value)
+irc_config_msgbuffer_create_option (void *data,
+                                    struct t_config_file *config_file,
+                                    struct t_config_section *section,
+                                    const char *option_name, const char *value)
+{
+    struct t_config_option *ptr_option;
+    int rc;
+    const char *pos_name;
+    
+    /* make C compiler happy */
+    (void) data;
+    
+    rc = WEECHAT_CONFIG_OPTION_SET_ERROR;
+    
+    if (option_name)
+    {
+        ptr_option = weechat_config_search_option (config_file, section,
+                                                   option_name);
+        if (ptr_option)
+        {
+            if (value)
+                rc = weechat_config_option_set (ptr_option, value, 1);
+            else
+            {
+                weechat_config_option_free (ptr_option);
+                rc = WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+            }
+        }
+        else
+        {
+            if (value)
+            {
+                pos_name = strchr (option_name, '.');
+                pos_name = (pos_name) ? pos_name + 1 : option_name;
+                
+                ptr_option = weechat_config_new_option (
+                    config_file, section,
+                    option_name, "integer",
+                    _("buffer used to display message received from IRC "
+                      "server"),
+                    "weechat|current|private", 0, 0, value, value, 0,
+                    NULL, NULL, NULL, NULL, NULL, NULL);
+                rc = (ptr_option) ?
+                    WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE : WEECHAT_CONFIG_OPTION_SET_ERROR;
+            }
+            else
+                rc = WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+        }
+    }
+    
+    if (rc == WEECHAT_CONFIG_OPTION_SET_ERROR)
+    {
+        weechat_printf (NULL,
+                        _("%s%s: error creating \"%s\" => \"%s\""),
+                        weechat_prefix ("error"), IRC_PLUGIN_NAME,
+                        option_name, value);
+    }
+    
+    return rc;
+}
+
+/*
+ * irc_config_ctcp_create_option: set a ctcp reply format
+ */
+
+int
+irc_config_ctcp_create_option (void *data, struct t_config_file *config_file,
+                               struct t_config_section *section,
+                               const char *option_name, const char *value)
+{
+    struct t_config_option *ptr_option;
+    int rc;
+    const char *default_value;
+    static char empty_value[1] = { '\0' };
+    const char *pos_name;
+    
+    /* make C compiler happy */
+    (void) data;
+    
+    rc = WEECHAT_CONFIG_OPTION_SET_ERROR;
+    
+    if (option_name)
+    {
+        ptr_option = weechat_config_search_option (config_file, section,
+                                                   option_name);
+        if (ptr_option)
+        {
+            if (value)
+                rc = weechat_config_option_set (ptr_option, value, 1);
+            else
+            {
+                weechat_config_option_free (ptr_option);
+                rc = WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+            }
+        }
+        else
+        {
+            if (value)
+            {
+                pos_name = strchr (option_name, '.');
+                pos_name = (pos_name) ? pos_name + 1 : option_name;
+                
+                default_value = irc_ctcp_get_default_reply (pos_name);
+                if (!default_value)
+                    default_value = empty_value;
+                
+                ptr_option = weechat_config_new_option (
+                    config_file, section,
+                    option_name, "string",
+                    _("format for CTCP reply or empty string for blocking "
+                      "CTCP (no reply), following variables are replaced: "
+                      "$version (WeeChat version), "
+                      "$compilation (compilation date), "
+                      "$osinfo (info about OS), "
+                      "$site (WeeChat site), "
+                      "$download (WeeChat site, download page), "
+                      "$time (current date and time as text), "
+                      "$username (username on server), "
+                      "$realname (realname on server)"),
+                    NULL, 0, 0, default_value, value, 0,
+                    NULL, NULL, NULL, NULL, NULL, NULL);
+                rc = (ptr_option) ?
+                    WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE : WEECHAT_CONFIG_OPTION_SET_ERROR;
+            }
+            else
+                rc = WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+        }
+    }
+    
+    if (rc == WEECHAT_CONFIG_OPTION_SET_ERROR)
+    {
+        weechat_printf (NULL,
+                        _("%s%s: error creating CTCP \"%s\" => \"%s\""),
+                        weechat_prefix ("error"), IRC_PLUGIN_NAME,
+                        option_name, value);
+    }
+    
+    return rc;
+}
+
+/*
+ * irc_config_ignore_read_cb: read ignore option from configuration file
+ *                            return 1 if ok, 0 if error
+ */
+
+int
+irc_config_ignore_read_cb (void *data,
+                           struct t_config_file *config_file,
+                           struct t_config_section *section,
+                           const char *option_name, const char *value)
 {
     char **argv, **argv_eol;
     int argc;
@@ -591,30 +786,34 @@ irc_config_ignore_read (void *data,
 }
 
 /*
- * irc_config_ignore_write: write ignore section in configuration file
+ * irc_config_ignore_write_cb: write ignore section in configuration file
  */
 
-void
-irc_config_ignore_write (void *data, struct t_config_file *config_file,
-                         const char *section_name)
+int
+irc_config_ignore_write_cb (void *data, struct t_config_file *config_file,
+                            const char *section_name)
 {
     struct t_irc_ignore *ptr_ignore;
     
     /* make C compiler happy */
     (void) data;
     
-    weechat_config_write_line (config_file, section_name, NULL);
+    if (!weechat_config_write_line (config_file, section_name, NULL))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     for (ptr_ignore = irc_ignore_list; ptr_ignore;
          ptr_ignore = ptr_ignore->next_ignore)
     {
-        weechat_config_write_line (config_file,
-                                   "ignore",
-                                   "%s;%s;%s",
-                                   (ptr_ignore->server) ? ptr_ignore->server : "*",
-                                   (ptr_ignore->channel) ? ptr_ignore->channel : "*",
-                                   ptr_ignore->mask);
+        if (!weechat_config_write_line (config_file,
+                                        "ignore",
+                                        "%s;%s;%s",
+                                        (ptr_ignore->server) ? ptr_ignore->server : "*",
+                                        (ptr_ignore->channel) ? ptr_ignore->channel : "*",
+                                        ptr_ignore->mask))
+            return WEECHAT_CONFIG_WRITE_ERROR;
     }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
@@ -622,7 +821,7 @@ irc_config_ignore_write (void *data, struct t_config_file *config_file,
  *                                     configuration file
  */
 
-void
+int
 irc_config_server_write_default_cb (void *data,
                                     struct t_config_file *config_file,
                                     const char *section_name)
@@ -633,7 +832,8 @@ irc_config_server_write_default_cb (void *data,
     /* make C compiler happy */
     (void) data;
     
-    weechat_config_write_line (config_file, section_name, NULL);
+    if (!weechat_config_write_line (config_file, section_name, NULL))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     for (i = 0; i < IRC_SERVER_NUM_OPTIONS; i++)
     {
@@ -643,17 +843,21 @@ irc_config_server_write_default_cb (void *data,
         switch (i)
         {
             case IRC_SERVER_OPTION_ADDRESSES:
-                weechat_config_write_line (config_file,
-                                           option_name,
-                                           "%s", "\"chat.freenode.net/6667\"");
+                if (!weechat_config_write_line (config_file,
+                                                option_name,
+                                                "%s", "\"chat.freenode.net/6667\""))
+                    return WEECHAT_CONFIG_WRITE_ERROR;
                 break;
             default:
-                weechat_config_write_line (config_file,
-                                           option_name,
-                                           WEECHAT_CONFIG_OPTION_NULL);
+                if (!weechat_config_write_line (config_file,
+                                                option_name,
+                                                WEECHAT_CONFIG_OPTION_NULL))
+                    return WEECHAT_CONFIG_WRITE_ERROR;
                 break;
         }
     }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
@@ -725,6 +929,42 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
+        case IRC_SERVER_OPTION_SSL_CERT:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "string",
+                N_("ssl certificate file used to automatically identify your nick"),
+                NULL, 0, 0,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
+        case IRC_SERVER_OPTION_SSL_DHKEY_SIZE:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "integer",
+                N_("size of the key used during the Diffie-Hellman Key Exchange"),
+                NULL, 0, INT_MAX,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
+        case IRC_SERVER_OPTION_SSL_VERIFY:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "boolean",
+                N_("check that the ssl connection is fully trusted"),
+                NULL, 0, 0,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
         case IRC_SERVER_OPTION_PASSWORD:
             new_option = weechat_config_new_option (
                 config_file, section,
@@ -737,6 +977,55 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
+        case IRC_SERVER_OPTION_SASL_MECHANISM:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "integer",
+                N_("mechanism for SASL authentication"),
+                "plain|dh-blowfish", 0, 0,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
+        case IRC_SERVER_OPTION_SASL_USERNAME:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "string",
+                N_("username for SASL authentication"),
+                NULL, 0, 0,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
+        case IRC_SERVER_OPTION_SASL_PASSWORD:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "string",
+                N_("password for SASL authentication"),
+                NULL, 0, 0,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
+        case IRC_SERVER_OPTION_SASL_TIMEOUT:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "integer",
+                N_("timeout (in seconds) before giving up SASL "
+                   "authentication"),
+                NULL, 1, 3600,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
         case IRC_SERVER_OPTION_AUTOCONNECT:
             new_option = weechat_config_new_option (
                 config_file, section,
@@ -867,7 +1156,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
             new_option = weechat_config_new_option (
                 config_file, section,
                 option_name, "boolean",
-                N_("automatically rejoin channels when kicked"),
+                N_("automatically rejoin channels after kick"),
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
@@ -875,6 +1164,18 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
+        case IRC_SERVER_OPTION_AUTOREJOIN_DELAY:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "integer",
+                N_("delay (in seconds) before autorejoin (after kick)"),
+                NULL, 0, 3600*24,
+                default_value, value,
+                null_value_allowed,
+                NULL, NULL,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
         case IRC_SERVER_NUM_OPTIONS:
             break;
     }
@@ -962,7 +1263,7 @@ irc_config_server_read_cb (void *data, struct t_config_file *config_file,
  * irc_config_server_write_cb: write server section in configuration file
  */
 
-void
+int
 irc_config_server_write_cb (void *data, struct t_config_file *config_file,
                             const char *section_name)
 {
@@ -972,7 +1273,8 @@ irc_config_server_write_cb (void *data, struct t_config_file *config_file,
     /* make C compiler happy */
     (void) data;
     
-    weechat_config_write_line (config_file, section_name, NULL);
+    if (!weechat_config_write_line (config_file, section_name, NULL))
+        return WEECHAT_CONFIG_WRITE_ERROR;
     
     for (ptr_server = irc_servers; ptr_server;
          ptr_server = ptr_server->next_server)
@@ -981,11 +1283,14 @@ irc_config_server_write_cb (void *data, struct t_config_file *config_file,
         {
             for (i = 0; i < IRC_SERVER_NUM_OPTIONS; i++)
             {
-                weechat_config_write_option (config_file,
-                                             ptr_server->options[i]);
+                if (!weechat_config_write_option (config_file,
+                                                  ptr_server->options[i]))
+                    return WEECHAT_CONFIG_WRITE_ERROR;
             }
         }
     }
+    
+    return WEECHAT_CONFIG_WRITE_OK;
 }
 
 /*
@@ -1110,6 +1415,12 @@ irc_config_init ()
         N_("use nick color in messages from server"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL,
         NULL, NULL, NULL, NULL);
+    irc_config_look_color_pv_nick_like_channel = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "color_pv_nick_like_channel", "boolean",
+        N_("use same nick color for channel and private"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
     irc_config_look_server_buffer = weechat_config_new_option (
         irc_config_file, ptr_section,
         "server_buffer", "integer",
@@ -1149,24 +1460,56 @@ irc_config_init ()
         N_("display message when (un)marking as away"),
         "off|local|channel", 0, 0, "local", NULL, 0, NULL, NULL, NULL, NULL,
         NULL, NULL);
-    irc_config_look_display_channel_modes = weechat_config_new_option (
+    irc_config_look_display_ctcp_blocked = weechat_config_new_option (
         irc_config_file, ptr_section,
-        "display_channel_modes", "boolean",
-        N_("display channel modes in \"buffer_name\" bar item"),
+        "display_ctcp_blocked", "boolean",
+        N_("display CTCP message even if it is blocked"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL,
-        &irc_config_change_look_display_channel_modes, NULL, NULL, NULL);
-    irc_config_look_display_nick_modes = weechat_config_new_option (
+        NULL, NULL, NULL, NULL);
+    irc_config_look_display_ctcp_reply = weechat_config_new_option (
         irc_config_file, ptr_section,
-        "display_nick_modes", "boolean",
-        N_("display nick modes in \"input_prompt\" bar item"),
+        "display_ctcp_reply", "boolean",
+        N_("display CTCP reply sent by WeeChat"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL,
-        &irc_config_change_look_display_nick_modes, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL);
+    irc_config_look_display_ctcp_unknown = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "display_ctcp_unknown", "boolean",
+        N_("display CTCP message even if it is unknown CTCP"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
     irc_config_look_display_old_topic = weechat_config_new_option (
         irc_config_file, ptr_section,
         "display_old_topic", "boolean",
         N_("display old topic when channel topic is changed"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL,
         NULL, NULL, NULL, NULL);
+    irc_config_look_item_channel_modes = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "item_channel_modes", "boolean",
+        N_("display channel modes in \"buffer_name\" bar item"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        &irc_config_change_look_item_channel_modes, NULL, NULL, NULL);
+    irc_config_look_item_channel_modes_hide_key = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "item_channel_modes_hide_key", "boolean",
+        N_("hide channel key if modes are displayed in \"buffer_name\" bar "
+           "item (this will hide all channel modes arguments if mode +k is "
+           "set on channel)"),
+        NULL, 0, 0, "off", NULL, 0, NULL, NULL,
+        &irc_config_change_look_item_channel_modes_hide_key, NULL, NULL, NULL);
+    irc_config_look_item_nick_modes = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "item_nick_modes", "boolean",
+        N_("display nick modes in \"input_prompt\" bar item"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        &irc_config_change_look_item_nick_modes, NULL, NULL, NULL);
+    irc_config_look_item_nick_prefix = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "item_nick_prefix", "boolean",
+        N_("display nick prefix in \"input_prompt\" bar item"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        &irc_config_change_look_item_nick_prefix, NULL, NULL, NULL);
     irc_config_look_hide_nickserv_pwd = weechat_config_new_option (
         irc_config_file, ptr_section,
         "hide_nickserv_pwd", "boolean",
@@ -1186,6 +1529,13 @@ irc_config_init ()
         N_("name of bar item where IRC server is displayed (for status bar)"),
         "buffer_plugin|buffer_name", 0, 0, "buffer_plugin", NULL, 0, NULL, NULL,
         &irc_config_change_look_item_display_server, NULL, NULL, NULL);
+    irc_config_look_msgbuffer_fallback = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "msgbuffer_fallback", "integer",
+        N_("default target buffer for msgbuffer options when target is "
+           "private and that private buffer is not found"),
+        "current|server", 0, 0, "current", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
     irc_config_look_raw_messages = weechat_config_new_option (
         irc_config_file, ptr_section,
         "raw_messages", "integer",
@@ -1203,18 +1553,33 @@ irc_config_init ()
         N_("filter join/part/quit messages for a nick if not speaking for "
            "some minutes on channel (you must create a filter on tag "
            "\"irc_smart_filter\")"),
-        NULL, 0, 0, "off", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_look_smart_filter_delay = weechat_config_new_option (
         irc_config_file, ptr_section,
         "smart_filter_delay", "integer",
         N_("delay for filtering join/part/quit messages (in minutes)"),
         NULL, 1, 60*24*7, "5", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    irc_config_look_smart_filter_join = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "smart_filter_join", "boolean",
+        N_("enable smart filter for \"join\" messages"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    irc_config_look_smart_filter_quit = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "smart_filter_quit", "boolean",
+        N_("enable smart filter for \"part\" and \"quit\" messages"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_look_notice_as_pv = weechat_config_new_option (
         irc_config_file, ptr_section,
         "notice_as_pv", "integer",
         N_("display notices as private messages (if auto, use private buffer "
            "if found)"),
         "auto|never|always", 0, 0, "auto", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    irc_config_look_part_closes_buffer = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "part_closes_buffer", "boolean",
+        N_("close buffer when /part is issued on a channel"),
+        NULL, 0, 0, "off", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_look_topic_strip_colors = weechat_config_new_option (
         irc_config_file, ptr_section,
         "topic_strip_colors", "boolean",
@@ -1283,6 +1648,14 @@ irc_config_init ()
         return 0;
     }
     
+    irc_config_network_connection_timeout = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "connection_timeout", "integer",
+        N_("timeout (in seconds) between TCP connection to server and message "
+           "001 received, if this timeout is reached before 001 message is "
+           "received, WeeChat will disconnect from server"),
+        NULL, 1, 3600, "60", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
     irc_config_network_default_msg_part = weechat_config_new_option (
         irc_config_file, ptr_section,
         "default_msg_part", "string",
@@ -1307,7 +1680,7 @@ irc_config_init ()
         "away_check_max_nicks", "integer",
         N_("do not check away nicks on channels with high number of nicks "
            "(0 = unlimited)"),
-        NULL, 0, INT_MAX, "0", NULL, 0, NULL, NULL,
+        NULL, 0, INT_MAX, "25", NULL, 0, NULL, NULL,
         &irc_config_change_network_away_check, NULL, NULL, NULL);
     irc_config_network_lag_check = weechat_config_new_option (
         irc_config_file, ptr_section,
@@ -1326,12 +1699,19 @@ irc_config_init ()
         N_("disconnect after important lag (in minutes, 0 = never "
            "disconnect)"),
         NULL, 0, INT_MAX, "0", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
-    irc_config_network_anti_flood = weechat_config_new_option (
+    irc_config_network_anti_flood[0] = weechat_config_new_option (
         irc_config_file, ptr_section,
-        "anti_flood", "integer",
-        N_("anti-flood: # seconds between two user messages (0 = no "
-           "anti-flood)"),
-        NULL, 0, 5, "2", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+        "anti_flood_prio_high", "integer",
+        N_("anti-flood for high priority queue: number of seconds between two "
+           "user messages or commands sent to IRC server (0 = no anti-flood)"),
+        NULL, 0, 60, "2", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    irc_config_network_anti_flood[1] = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "anti_flood_prio_low", "integer",
+        N_("anti-flood for low priority queue: number of seconds between two "
+           "messages sent to IRC server (messages like automatic CTCP replies) "
+           "(0 = no anti-flood)"),
+        NULL, 0, 60, "2", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_network_colors_receive = weechat_config_new_option (
         irc_config_file, ptr_section,
         "colors_receive", "boolean",
@@ -1351,12 +1731,40 @@ irc_config_init ()
         NULL, 0, 0, "off", NULL, 0, NULL, NULL,
         &irc_config_change_network_send_unknown_commands, NULL, NULL, NULL);
     
+    /* msgbuffer */
+    ptr_section = weechat_config_new_section (irc_config_file, "msgbuffer",
+                                              1, 1,
+                                              NULL, NULL, NULL, NULL,
+                                              NULL, NULL,
+                                              &irc_config_msgbuffer_create_option, NULL,
+                                              NULL, NULL);
+    if (!ptr_section)
+    {
+        weechat_config_free (irc_config_file);
+        return 0;
+    }
+    irc_config_section_msgbuffer = ptr_section;
+    
+    /* CTCP */
+    ptr_section = weechat_config_new_section (irc_config_file, "ctcp",
+                                              1, 1,
+                                              NULL, NULL, NULL, NULL,
+                                              NULL, NULL,
+                                              &irc_config_ctcp_create_option, NULL,
+                                              NULL, NULL);
+    if (!ptr_section)
+    {
+        weechat_config_free (irc_config_file);
+        return 0;
+    }
+    irc_config_section_ctcp = ptr_section;
+    
     /* ignore */
     ptr_section = weechat_config_new_section (irc_config_file, "ignore",
                                               0, 0,
-                                              &irc_config_ignore_read, NULL,
-                                              &irc_config_ignore_write, NULL,
-                                              &irc_config_ignore_write, NULL,
+                                              &irc_config_ignore_read_cb, NULL,
+                                              &irc_config_ignore_write_cb, NULL,
+                                              &irc_config_ignore_write_cb, NULL,
                                               NULL, NULL, NULL, NULL);
     if (!ptr_section)
     {
@@ -1375,7 +1783,6 @@ irc_config_init ()
         weechat_config_free (irc_config_file);
         return 0;
     }
-    
     irc_config_section_server_default = ptr_section;
     
     irc_config_server_create_default_options (ptr_section);
@@ -1393,7 +1800,6 @@ irc_config_init ()
         weechat_config_free (irc_config_file);
         return 0;
     }
-    
     irc_config_section_server = ptr_section;
     
     hook_config_color_nicks_number = weechat_hook_config ("weechat.look.color_nicks_number",
