@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* irc-display.c: display functions for IRC plugin */
+/*
+ * irc-display.c: display functions for IRC plugin
+ */
 
 
 #include <stdlib.h>
@@ -29,8 +31,9 @@
 #include "irc-channel.h"
 #include "irc-command.h"
 #include "irc-config.h"
-#include "irc-server.h"
 #include "irc-nick.h"
+#include "irc-server.h"
+#include "irc-sasl.h"
 
 
 /*
@@ -172,7 +175,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                         IRC_COLOR_CHAT_DELIMITERS,
                         IRC_COLOR_CHAT,
                         (server->temp_server) ? _(" (temporary)") : "");
-        
+        /* addresses */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_ADDRESSES]))
             weechat_printf (NULL, "  addresses. . . . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_ADDRESSES));
@@ -180,6 +183,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  addresses. . . . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_ADDRESSES]));
+        /* proxy */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_PROXY]))
             weechat_printf (NULL, "  proxy. . . . . . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_PROXY));
@@ -187,6 +191,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  proxy. . . . . . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_PROXY]));
+        /* ipv6 */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_IPV6]))
             weechat_printf (NULL, "  ipv6 . . . . . . . . :   (%s)",
                             (IRC_SERVER_OPTION_BOOLEAN(server, IRC_SERVER_OPTION_IPV6)) ?
@@ -196,6 +201,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_boolean (server->options[IRC_SERVER_OPTION_IPV6]) ?
                             _("on") : _("off"));
+        /* ssl */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SSL]))
             weechat_printf (NULL, "  ssl. . . . . . . . . :   (%s)",
                             (IRC_SERVER_OPTION_BOOLEAN(server, IRC_SERVER_OPTION_SSL)) ?
@@ -205,6 +211,33 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_boolean (server->options[IRC_SERVER_OPTION_SSL]) ?
                             _("on") : _("off"));
+        /* ssl_cert */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SSL_CERT]))
+            weechat_printf (NULL, "  ssl_cert . . . . . . :   ('%s')",
+                            IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_SSL_CERT));
+        else
+            weechat_printf (NULL, "  ssl_cert . . . . . . : %s'%s'",
+                            IRC_COLOR_CHAT_VALUE,
+                            weechat_config_string (server->options[IRC_SERVER_OPTION_SSL_CERT]));
+        /* ssl_dhkey_size */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SSL_DHKEY_SIZE]))
+            weechat_printf (NULL, "  ssl_dhkey_size . . . :   (%d)",
+                            IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_SSL_DHKEY_SIZE));
+        else
+            weechat_printf (NULL, "  ssl_dhkey_size . . . : %s%d",
+                            IRC_COLOR_CHAT_VALUE,
+                            weechat_config_integer (server->options[IRC_SERVER_OPTION_SSL_DHKEY_SIZE]));
+        /* ssl_verify */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SSL_VERIFY]))
+            weechat_printf (NULL, "  ssl_verify . . . . . :   (%s)",
+                            (IRC_SERVER_OPTION_BOOLEAN(server, IRC_SERVER_OPTION_SSL_VERIFY)) ?
+                            _("on") : _("off"));
+        else
+            weechat_printf (NULL, "  ssl_verify . . . . . : %s%s",
+                            IRC_COLOR_CHAT_VALUE,
+                            weechat_config_boolean (server->options[IRC_SERVER_OPTION_SSL_VERIFY]) ?
+                            _("on") : _("off"));
+        /* password */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_PASSWORD]))
             weechat_printf (NULL, "  password . . . . . . :   %s",
                             _("(hidden)"));
@@ -212,6 +245,31 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  password . . . . . . : %s%s",
                             IRC_COLOR_CHAT_VALUE,
                             _("(hidden)"));
+        /* sasl_mechanism */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SASL_MECHANISM]))
+            weechat_printf (NULL, "  sasl_mechanism . . . :   ('%s')",
+                            irc_sasl_mechanism_string[IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_SASL_MECHANISM)]);
+        else
+            weechat_printf (NULL, "  sasl_mechanism . . . : %s'%s'",
+                            IRC_COLOR_CHAT_VALUE,
+                            irc_sasl_mechanism_string[weechat_config_integer (server->options[IRC_SERVER_OPTION_SASL_MECHANISM])]);
+        /* sasl_username */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SASL_USERNAME]))
+            weechat_printf (NULL, "  sasl_username. . . . :   ('%s')",
+                            IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_SASL_USERNAME));
+        else
+            weechat_printf (NULL, "  sasl_username. . . . : %s'%s'",
+                            IRC_COLOR_CHAT_VALUE,
+                            weechat_config_string (server->options[IRC_SERVER_OPTION_SASL_USERNAME]));
+        /* sasl_password */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_SASL_PASSWORD]))
+            weechat_printf (NULL, "  sasl_password. . . . :   %s",
+                            _("(hidden)"));
+        else
+            weechat_printf (NULL, "  sasl_password. . . . : %s%s",
+                            IRC_COLOR_CHAT_VALUE,
+                            _("(hidden)"));
+        /* autoconnect */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_AUTOCONNECT]))
             weechat_printf (NULL, "  autoconnect. . . . . :   (%s)",
                             (IRC_SERVER_OPTION_BOOLEAN(server, IRC_SERVER_OPTION_AUTOCONNECT)) ?
@@ -221,6 +279,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_boolean (server->options[IRC_SERVER_OPTION_AUTOCONNECT]) ?
                             _("on") : _("off"));
+        /* autoreconnect */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_AUTORECONNECT]))
             weechat_printf (NULL, "  autoreconnect. . . . :   (%s)",
                             (IRC_SERVER_OPTION_BOOLEAN(server, IRC_SERVER_OPTION_AUTORECONNECT)) ?
@@ -230,6 +289,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_boolean (server->options[IRC_SERVER_OPTION_AUTORECONNECT]) ?
                             _("on") : _("off"));
+        /* autoreconnect_delay */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_AUTORECONNECT_DELAY]))
             weechat_printf (NULL, "  autoreconnect_delay. :   (%d %s)",
                             IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AUTORECONNECT_DELAY),
@@ -239,6 +299,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_integer (server->options[IRC_SERVER_OPTION_AUTORECONNECT_DELAY]),
                             NG_("second", "seconds", weechat_config_integer (server->options[IRC_SERVER_OPTION_AUTORECONNECT_DELAY])));
+        /* nicks */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_NICKS]))
             weechat_printf (NULL, "  nicks. . . . . . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_NICKS));
@@ -246,6 +307,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  nicks. . . . . . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_NICKS]));
+        /* username */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_USERNAME]))
             weechat_printf (NULL, "  username . . . . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_USERNAME));
@@ -253,6 +315,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  username . . . . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_USERNAME]));
+        /* realname */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_REALNAME]))
             weechat_printf (NULL, "  realname . . . . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_REALNAME));
@@ -260,6 +323,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  realname . . . . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_REALNAME]));
+        /* local_hostname */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_LOCAL_HOSTNAME]))
             weechat_printf (NULL, "  local_hostname . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_LOCAL_HOSTNAME));
@@ -267,6 +331,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  local_hostname . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_LOCAL_HOSTNAME]));
+        /* command */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_COMMAND]))
         {
             string = strdup (IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_COMMAND));
@@ -288,6 +353,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             if (string)
                 free (string);
         }
+        /* command_delay */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_COMMAND_DELAY]))
             weechat_printf (NULL, "  command_delay. . . . :   (%d %s)",
                             IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_COMMAND_DELAY),
@@ -297,6 +363,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_integer (server->options[IRC_SERVER_OPTION_COMMAND_DELAY]),
                             NG_("second", "seconds", weechat_config_integer (server->options[IRC_SERVER_OPTION_COMMAND_DELAY])));
+        /* autojoin */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_AUTOJOIN]))
             weechat_printf (NULL, "  autojoin . . . . . . :   ('%s')",
                             IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_AUTOJOIN));
@@ -304,6 +371,7 @@ irc_display_server (struct t_irc_server *server, int with_detail)
             weechat_printf (NULL, "  autojoin . . . . . . : %s'%s'",
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_string (server->options[IRC_SERVER_OPTION_AUTOJOIN]));
+        /* autorejoin */
         if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_AUTOREJOIN]))
             weechat_printf (NULL, "  autorejoin . . . . . :   (%s)",
                             (IRC_SERVER_OPTION_BOOLEAN(server, IRC_SERVER_OPTION_AUTOREJOIN)) ?
@@ -313,6 +381,16 @@ irc_display_server (struct t_irc_server *server, int with_detail)
                             IRC_COLOR_CHAT_VALUE,
                             weechat_config_boolean (server->options[IRC_SERVER_OPTION_AUTOREJOIN]) ?
                             _("on") : _("off"));
+        /* autorejoin_delay */
+        if (weechat_config_option_is_null (server->options[IRC_SERVER_OPTION_AUTOREJOIN_DELAY]))
+            weechat_printf (NULL, "  autorejoin_delay . . :   (%d %s)",
+                            IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AUTOREJOIN_DELAY),
+                            NG_("second", "seconds", IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AUTOREJOIN_DELAY)));
+        else
+            weechat_printf (NULL, "  autorejoin_delay . . : %s%d %s",
+                            IRC_COLOR_CHAT_VALUE,
+                            weechat_config_integer (server->options[IRC_SERVER_OPTION_AUTOREJOIN_DELAY]),
+                            NG_("second", "seconds", weechat_config_integer (server->options[IRC_SERVER_OPTION_AUTOREJOIN_DELAY])));
     }
     else
     {
