@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@ extern int irc_mode_channel_set (struct t_irc_server *server,
                                  struct t_irc_channel *channel,
                                  const char *modes);
 extern void irc_mode_user_set (struct t_irc_server *server, const char *modes);
-extern int irc_mode_nick_prefix_allowed (struct t_irc_server *server,
-                                         char prefix);
+extern int irc_mode_get_nick_prefix (struct t_irc_server *server, char *mode,
+                                     char prefix);
 
 #endif /* irc-mode.h */
