@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* irc-nick.c: manages nick list for channels for IRC plugin */
+/*
+ * irc-nick.c: nick management for IRC plugin
+ */
 
 
 #include <stdlib.h>
@@ -56,24 +58,56 @@ irc_nick_valid (struct t_irc_channel *channel, struct t_irc_nick *nick)
     return 0;
 }
 
+/*
+ * irc_nick_is_nick: check if string is a valid nick string (RFC 1459)
+ *                   return 1 if string valid nick
+ *                          0 if not a valid nick
+ */
+
+int
+irc_nick_is_nick (const char *string)
+{
+    const char *ptr;
+    
+    if (!string || !string[0])
+        return 0;
+    
+    /* first char must not be a number or hyphen */
+    ptr = string;
+    if (strchr ("0123456789-", *ptr))
+        return 0;
+    
+    while (ptr && ptr[0])
+    {
+        if (!strchr (IRC_NICK_VALID_CHARS, *ptr))
+            return 0;
+        ptr++;
+    }
+    
+    return 1;
+}
+
 /*
  * irc_nick_find_color: find a color for a nick (according to nick letters)
  */
 
 const char *
-irc_nick_find_color (struct t_irc_nick *nick)
+irc_nick_find_color (const char *nickname)
 {
-    int i, color;
+    int color;
     char color_name[64];
+    const char *ptr_nick;
     
     color = 0;
-    for (i = strlen (nick->name) - 1; i >= 0; i--)
+    ptr_nick = nickname;
+    while (ptr_nick && ptr_nick[0])
     {
-        color += (int)(nick->name[i]);
+        color += weechat_utf8_char_int (ptr_nick);
+        ptr_nick = weechat_utf8_next_char (ptr_nick);
     }
     color = (color %
              weechat_config_integer (weechat_config_get ("weechat.look.color_nicks_number")));
-
+    
     snprintf (color_name, sizeof (color_name),
               "chat_nick_color%02d", color + 1);
     
@@ -179,7 +213,7 @@ irc_nick_get_gui_infos (struct t_irc_nick *nick,
 
 struct t_irc_nick *
 irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
-              const char *nick_name, int is_chanowner, int is_chanadmin,
+              const char *nickname, int is_chanowner, int is_chanadmin,
               int is_chanadmin2, int is_op, int is_halfop, int has_voice,
               int is_chanuser, int is_away)
 {
@@ -189,7 +223,7 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     struct t_gui_nick_group *ptr_group;
     
     /* nick already exists on this channel? */
-    ptr_nick = irc_nick_search (channel, nick_name);
+    ptr_nick = irc_nick_search (channel, nickname);
     if (ptr_nick)
     {
         /* remove old nick from nicklist */
@@ -232,7 +266,7 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
         return NULL;
     
     /* initialize new nick */
-    new_nick->name = strdup (nick_name);
+    new_nick->name = strdup (nickname);
     new_nick->host = NULL;
     new_nick->flags = 0;
     IRC_NICK_SET_FLAG(new_nick, is_chanowner, IRC_NICK_CHANOWNER);
@@ -246,7 +280,7 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     if (weechat_strcasecmp (new_nick->name, server->nick) == 0)
         new_nick->color = IRC_COLOR_CHAT_NICK_SELF;
     else
-        new_nick->color = irc_nick_find_color (new_nick);
+        new_nick->color = irc_nick_find_color (new_nick->name);
     
     /* add nick to end of list */
     new_nick->prev_nick = channel->last_nick;
@@ -311,7 +345,7 @@ irc_nick_change (struct t_irc_server *server, struct t_irc_channel *channel,
     if (nick_is_me)
         nick->color = IRC_COLOR_CHAT_NICK_SELF;
     else
-        nick->color = irc_nick_find_color (nick);
+        nick->color = irc_nick_find_color (nick->name);
     
     /* add nick in nicklist */
     prefix[0] = ' ';
@@ -322,7 +356,9 @@ irc_nick_change (struct t_irc_server *server, struct t_irc_channel *channel,
               "weechat.color.nicklist_prefix%d",
               prefix_color);
     weechat_nicklist_add_nick (channel->buffer, ptr_group,
-                               nick->name, "bar_fg",
+                               nick->name,
+                               (nick->flags & IRC_NICK_AWAY) ?
+                               "weechat.color.nicklist_away" : "bar_fg",
                                prefix, str_prefix_color, 1);
 }
 
@@ -331,7 +367,7 @@ irc_nick_change (struct t_irc_server *server, struct t_irc_channel *channel,
  */
 
 void
-irc_nick_set (struct t_irc_channel *channel,
+irc_nick_set (struct t_irc_server *server, struct t_irc_channel *channel,
               struct t_irc_nick *nick, int set, int flag)
 {
     char prefix[2], str_prefix_color[64];
@@ -362,6 +398,9 @@ irc_nick_set (struct t_irc_channel *channel,
                                (nick->flags & IRC_NICK_AWAY) ?
                                "weechat.color.nicklist_away" : "bar_fg",
                                prefix, str_prefix_color, 1);
+    
+    if (strcmp (nick->name, server->nick) == 0)
+        weechat_bar_item_update ("input_prompt");
 }
 
 /*
@@ -502,8 +541,8 @@ irc_nick_count (struct t_irc_channel *channel, int *total, int *count_op,
  */
 
 void
-irc_nick_set_away (struct t_irc_channel *channel, struct t_irc_nick *nick,
-                   int is_away)
+irc_nick_set_away (struct t_irc_server *server, struct t_irc_channel *channel,
+                   struct t_irc_nick *nick, int is_away)
 {
     if ((weechat_config_integer (irc_config_network_away_check) > 0)
         && ((weechat_config_integer (irc_config_network_away_check_max_nicks) == 0) ||
@@ -512,7 +551,7 @@ irc_nick_set_away (struct t_irc_channel *channel, struct t_irc_nick *nick,
         if (((is_away) && (!(nick->flags & IRC_NICK_AWAY))) ||
             ((!is_away) && (nick->flags & IRC_NICK_AWAY)))
         {
-            irc_nick_set (channel, nick, is_away, IRC_NICK_AWAY);
+            irc_nick_set (server, channel, nick, is_away, IRC_NICK_AWAY);
         }
     }
 }
@@ -579,6 +618,24 @@ irc_nick_as_prefix (struct t_irc_nick *nick, const char *nickname,
     return result;
 }
 
+/*
+ * irc_nick_color_for_pv: return string with color of nick for private
+ */
+
+const char *
+irc_nick_color_for_pv (struct t_irc_channel *channel, const char *nickname)
+{
+    if (weechat_config_boolean (irc_config_look_color_pv_nick_like_channel))
+    {
+        if (!channel->pv_remote_nick_color)
+            channel->pv_remote_nick_color = strdup (irc_nick_find_color (nickname));
+        if (channel->pv_remote_nick_color)
+            return channel->pv_remote_nick_color;
+    }
+    
+    return IRC_COLOR_CHAT_NICK_OTHER;
+}
+
 /*
  * irc_nick_add_to_infolist: add a nick in an infolist
  *                           return 1 if ok, 0 if error
