@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,8 @@
 
 extern struct t_weechat_plugin *weechat_logger_plugin;
 
-extern void logger_start_buffer_all ();
-extern void logger_stop_all ();
+extern void logger_start_buffer_all (int write_info_line);
+extern void logger_stop_all (int write_info_line);
+extern void logger_adjust_log_filenames ();
 
 #endif /* logger.h */
