@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* plugin-api.c: WeeChat <--> plugin API (extra functions) */
+/*
+ * plugin-api.c: extra functions for plugin API
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -24,6 +26,7 @@
 #endif
 
 #include <errno.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdarg.h>
@@ -38,6 +41,7 @@
 #include "../core/wee-infolist.h"
 #include "../core/wee-input.h"
 #include "../core/wee-string.h"
+#include "../core/wee-util.h"
 #include "../gui/gui-bar.h"
 #include "../gui/gui-bar-item.h"
 #include "../gui/gui-bar-window.h"
@@ -215,16 +219,16 @@ plugin_api_prefix (const char *prefix)
 const char *
 plugin_api_color (const char *color_name)
 {
-    int num_color;
+    const char *str_color;
     
     if (!color_name)
         return GUI_NO_COLOR;
     
     /* name is a weechat color option ? => then return this color */
-    num_color = gui_color_search_config_int (color_name);
-    if (num_color >= 0)
-        return GUI_COLOR(num_color);
-
+    str_color = gui_color_search_config (color_name);
+    if (str_color)
+        return str_color;
+    
     return gui_color_get_custom (color_name);
 }
 
@@ -258,7 +262,8 @@ plugin_api_info_get_internal (void *data, const char *info_name,
                               const char *arguments)
 {
     time_t inactivity;
-    static char value[32];
+    static char value[32], version_number[32] = { '\0' };
+    static char weechat_dir_absolute_path[PATH_MAX] = { '\0' };
     
     /* make C compiler happy */
     (void) data;
@@ -271,7 +276,16 @@ plugin_api_info_get_internal (void *data, const char *info_name,
     {
         return PACKAGE_VERSION;
     }
-    if (string_strcasecmp (info_name, "date") == 0)
+    else if (string_strcasecmp (info_name, "version_number") == 0)
+    {
+        if (!version_number[0])
+        {
+            snprintf (version_number, sizeof (version_number), "%d",
+                      util_version_number (PACKAGE_VERSION));
+        }
+        return version_number;
+    }
+    else if (string_strcasecmp (info_name, "date") == 0)
     {
         return __DATE__;
     }
@@ -281,7 +295,12 @@ plugin_api_info_get_internal (void *data, const char *info_name,
     }
     else if (string_strcasecmp (info_name, "weechat_dir") == 0)
     {
-        return weechat_home;
+        if (!weechat_dir_absolute_path[0])
+        {
+            realpath (weechat_home, weechat_dir_absolute_path);
+        }
+        return (weechat_dir_absolute_path[0]) ?
+            weechat_dir_absolute_path : weechat_home;
     }
     else if (string_strcasecmp (info_name, "weechat_libdir") == 0)
     {
@@ -295,6 +314,14 @@ plugin_api_info_get_internal (void *data, const char *info_name,
     {
         return LOCALEDIR;
     }
+    else if (string_strcasecmp (info_name, "weechat_site") == 0)
+    {
+        return WEECHAT_WEBSITE;
+    }
+    else if (string_strcasecmp (info_name, "weechat_site_download") == 0)
+    {
+        return WEECHAT_WEBSITE_DOWNLOAD;
+    }
     else if (string_strcasecmp (info_name, "charset_terminal") == 0)
     {
         return weechat_local_charset;
@@ -650,7 +677,7 @@ plugin_api_infolist_get_internal (void *data, const char *infolist_name,
         ptr_infolist = infolist_new ();
         if (ptr_infolist)
         {
-            if (!gui_nicklist_add_to_infolist (ptr_infolist, pointer))
+            if (!gui_nicklist_add_to_infolist (ptr_infolist, pointer, arguments))
             {
                 infolist_free (ptr_infolist);
                 return NULL;
@@ -910,56 +937,91 @@ void
 plugin_api_init ()
 {
     /* WeeChat core info hooks */
-    hook_info (NULL, "version", N_("WeeChat version"),
+    hook_info (NULL, "version", N_("WeeChat version"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "date", N_("WeeChat compilation date"),
+    hook_info (NULL, "version_number", N_("WeeChat version (as number)"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "dir_separator", N_("directory separator"),
+    hook_info (NULL, "date", N_("WeeChat compilation date"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "weechat_dir", N_("WeeChat directory"),
+    hook_info (NULL, "dir_separator", N_("directory separator"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "weechat_libdir", N_("WeeChat \"lib\" directory"),
+    hook_info (NULL, "weechat_dir", N_("WeeChat directory"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "weechat_sharedir", N_("WeeChat \"share\" directory"),
+    hook_info (NULL, "weechat_libdir", N_("WeeChat \"lib\" directory"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "weechat_localedir", N_("WeeChat \"locale\" directory"),
+    hook_info (NULL, "weechat_sharedir", N_("WeeChat \"share\" directory"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "charset_terminal", N_("terminal charset"),
+    hook_info (NULL, "weechat_localedir", N_("WeeChat \"locale\" directory"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "charset_internal", N_("WeeChat internal charset"),
+    hook_info (NULL, "weechat_site", N_("WeeChat site"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "inactivity", N_("keyboard inactivity (seconds)"),
+    hook_info (NULL, "weechat_site_download", N_("WeeChat site, download page"), NULL,
                &plugin_api_info_get_internal, NULL);
-    hook_info (NULL, "filters_enabled", N_("1 if filters are enabled"),
+    hook_info (NULL, "charset_terminal", N_("terminal charset"), NULL,
+               &plugin_api_info_get_internal, NULL);
+    hook_info (NULL, "charset_internal", N_("WeeChat internal charset"), NULL,
+               &plugin_api_info_get_internal, NULL);
+    hook_info (NULL, "inactivity", N_("keyboard inactivity (seconds)"), NULL,
+               &plugin_api_info_get_internal, NULL);
+    hook_info (NULL, "filters_enabled", N_("1 if filters are enabled"), NULL,
                &plugin_api_info_get_internal, NULL);
     
     /* WeeChat core infolist hooks */
     hook_infolist (NULL, "bar", N_("list of bars"),
+                   N_("bar pointer (optional)"),
+                   N_("bar name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "bar_item", N_("list of bar items"),
+                   N_("bar item pointer (optional)"),
+                   N_("bar item name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "bar_window", N_("list of bar windows"),
+                   N_("bar window pointer (optional)"),
+                   NULL,
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "buffer", N_("list of buffers"),
+                   N_("buffer pointer (optional)"),
+                   N_("buffer name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "buffer_lines", N_("lines of a buffer"),
+                   N_("buffer pointer"),
+                   NULL,
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "filter", N_("list of filters"),
+                   NULL,
+                   N_("filter name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "history", N_("history of commands"),
+                   N_("buffer pointer (if not set, return global history) (optional)"),
+                   NULL,
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "hook", N_("list of hooks"),
+                   NULL,
+                   N_("hook type: command, timer, .. (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "hotlist", N_("list of buffers in hotlist"),
+                   NULL,
+                   NULL,
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "key", N_("list of key bindings"),
+                   NULL,
+                   NULL,
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "nicklist", N_("nicks in nicklist for a buffer"),
+                   N_("buffer pointer"),
+                   N_("nick_xxx or group_xxx to get only nick/group xxx "
+                      "(optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "option", N_("list of options"),
+                   NULL,
+                   N_("option name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "plugin", N_("list of plugins"),
+                   N_("plugin pointer (optional)"),
+                   N_("plugin name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "window", N_("list of windows"),
+                   N_("window pointer (optional)"),
+                   N_("window name (can start or end with \"*\" as joker) (optional)"),
                    &plugin_api_infolist_get_internal, NULL);
 }
