@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-lua.c: Lua plugin for WeeChat */
+/*
+ * weechat-lua.c: lua plugin for WeeChat
+ */
 
 
 #undef _
@@ -37,7 +39,7 @@ WEECHAT_PLUGIN_NAME(LUA_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Lua plugin for WeeChat");
 WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_lua_plugin;
 
@@ -45,21 +47,24 @@ int lua_quiet = 0;
 struct t_plugin_script *lua_scripts = NULL;
 struct t_plugin_script *last_lua_script = NULL;
 struct t_plugin_script *lua_current_script = NULL;
+struct t_plugin_script *lua_registered_script = NULL;
 const char *lua_current_script_filename = NULL;
 lua_State *lua_current_interpreter = NULL;
 
-/* string used to execute action "install":
-   when signal "lua_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "lua_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *lua_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "lua_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "lua_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *lua_action_remove_list = NULL;
 
 
@@ -69,7 +74,7 @@ char *lua_action_remove_list = NULL;
 
 void *
 weechat_lua_exec (struct t_plugin_script *script,
-		  int ret_type, const char *function, char **argv)
+                  int ret_type, const char *function, char **argv)
 {
     void *ret_value;
     int argc, *ret_i;
@@ -128,10 +133,10 @@ weechat_lua_exec (struct t_plugin_script *script,
     
     if (lua_pcall (lua_current_interpreter, argc, 1, 0) != 0)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to run function \"%s\""),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME, function);
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: error: %s"),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME,
                         lua_tostring (lua_current_interpreter, -1));
@@ -140,24 +145,24 @@ weechat_lua_exec (struct t_plugin_script *script,
     }
     
     if (ret_type == WEECHAT_SCRIPT_EXEC_STRING)
-	ret_value = strdup ((char *) lua_tostring (lua_current_interpreter, -1));
+        ret_value = strdup ((char *) lua_tostring (lua_current_interpreter, -1));
     else if (ret_type == WEECHAT_SCRIPT_EXEC_INT)
     {
-	ret_i = malloc (sizeof (*ret_i));
-	if (ret_i)
-	    *ret_i = lua_tonumber (lua_current_interpreter, -1);
-	ret_value = ret_i;
+        ret_i = malloc (sizeof (*ret_i));
+        if (ret_i)
+            *ret_i = lua_tonumber (lua_current_interpreter, -1);
+        ret_value = ret_i;
     }
     else
     {
         WEECHAT_SCRIPT_MSG_WRONG_ARGS(LUA_CURRENT_SCRIPT_NAME, function);
         lua_current_script = old_lua_current_script;
-	return NULL;
+        return NULL;
     }
     
     lua_current_script = old_lua_current_script;
     
-    return ret_value; 
+    return ret_value;
 }
 
 int
@@ -165,13 +170,13 @@ weechat_lua_load (const char *filename)
 {
     FILE *fp;
     char *weechat_lua_code = {
-	"weechat_outputs = {\n"
-	"    write = function (self, str)\n"
-	"        weechat.print(\"\", \"lua: stdout/stderr: \" .. str)\n"
+        "weechat_outputs = {\n"
+        "    write = function (self, str)\n"
+        "        weechat.print(\"\", \"lua: stdout/stderr: \" .. str)\n"
         "    end\n"
-	"}\n"
-	"io.stdout = weechat_outputs\n"
-	"io.stderr = weechat_outputs\n"
+        "}\n"
+        "io.stdout = weechat_outputs\n"
+        "io.stderr = weechat_outputs\n"
     };
     
     if ((fp = fopen (filename, "r")) == NULL)
@@ -190,6 +195,7 @@ weechat_lua_load (const char *filename)
     }
     
     lua_current_script = NULL;
+    lua_registered_script = NULL;
     
     lua_current_interpreter = lua_open ();
 
@@ -218,7 +224,7 @@ weechat_lua_load (const char *filename)
 
 #ifdef LUA_VERSION_NUM
     if (luaL_dostring (lua_current_interpreter, weechat_lua_code) != 0)
-#else    
+#else
     if (lua_dostring (lua_current_interpreter, weechat_lua_code) != 0)
 #endif
     {
@@ -235,7 +241,7 @@ weechat_lua_load (const char *filename)
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to load file \"%s\""),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: error: %s"),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME,
                         lua_tostring (lua_current_interpreter, -1));
@@ -250,29 +256,30 @@ weechat_lua_load (const char *filename)
                         weechat_gettext ("%s%s: unable to execute file "
                                          "\"%s\""),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: error: %s"),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME,
                         lua_tostring (lua_current_interpreter, -1));
         lua_close (lua_current_interpreter);
         fclose (fp);
-	/* if script was registered, removing from list */
-	if (lua_current_script)
-	    script_remove (weechat_lua_plugin, &lua_scripts, &last_lua_script,
+        /* if script was registered, removing from list */
+        if (lua_current_script)
+            script_remove (weechat_lua_plugin, &lua_scripts, &last_lua_script,
                            lua_current_script);
         return 0;
     }
     fclose (fp);
     
-    if (lua_current_script == NULL)
+    if (!lua_registered_script)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
-	lua_close (lua_current_interpreter);
+        lua_close (lua_current_interpreter);
         return 0;
     }
+    lua_current_script = lua_registered_script;
     
     lua_current_script->interpreter = (lua_State *) lua_current_interpreter;
     
@@ -303,18 +310,21 @@ weechat_lua_unload (struct t_plugin_script *script)
     char *lua_argv[1] = { NULL };
     void *interpreter;
     
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    LUA_PLUGIN_NAME, script->name);
+    if ((weechat_lua_plugin->debug >= 1) || !lua_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        LUA_PLUGIN_NAME, script->name);
+    }
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
         r = weechat_lua_exec (script,
                               WEECHAT_SCRIPT_EXEC_INT,
-			      script->shutdown_func,
+                              script->shutdown_func,
                               lua_argv);
-	if (r)
-	    free (r);
+        if (r)
+            free (r);
     }
     
     interpreter = script->interpreter;
@@ -353,6 +363,38 @@ weechat_lua_unload_name (const char *name)
     }
 }
 
+/*
+ * weechat_lua_reload_name: reload a Lua script by name
+ */
+
+void
+weechat_lua_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_lua_plugin, lua_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_lua_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            LUA_PLUGIN_NAME, name);
+            weechat_lua_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), LUA_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_lua_unload_all: unload all Lua scripts
  */
@@ -432,6 +474,11 @@ weechat_lua_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Lua script */
+            weechat_lua_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Lua script */
@@ -637,11 +684,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    /* make C compiler happy */
-    (void) plugin;
-    
     /* unload all scripts */
-    weechat_lua_unload_all ();
+    lua_quiet = 1;
+    script_end (plugin, &lua_scripts, &weechat_lua_unload_all);
+    lua_quiet = 0;
     
     return WEECHAT_RC_OK;
 }
