@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-perl.c: Perl plugin for WeeChat */
+/*
+ * weechat-perl.c: perl plugin for WeeChat
+ */
 
 
 #undef _
@@ -35,7 +37,7 @@ WEECHAT_PLUGIN_NAME(PERL_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Perl plugin for WeeChat");
 WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_perl_plugin = NULL;
 
@@ -43,21 +45,24 @@ int perl_quiet = 0;
 struct t_plugin_script *perl_scripts = NULL;
 struct t_plugin_script *last_perl_script = NULL;
 struct t_plugin_script *perl_current_script = NULL;
+struct t_plugin_script *perl_registered_script = NULL;
 const char *perl_current_script_filename = NULL;
 int perl_quit_or_upgrade = 0;
 
-/* string used to execute action "install":
-   when signal "perl_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "perl_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *perl_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "perl_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "perl_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *perl_action_remove_list = NULL;
 
 #ifdef NO_PERL_MULTIPLICITY
@@ -115,8 +120,8 @@ char *perl_weechat_code =
     "    }"
     "    return 0;"
     "}"
-    "$SIG{__WARN__} = sub { weechat::print(\"\", \"perl error: $_[0]\"); };"
-    "$SIG{__DIE__} = sub { weechat::print(\"\", \"perl error: $_[0]\"); };"
+    "$SIG{__WARN__} = sub { weechat::print(\"\", \"\tperl error: $_[0]\"); };"
+    "$SIG{__DIE__} = sub { weechat::print(\"\", \"\tperl error: $_[0]\"); };"
 };
 
 
@@ -126,7 +131,7 @@ char *perl_weechat_code =
 
 void *
 weechat_perl_exec (struct t_plugin_script *script,
-		   int ret_type, const char *function, char **argv)
+                   int ret_type, const char *function, char **argv)
 {
     char *func;
     unsigned int count;
@@ -148,7 +153,8 @@ weechat_perl_exec (struct t_plugin_script *script,
     (void) length;
     func = (char *) function;
     old_context = PERL_GET_CONTEXT;
-    PERL_SET_CONTEXT (script->interpreter);
+    if (script->interpreter)
+        PERL_SET_CONTEXT (script->interpreter);
 #else
     length = strlen (script->interpreter) + strlen (function) + 3;
     func = (char *) malloc (length);
@@ -173,7 +179,7 @@ weechat_perl_exec (struct t_plugin_script *script,
                         weechat_gettext ("%s%s: error: %s"),
                         weechat_prefix ("error"), PERL_PLUGIN_NAME,
                         SvPV_nolen (ERRSV));
-        (void) POPs; /* poping the 'undef' */	
+        (void) POPs; /* poping the 'undef' */
         mem_err = 0;
     }
     else
@@ -218,8 +224,7 @@ weechat_perl_exec (struct t_plugin_script *script,
     FREETMPS;
     LEAVE;
     
-    if (old_perl_current_script)
-        perl_current_script = old_perl_current_script;
+    perl_current_script = old_perl_current_script;
 #ifdef MULTIPLICITY
     PERL_SET_CONTEXT (old_context);
 #else
@@ -283,6 +288,8 @@ weechat_perl_load (const char *filename)
     }
     
     perl_current_script = NULL;
+    perl_current_script_filename = filename;
+    perl_registered_script = NULL;
     
 #ifdef MULTIPLICITY
     perl_current_interpreter = perl_alloc();
@@ -296,13 +303,11 @@ weechat_perl_load (const char *filename)
         return 0;
     }
     
-    perl_current_script_filename = filename;
-    
     PERL_SET_CONTEXT (perl_current_interpreter);
     perl_construct (perl_current_interpreter);
     temp_script.interpreter = (PerlInterpreter *) perl_current_interpreter;
     perl_parse (perl_current_interpreter, weechat_perl_api_init,
-		perl_args_count, perl_args, NULL);
+                perl_args_count, perl_args, NULL);
     
     eval_pv (perl_weechat_code, TRUE);
     perl_argv[0] = (char *)filename;
@@ -316,23 +321,23 @@ weechat_perl_load (const char *filename)
     perl_argv[2] = NULL;
 #endif
     eval = weechat_perl_exec (&temp_script,
-			      WEECHAT_SCRIPT_EXEC_INT,
-			      "weechat_perl_load_eval_file",
+                              WEECHAT_SCRIPT_EXEC_INT,
+                              "weechat_perl_load_eval_file",
                               perl_argv);
     if (!eval)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: not enough memory to parse "
                                          "file \"%s\""),
                         weechat_prefix ("error"), PERL_PLUGIN_NAME, filename);
-	return 0;
+        return 0;
     }
     
-    if (*eval != 0) 
+    if (*eval != 0)
     {
-	if (*eval == 2) 
-	{
-	    weechat_printf (NULL,
+        if (*eval == 2)
+        {
+            weechat_printf (NULL,
                             weechat_gettext ("%s%s: unable to parse file "
                                              "\"%s\""),
                             weechat_prefix ("error"), PERL_PLUGIN_NAME,
@@ -351,42 +356,42 @@ weechat_perl_load (const char *filename)
                                              "weechat_perl_load_eval_file_error",
                                              FALSE), len));
 #endif
-	}
-	else if (*eval == 1)
-	{
-	    weechat_printf (NULL,
+        }
+        else if (*eval == 1)
+        {
+            weechat_printf (NULL,
                             weechat_gettext ("%s%s: unable to run file \"%s\""),
                             weechat_prefix ("error"), PERL_PLUGIN_NAME,
                             filename);
-	}
-	else
+        }
+        else
         {
-	    weechat_printf (NULL,
+            weechat_printf (NULL,
                             weechat_gettext ("%s%s: unknown error while "
                                              "loading file \"%s\""),
                             weechat_prefix ("error"), PERL_PLUGIN_NAME,
                             filename);
-	}
+        }
 #ifdef MULTIPLICITY
-	perl_destruct (perl_current_interpreter);
+        perl_destruct (perl_current_interpreter);
         perl_free (perl_current_interpreter);
 #endif
-	if (perl_current_script && (perl_current_script != &temp_script))
+        if (perl_current_script && (perl_current_script != &temp_script))
         {
             script_remove (weechat_perl_plugin,
                            &perl_scripts, &last_perl_script,
                            perl_current_script);
         }
-	
-	free (eval);
-	return 0;
+
+        free (eval);
+        return 0;
     }
     
     free (eval);
-
-    if (!perl_current_script)
+    
+    if (!perl_registered_script)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), PERL_PLUGIN_NAME, filename);
@@ -396,7 +401,8 @@ weechat_perl_load (const char *filename)
 #endif
         return 0;
     }
-
+    perl_current_script = perl_registered_script;
+    
 #ifdef MULTIPLICITY
     perl_current_script->interpreter = (PerlInterpreter *)perl_current_interpreter;
 #else
@@ -429,25 +435,28 @@ weechat_perl_unload (struct t_plugin_script *script)
     int *r;
     char *perl_argv[1] = { NULL };
     void *interpreter;
-    
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    PERL_PLUGIN_NAME, script->name);
+
+    if ((weechat_perl_plugin->debug >= 1) || !perl_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        PERL_PLUGIN_NAME, script->name);
+    }
     
 #ifdef MULTIPLICITY
     PERL_SET_CONTEXT (script->interpreter);
 #else
     eval_pv (script->interpreter, TRUE);
-#endif        
+#endif
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
         r = (int *) weechat_perl_exec (script,
                                        WEECHAT_SCRIPT_EXEC_INT,
-				       script->shutdown_func,
+                                       script->shutdown_func,
                                        perl_argv);
-	if (r)
-	    free (r);
+        if (r)
+            free (r);
     }
     
     interpreter = script->interpreter;
@@ -464,7 +473,7 @@ weechat_perl_unload (struct t_plugin_script *script)
     perl_free (interpreter);
 #else
     if (interpreter)
-	free (interpreter);
+        free (interpreter);
 #endif
 }
 
@@ -506,6 +515,38 @@ weechat_perl_unload_all ()
     }
 }
 
+/*
+ * weechat_perl_reload_name: reload a Perl script by name
+ */
+
+void
+weechat_perl_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_perl_plugin, perl_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_perl_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            PERL_PLUGIN_NAME, name);
+            weechat_perl_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), PERL_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_perl_command_cb: callback for "/perl" command
  */
@@ -572,6 +613,11 @@ weechat_perl_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Perl script */
+            weechat_perl_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Perl script */
@@ -794,7 +840,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     
     perl_construct (perl_main);
     perl_parse (perl_main, weechat_perl_api_init, perl_args_count,
-		perl_args, NULL);
+                perl_args, NULL);
     eval_pv (perl_weechat_code, TRUE);
 #endif
     
@@ -831,23 +877,26 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    /* make C compiler happy */
-    (void) plugin;
-    
     /* unload all scripts */
-    weechat_perl_unload_all ();
+    perl_quiet = 1;
+    script_end (plugin, &perl_scripts, &weechat_perl_unload_all);
+    perl_quiet = 0;
     
 #ifndef MULTIPLICITY
     /* free perl intepreter */
     if (perl_main)
     {
-	perl_destruct (perl_main);
-	perl_free (perl_main);
-	perl_main = NULL;
+        perl_destruct (perl_main);
+        perl_free (perl_main);
+        perl_main = NULL;
     }
 #endif
-
-#ifdef PERL_SYS_TERM
+    
+#if defined(PERL_SYS_TERM) && !defined(__FreeBSD__) && !defined(WIN32) && !defined(__CYGWIN__)
+    /*
+     * we call this function on all OS, but NOT on FreeBSD or Cygwin,
+     * because it crashes with no reason (bug in Perl?)
+     */
     if (perl_quit_or_upgrade)
         PERL_SYS_TERM ();
 #endif
