@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-python.c: Python plugin for WeeChat */
+/*
+ * weechat-python.c: python plugin for WeeChat
+ */
 
 
 #undef _
@@ -33,7 +35,7 @@ WEECHAT_PLUGIN_NAME(PYTHON_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Python plugin for WeeChat");
 WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_python_plugin = NULL;
 
@@ -41,21 +43,24 @@ int python_quiet;
 struct t_plugin_script *python_scripts = NULL;
 struct t_plugin_script *last_python_script = NULL;
 struct t_plugin_script *python_current_script = NULL;
+struct t_plugin_script *python_registered_script = NULL;
 const char *python_current_script_filename = NULL;
 PyThreadState *python_mainThreadState = NULL;
 
-/* string used to execute action "install":
-   when signal "python_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "python_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *python_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "python_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "python_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *python_action_remove_list = NULL;
 
 char python_buffer_output[128];
@@ -67,7 +72,7 @@ char python_buffer_output[128];
 
 void *
 weechat_python_exec (struct t_plugin_script *script,
-		     int ret_type, const char *function, char **argv)
+                     int ret_type, const char *function, char **argv)
 {
     struct t_plugin_script *old_python_current_script;
     PyThreadState *old_interpreter;
@@ -95,12 +100,12 @@ weechat_python_exec (struct t_plugin_script *script,
     if ( !(evFunc && PyCallable_Check (evFunc)) )
     {
         weechat_printf (NULL,
-                        weechat_gettext ("%s%s unable to run function \"%s\""),
+                        weechat_gettext ("%s%s: unable to run function \"%s\""),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, function);
-	/* PyEval_ReleaseThread (python_current_script->interpreter); */
+        /* PyEval_ReleaseThread (python_current_script->interpreter); */
         if (old_interpreter)
             PyThreadState_Swap (old_interpreter);
-	return NULL;
+        return NULL;
     }
     
     python_current_script = script;
@@ -179,12 +184,12 @@ weechat_python_exec (struct t_plugin_script *script,
     
     ret_value = NULL;
     
-    /* 
-       ugly hack : rc = NULL while 'return weechat.WEECHAT_RC_OK .... 
-       because of '#define WEECHAT_RC_OK 0'
-    */
+    /*
+     * ugly hack : rc = NULL while 'return weechat.WEECHAT_RC_OK ....
+     * because of '#define WEECHAT_RC_OK 0'
+     */
     if (rc ==  NULL)
-	rc = PyInt_FromLong (0);
+        rc = PyInt_FromLong (0);
     
     if (PyErr_Occurred())
     {
@@ -193,19 +198,18 @@ weechat_python_exec (struct t_plugin_script *script,
     }
     else if (PyString_Check (rc) && (ret_type == WEECHAT_SCRIPT_EXEC_STRING))
     {
-	if (PyString_AsString (rc))
-	    ret_value = strdup (PyString_AsString(rc));
-	else
-	    ret_value = NULL;
-	
-	Py_XDECREF(rc);
+        if (PyString_AsString (rc))
+            ret_value = strdup (PyString_AsString(rc));
+        else
+            ret_value = NULL;
+
+        Py_XDECREF(rc);
     }
     else if (PyInt_Check (rc) && (ret_type == WEECHAT_SCRIPT_EXEC_INT))
     {
-	
-	ret_i = malloc (sizeof (*ret_i));
-	if (ret_i)
-	    *ret_i = (int) PyInt_AsLong(rc);
+        ret_i = malloc (sizeof (*ret_i));
+        if (ret_i)
+            *ret_i = (int) PyInt_AsLong(rc);
         ret_value = ret_i;
         
         Py_XDECREF(rc);
@@ -227,8 +231,7 @@ weechat_python_exec (struct t_plugin_script *script,
     
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
     
-    if (old_python_current_script)
-        python_current_script = old_python_current_script;
+    python_current_script = old_python_current_script;
     
     if (old_interpreter)
         PyThreadState_Swap (old_interpreter);
@@ -253,39 +256,39 @@ weechat_python_output (PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple (args, "s", &msg))
     {
         if (strlen(python_buffer_output) > 0)
-	{
-	    weechat_printf (NULL,
+        {
+            weechat_printf (NULL,
                             weechat_gettext ("%s: stdout/stderr: %s%s"),
                             PYTHON_PLUGIN_NAME, python_buffer_output, "");
-	    python_buffer_output[0] = '\0';
-	}
+            python_buffer_output[0] = '\0';
+        }
     }
-    else 
+    else
     {
-	m = msg;
-	while ((p = strchr (m, '\n')) != NULL)
-	{
-	    *p = '\0';
-	    if (strlen (m) + strlen (python_buffer_output) > 0)
+        m = msg;
+        while ((p = strchr (m, '\n')) != NULL)
+        {
+            *p = '\0';
+            if (strlen (m) + strlen (python_buffer_output) > 0)
             {
-		weechat_printf (NULL,
+                weechat_printf (NULL,
                                 weechat_gettext ("%s: stdout/stderr: %s%s"),
                                 PYTHON_PLUGIN_NAME, python_buffer_output, m);
             }
-	    *p = '\n';
-	    python_buffer_output[0] = '\0';
-	    m = ++p;
-	}
-	
-	if (strlen(m) + strlen(python_buffer_output) > sizeof(python_buffer_output))
-	{
-	    weechat_printf (NULL,
+            *p = '\n';
+            python_buffer_output[0] = '\0';
+            m = ++p;
+        }
+
+        if (strlen(m) + strlen(python_buffer_output) > sizeof(python_buffer_output))
+        {
+            weechat_printf (NULL,
                             weechat_gettext ("%s: stdout/stderr: %s%s"),
                             PYTHON_PLUGIN_NAME, python_buffer_output, m);
-	    python_buffer_output[0] = '\0';
-	}
-	else
-	    strcat (python_buffer_output, m);
+            python_buffer_output[0] = '\0';
+        }
+        else
+            strcat (python_buffer_output, m);
     }
     
     Py_INCREF(Py_None);
@@ -334,6 +337,7 @@ weechat_python_load (const char *filename)
     }
     
     python_current_script = NULL;
+    python_registered_script = NULL;
     
     /* PyEval_AcquireLock (); */
     python_current_interpreter = Py_NewInterpreter ();
@@ -361,14 +365,14 @@ weechat_python_load (const char *filename)
                                          "module"),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
         fclose (fp);
-	
+        
         Py_EndInterpreter (python_current_interpreter);
         /* PyEval_ReleaseLock (); */
 
         return 0;
     }
 
-    /* adding $weechat_dir/python in $PYTHONPATH */    
+    /* adding $weechat_dir/python in $PYTHONPATH */
     python_path = PySys_GetObject ("path");
     weechat_home = weechat_info_get ("weechat_dir", "");
     if (weechat_home)
@@ -491,13 +495,13 @@ weechat_python_load (const char *filename)
 
     fclose (fp);
     
-    if (python_current_script == NULL)
+    if (!python_registered_script)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, filename);
-	
+        
         if (PyErr_Occurred ())
             PyErr_Print ();
         Py_EndInterpreter (python_current_interpreter);
@@ -505,6 +509,7 @@ weechat_python_load (const char *filename)
 
         return 0;
     }
+    python_current_script = python_registered_script;
     
     python_current_script->interpreter = (PyThreadState *) python_current_interpreter;
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
@@ -536,16 +541,19 @@ weechat_python_unload (struct t_plugin_script *script)
     void *interpreter;
     PyThreadState *old_interpreter;
     
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    PYTHON_PLUGIN_NAME, script->name);
+    if ((weechat_python_plugin->debug >= 1) || !python_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        PYTHON_PLUGIN_NAME, script->name);
+    }
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
         r = (int *) weechat_python_exec (script, WEECHAT_SCRIPT_EXEC_INT,
-					 script->shutdown_func, NULL);
-	if (r)
-	    free (r);
+                                         script->shutdown_func, NULL);
+        if (r)
+            free (r);
     }
 
     old_interpreter = PyThreadState_Swap (NULL);
@@ -603,6 +611,38 @@ weechat_python_unload_all ()
     }
 }
 
+/*
+ * weechat_python_reload_name: reload a Python script by name
+ */
+
+void
+weechat_python_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_python_plugin, python_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_python_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            PYTHON_PLUGIN_NAME, name);
+            weechat_python_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), PYTHON_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_python_cmd: callback for "/python" command
  */
@@ -669,6 +709,11 @@ weechat_python_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Python script */
+            weechat_python_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Python script */
@@ -905,18 +950,17 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    /* make C compiler happy */
-    (void) plugin;
-    
     /* unload all scripts */
-    weechat_python_unload_all ();
+    python_quiet = 1;
+    script_end (plugin, &python_scripts, &weechat_python_unload_all);
+    python_quiet = 0;
     
     /* free Python interpreter */
     if (python_mainThreadState != NULL)
     {
-	/* PyEval_AcquireLock (); */
+        /* PyEval_AcquireLock (); */
         PyThreadState_Swap (python_mainThreadState);
-	/* PyEval_ReleaseLock (); */
+        /* PyEval_ReleaseLock (); */
         python_mainThreadState = NULL;
     }
     
