@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,12 +16,18 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-ruby.c: Ruby plugin for WeeChat */
+/*
+ * weechat-ruby.c: ruby plugin for WeeChat
+ */
 
 
 #undef _
 
 #include <ruby.h>
+#if defined(RUBY_VERSION) && RUBY_VERSION >=19
+#include <ruby/encoding.h>
+#endif
+
 #include <stdarg.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -31,12 +37,28 @@
 #include "weechat-ruby.h"
 #include "weechat-ruby-api.h"
 
+#ifndef StringValuePtr
+#define StringValuePtr(s) STR2CSTR(s)
+#endif
+#ifndef RARRAY_LEN
+#define RARRAY_LEN(s) RARRAY(s)->len
+#endif
+#ifndef RARRAY_PTR
+#define RARRAY_PTR(s) RARRAY(s)->ptr
+#endif
+#ifndef RSTRING_LEN
+#define RSTRING_LEN(s) RSTRING(s)->len
+#endif
+#ifndef RSTRING_PTR
+#define RSTRING_PTR(s) RSTRING(s)->ptr
+#endif
+
 
 WEECHAT_PLUGIN_NAME(RUBY_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Ruby plugin for WeeChat");
 WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_ruby_plugin = NULL;
 
@@ -44,20 +66,23 @@ int ruby_quiet = 0;
 struct t_plugin_script *ruby_scripts = NULL;
 struct t_plugin_script *last_ruby_script = NULL;
 struct t_plugin_script *ruby_current_script = NULL;
+struct t_plugin_script *ruby_registered_script = NULL;
 const char *ruby_current_script_filename = NULL;
 
-/* string used to execute action "install":
-   when signal "ruby_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "ruby_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *ruby_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "ruby_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "ruby_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *ruby_action_remove_list = NULL;
 
 VALUE ruby_mWeechat, ruby_mWeechatOutputs;
@@ -75,20 +100,20 @@ typedef struct protect_call_arg {
 } protect_call_arg_t;
 
 
-/* 
+/*
  * protect_funcall0 : used to protect a function call
  */
 
-static VALUE 
-protect_funcall0(VALUE arg)
+static VALUE
+protect_funcall0 (VALUE arg)
 {
-    return rb_funcall2(((protect_call_arg_t *) arg)->recv,
-                       ((protect_call_arg_t *) arg)->mid,
-                       ((protect_call_arg_t *) arg)->argc,
-                       ((protect_call_arg_t *) arg)->argv);
+    return rb_funcall2 (((protect_call_arg_t *)arg)->recv,
+                        ((protect_call_arg_t *)arg)->mid,
+                        ((protect_call_arg_t *)arg)->argc,
+                        ((protect_call_arg_t *)arg)->argv);
 }
 
-/* 
+/*
  * rb_protect_funcall : function call in protect mode
  */
 
@@ -118,12 +143,90 @@ rb_protect_funcall (VALUE recv, ID mid, int *state, int argc, ...)
 }
 
 /*
- * weechat_ruby_exec: execute a Ruby script
+ *  weechat_ruby_print_exception: display ruby exception
+ */
+
+int
+weechat_ruby_print_exception (VALUE err)
+{
+    VALUE backtrace;
+    int i;
+    int ruby_error;
+    char* line;
+    char* cline;
+    char* err_msg;
+    char* err_class;
+    
+    backtrace = rb_protect_funcall (err, rb_intern("backtrace"),
+                                    &ruby_error, 0);
+    err_msg = STR2CSTR(rb_protect_funcall(err, rb_intern("message"),
+                                          &ruby_error, 0));
+    err_class = STR2CSTR(rb_protect_funcall(rb_protect_funcall(err,
+                                                               rb_intern("class"),
+                                                               &ruby_error, 0),
+                                            rb_intern("name"), &ruby_error, 0));
+    
+    if (strcmp (err_class, "SyntaxError") == 0)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: error: %s"),
+                        weechat_prefix ("error"), RUBY_PLUGIN_NAME,
+                        STR2CSTR(rb_inspect(err)));
+    }
+    else
+    {
+        for (i = 0; i < RARRAY_LEN(backtrace); i++)
+        {
+            line = STR2CSTR(RARRAY_PTR(backtrace)[i]);
+            cline = NULL;
+            if (i == 0)
+            {
+                cline = (char *)calloc (strlen (line) + 2 + strlen (err_msg) +
+                                        3 + strlen (err_class) + 1,
+                                        sizeof (char));
+                if (cline)
+                {
+                    strcat (cline, line);
+                    strcat (cline, ": ");
+                    strcat (cline, err_msg);
+                    strcat (cline, " (");
+                    strcat (cline, err_class);
+                    strcat (cline, ")");
+                }
+            }
+            else
+            {
+                cline = (char *)calloc(strlen (line) + strlen ("     from ") + 1,
+                                       sizeof (char));
+                if (cline)
+                {
+                    strcat (cline, "     from ");
+                    strcat (cline, line);
+                }
+            }
+            if (cline)
+            {
+                weechat_printf (NULL,
+                                weechat_gettext ("%s%s: error: %s"),
+                                weechat_prefix ("error"), RUBY_PLUGIN_NAME,
+                                cline);
+            }
+            
+            if (cline)
+                free (cline);
+        }
+    }
+    
+    return 0;
+}
+
+/*
+ * weechat_ruby_exec: call a ruby command
  */
 
 void *
 weechat_ruby_exec (struct t_plugin_script *script,
-		   int ret_type, const char *function, char **argv)
+                   int ret_type, const char *function, char **argv)
 {
     VALUE rc, err;
     int ruby_error, *ret_i;
@@ -238,51 +341,48 @@ weechat_ruby_exec (struct t_plugin_script *script,
     
     if (ruby_error)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to run function \"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, function);
-	
-	err = rb_inspect(rb_gv_get("$!"));
-	weechat_printf (NULL,
-                        weechat_gettext ("%s%s: error: \"%s\""),
-                        weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                        STR2CSTR(err));
         
-	return NULL;
+        err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+        
+        return NULL;
     }
     
     if ((TYPE(rc) == T_STRING) && (ret_type == WEECHAT_SCRIPT_EXEC_STRING))
     {
-	if (STR2CSTR (rc))
-	    ret_value = strdup (STR2CSTR (rc));
-	else
-	    ret_value = NULL;
+        if (STR2CSTR (rc))
+            ret_value = strdup (STR2CSTR (rc));
+        else
+            ret_value = NULL;
     }
     else if ((TYPE(rc) == T_FIXNUM) && (ret_type == WEECHAT_SCRIPT_EXEC_INT))
     {
-	ret_i = malloc (sizeof (*ret_i));
-	if (ret_i)
-	    *ret_i = NUM2INT(rc);
-	ret_value = ret_i;
+        ret_i = malloc (sizeof (*ret_i));
+        if (ret_i)
+            *ret_i = NUM2INT(rc);
+        ret_value = ret_i;
     }
     else
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"%s\" must return a "
                                          "valid value"),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, function);
         ruby_current_script = old_ruby_current_script;
-	return WEECHAT_RC_OK;
+        return WEECHAT_RC_OK;
     }
     
     if (ret_value == NULL)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: not enough memory in function "
                                          "\"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, function);
         ruby_current_script = old_ruby_current_script;
-	return NULL;
+        return NULL;
     }
     
     ruby_current_script = old_ruby_current_script;
@@ -294,7 +394,7 @@ weechat_ruby_exec (struct t_plugin_script *script,
  * weechat_ruby_output: redirection for stdout and stderr
  */
 
-static VALUE 
+static VALUE
 weechat_ruby_output (VALUE self, VALUE str)
 {
     char *msg, *p, *m;
@@ -307,32 +407,32 @@ weechat_ruby_output (VALUE self, VALUE str)
     m = msg;
     while ((p = strchr (m, '\n')) != NULL)
     {
-	*p = '\0';
-	if (strlen (m) + strlen (ruby_buffer_output) > 0)
+        *p = '\0';
+        if (strlen (m) + strlen (ruby_buffer_output) > 0)
         {
             weechat_printf (NULL,
                             weechat_gettext ("%s%s: stdout/stderr: %s%s"),
                             weechat_prefix ("error"), RUBY_PLUGIN_NAME,
                             ruby_buffer_output, m);
         }
-	*p = '\n';
-	ruby_buffer_output[0] = '\0';
-	m = ++p;
+        *p = '\n';
+        ruby_buffer_output[0] = '\0';
+        m = ++p;
     }
     
     if (strlen(m) + strlen(ruby_buffer_output) > sizeof(ruby_buffer_output))
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: stdout/stderr: %s%s"),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME,
                         ruby_buffer_output, m);
-	ruby_buffer_output[0] = '\0';
+        ruby_buffer_output[0] = '\0';
     }
     else
-	strcat (ruby_buffer_output, m);
+        strcat (ruby_buffer_output, m);
 
     if (msg)
-	free (msg);
+        free (msg);
     
     return Qnil;
 }
@@ -341,7 +441,7 @@ weechat_ruby_output (VALUE self, VALUE str)
  * weechat_ruby_output_flush: just for compatibility
  */
 
-static VALUE 
+static VALUE
 weechat_ruby_output_flush (VALUE self)
 {
     /* make C compiler happy */
@@ -379,6 +479,7 @@ weechat_ruby_load (const char *filename)
     }
     
     ruby_current_script = NULL;
+    ruby_registered_script = NULL;
     
     snprintf (modname, sizeof(modname), "%s%d", MOD_NAME_PREFIX, ruby_num);
     ruby_num++;
@@ -388,24 +489,19 @@ weechat_ruby_load (const char *filename)
     ruby_current_script_filename = filename;
     
     ruby_retcode = rb_protect_funcall (curModule, rb_intern("load_eval_file"),
-				       &ruby_error, 1, rb_str_new2(filename));
+                                       &ruby_error, 1, rb_str_new2(filename));
     
     if (ruby_retcode == Qnil)
     {
-	err = rb_inspect(rb_gv_get("$!"));
-	weechat_printf (NULL,
-                        weechat_gettext ("%s%s: error: \"%s\""),
-                        weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                        STR2CSTR(err));
-	return 0;
+        err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+        return 0;
     }
     
     if (NUM2INT(ruby_retcode) != 0)
     {
-	VALUE ruby_eval_error;
-	
-	switch (NUM2INT(ruby_retcode))
-	{
+        switch (NUM2INT(ruby_retcode))
+        {
             case 1:
                 weechat_printf (NULL,
                                 weechat_gettext ("%s%s: unable to read file "
@@ -428,21 +524,14 @@ weechat_ruby_load (const char *filename)
                                 weechat_prefix ("error"), RUBY_PLUGIN_NAME,
                                 filename);
                 break;
-	}
-	
-	if (NUM2INT(ruby_retcode) == 1 || NUM2INT(ruby_retcode) == 2)
-	{
-	    ruby_eval_error = rb_iv_get(curModule, "@load_eval_file_error");
-	    if (ruby_eval_error)
-            {
-		weechat_printf (NULL,
-                                weechat_gettext ("%s%s: error: %s"),
-                                weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                                STR2CSTR(ruby_eval_error));
-            }
-	}
-	
-	return 0;
+        }
+
+        if (NUM2INT(ruby_retcode) == 1 || NUM2INT(ruby_retcode) == 2)
+        {
+            weechat_ruby_print_exception(rb_iv_get(curModule, "@load_eval_file_error"));
+        }
+        
+        return 0;
     }
     
     ruby_retcode = rb_protect_funcall (curModule, rb_intern("weechat_init"),
@@ -450,35 +539,33 @@ weechat_ruby_load (const char *filename)
     
     if (ruby_error)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to eval function "
                                          "\"weechat_init\" in file \"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, filename);
-	
-	err = rb_inspect(rb_gv_get("$!"));
-	weechat_printf (NULL,
-                        weechat_gettext ("%s%s: error: \"%s\""),
-                        weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                        STR2CSTR(err));
-	
-	if (ruby_current_script != NULL)
+        
+        err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+
+        if (ruby_current_script != NULL)
         {
-	    script_remove (weechat_ruby_plugin,
+            script_remove (weechat_ruby_plugin,
                            &ruby_scripts, &last_ruby_script,
                            ruby_current_script);
         }
-        
-	return 0;
+
+        return 0;
     }
     
-    if (ruby_current_script == NULL)
+    if (!ruby_registered_script)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, filename);
         return 0;
     }
+    ruby_current_script = ruby_registered_script;
     
     ruby_current_script->interpreter = (VALUE *) curModule;
     rb_gc_register_address (ruby_current_script->interpreter);
@@ -510,18 +597,21 @@ weechat_ruby_unload (struct t_plugin_script *script)
     char *ruby_argv[1] = { NULL };
     void *interpreter;
     
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    RUBY_PLUGIN_NAME, script->name);
+    if ((weechat_ruby_plugin->debug >= 1) || !ruby_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        RUBY_PLUGIN_NAME, script->name);
+    }
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
         r = (int *) weechat_ruby_exec (script,
                                        WEECHAT_SCRIPT_EXEC_INT,
-				       script->shutdown_func,
+                                       script->shutdown_func,
                                        ruby_argv);
-	if (r)
-	    free (r);
+        if (r)
+            free (r);
     }
     
     interpreter = script->interpreter;
@@ -534,7 +624,7 @@ weechat_ruby_unload (struct t_plugin_script *script)
                    script);
     
     if (interpreter)
-	rb_gc_unregister_address (interpreter);
+        rb_gc_unregister_address (interpreter);
 }
 
 /*
@@ -562,6 +652,38 @@ weechat_ruby_unload_name (const char *name)
     }
 }
 
+/*
+ * weechat_ruby_reload_name: reload a Ruby script by name
+ */
+
+void
+weechat_ruby_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_ruby_plugin, ruby_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_ruby_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            RUBY_PLUGIN_NAME, name);
+            weechat_ruby_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), RUBY_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_ruby_unload_all: unload all Ruby scripts
  */
@@ -571,7 +693,7 @@ weechat_ruby_unload_all ()
 {
     while (ruby_scripts)
     {
-	weechat_ruby_unload (ruby_scripts);
+        weechat_ruby_unload (ruby_scripts);
     }
 }
 
@@ -641,6 +763,11 @@ weechat_ruby_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Ruby script */
+            weechat_ruby_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Ruby script */
@@ -819,44 +946,59 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
         {
             "$stdout = WeechatOutputs\n"
             "$stderr = WeechatOutputs\n"
+            "begin"
+            "  if RUBY_VERSION.split('.')[1] == '9'\n"
+            "    require 'enc/encdb.so'\n"
+            "    require 'enc/trans/transdb.so'\n"
+            "\n"
+            "    require 'thread'\n"
+            "    class ::Mutex\n"
+            "      def synchronize(*args)\n"
+            "        yield\n"
+            "      end\n"
+            "    end\n"
+            "    require 'rubygems'\n"
+            "    $LOAD_PATH.concat Gem.latest_load_paths\n"
+            "  else\n"
+            "    require 'rubygems'\n"
+            "  end\n"
+            "rescue LoadError\n"
+            "end\n"
             "\n"
             "class Module\n"
-            "  @load_eval_file_error = ''\n"
             "\n"
             "  def load_eval_file (file)\n"
             "    lines = ''\n"
-	    "    begin\n"
-	    "      f = File.open(file, 'r')\n"
-	    "      lines = f.readlines.join\n"
-	    "    rescue => e\n"
-	    "      @load_eval_file_error = e\n"
-	    "      return 1\n"
-	    "    end\n"
-	    "\n"	   
-	    "    begin\n"
-	    "      module_eval(lines)\n"
-	    "    rescue => e\n"
-	    "      @load_eval_file_error = e\n"
-	    "      return 2\n"
-	    "    end\n"
-	    "\n"
-	    "    has_init = false\n"
-	    "\n"
-	    "    instance_methods.each do |meth|\n"
-	    "      if meth == 'weechat_init'\n"
-	    "        has_init = true\n"
-	    "      end\n"
-	    "      module_eval('module_function :' + meth)\n"
-	    "    end\n"
-	    "\n"
-	    "    unless has_init\n"
-	    "      return 3\n"
-	    "    end\n"
-	    "\n"
-	    "    return 0\n"
-	    "  end\n"
-	    "end\n"
-	};
+            "    begin\n"
+            "      lines = File.read(file)\n"
+            "    rescue => e\n"
+            "      return 1\n"
+            "    end\n"
+            "\n"
+            "    begin\n"
+            "      module_eval(lines)\n"
+            "    rescue Exception => e\n"
+            "      @load_eval_file_error = e\n"
+            "      return 2\n"
+            "    end\n"
+            "\n"
+            "    has_init = false\n"
+            "\n"
+            "    instance_methods.each do |meth|\n"
+            "      if meth.to_s == 'weechat_init'\n"
+            "        has_init = true\n"
+            "      end\n"
+            "      module_eval('module_function :' + meth)\n"
+            "    end\n"
+            "\n"
+            "    unless has_init\n"
+            "      return 3\n"
+            "    end\n"
+            "\n"
+            "    return 0\n"
+            "  end\n"
+            "end\n"
+        };
     
     weechat_ruby_plugin = plugin;
     
@@ -865,6 +1007,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     /* init stdout/stderr buffer */
     ruby_buffer_output[0] = '\0';
     
+#if defined(RUBY_VERSION) && RUBY_VERSION >= 19
+    RUBY_INIT_STACK;
+#endif
+    
     ruby_init ();
     ruby_init_loadpath ();
     ruby_script ("__weechat_plugin__");
@@ -886,16 +1032,13 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     rb_eval_string_protect(weechat_ruby_code, &ruby_error);
     if (ruby_error)
     {
-	VALUE ruby_error_info = rb_inspect(ruby_errinfo);
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to eval WeeChat ruby "
                                          "internal code"),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME);
-	weechat_printf (NULL,
-                        weechat_gettext ("%s%s: error: %s"),
-                        weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                        STR2CSTR(ruby_error_info));
-	return WEECHAT_RC_ERROR;
+        VALUE err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+        return WEECHAT_RC_ERROR;
     }
     
     ruby_quiet = 1;
@@ -928,13 +1071,12 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    /* make C compiler happy */
-    (void) plugin;
-    
     /* unload all scripts */
-    weechat_ruby_unload_all ();
+    ruby_quiet = 1;
+    script_end (plugin, &ruby_scripts, &weechat_ruby_unload_all);
+    ruby_quiet = 0;
     
-    ruby_finalize();
+    ruby_cleanup (0);
     
     return WEECHAT_RC_OK;
 }
