@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* script-api.c: script API */
+/*
+ * script-api.c: script API functions, used by script plugins (perl/python/..)
+ */
 
 
 #include <stdlib.h>
@@ -108,14 +110,14 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
                                                     const char *value),
                                const char *function_read,
                                const char *data_read,
-                               void (*callback_write)(void *data,
-                                                      struct t_config_file *config_file,
-                                                      const char *section_name),
+                               int (*callback_write)(void *data,
+                                                     struct t_config_file *config_file,
+                                                     const char *section_name),
                                const char *function_write,
                                const char *data_write,
-                               void (*callback_write_default)(void *data,
-                                                              struct t_config_file *config_file,
-                                                              const char *section_name),
+                               int (*callback_write_default)(void *data,
+                                                             struct t_config_file *config_file,
+                                                             const char *section_name),
                                const char *function_write_default,
                                const char *data_write_default,
                                int (*callback_create_option)(void *data,
@@ -926,8 +928,10 @@ script_api_hook_connect (struct t_weechat_plugin *weechat_plugin,
                          struct t_plugin_script *script,
                          const char *proxy, const char *address, int port,
                          int sock, int ipv6, void *gnutls_sess,
+                         void *gnutls_cb, int gnutls_dhkey_size,
                          const char *local_hostname,
                          int (*callback)(void *data, int status,
+                                         int gnutls_rc,
                                          const char *error,
                                          const char *ip_address),
                          const char *function,
@@ -941,7 +945,8 @@ script_api_hook_connect (struct t_weechat_plugin *weechat_plugin,
         return NULL;
     
     new_hook = weechat_hook_connect (proxy, address, port, sock, ipv6,
-                                     gnutls_sess, local_hostname, callback,
+                                     gnutls_sess, gnutls_cb, gnutls_dhkey_size,
+                                     local_hostname, callback,
                                      new_script_callback);
     if (!new_hook)
     {
@@ -1166,6 +1171,7 @@ script_api_hook_info (struct t_weechat_plugin *weechat_plugin,
                       struct t_plugin_script *script,
                       const char *info_name,
                       const char *description,
+                      const char *args_description,
                       const char *(*callback)(void *data,
                                               const char *info_name,
                                               const char *arguments),
@@ -1179,7 +1185,7 @@ script_api_hook_info (struct t_weechat_plugin *weechat_plugin,
     if (!new_script_callback)
         return NULL;
     
-    new_hook = weechat_hook_info (info_name, description,
+    new_hook = weechat_hook_info (info_name, description, args_description,
                                   callback, new_script_callback);
     if (!new_hook)
     {
@@ -1206,6 +1212,8 @@ script_api_hook_infolist (struct t_weechat_plugin *weechat_plugin,
                           struct t_plugin_script *script,
                           const char *infolist_name,
                           const char *description,
+                          const char *pointer_description,
+                          const char *args_description,
                           struct t_infolist *(*callback)(void *data,
                                                          const char *infolist_name,
                                                          void *pointer,
@@ -1221,6 +1229,7 @@ script_api_hook_infolist (struct t_weechat_plugin *weechat_plugin,
         return NULL;
     
     new_hook = weechat_hook_infolist (infolist_name, description,
+                                      pointer_description, args_description,
                                       callback, new_script_callback);
     if (!new_hook)
     {
