@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,12 +41,12 @@ extern struct t_config_section *script_api_config_new_section (struct t_weechat_
                                                                                     const char *value),
                                                                const char *function_read,
                                                                const char *data_read,
-                                                               void (*callback_write)(void *data,
+                                                               int (*callback_write)(void *data,
                                                                                       struct t_config_file *config_file,
                                                                                       const char *section_name),
                                                                const char *function_write,
                                                                const char *data_write,
-                                                               void (*callback_write_default)(void *data,
+                                                               int (*callback_write_default)(void *data,
                                                                                               struct t_config_file *config_file,
                                                                                               const char *section_name),
                                                                const char *function_write_default,
@@ -169,10 +169,12 @@ extern struct t_hook *script_api_hook_connect (struct t_weechat_plugin *weechat_
                                                int port,
                                                int sock,
                                                int ipv6,
-                                               void *gnutls_sess,
+                                               void *gnutls_sess, void *gnutls_cb,
+                                               int gnutls_dhkey_size,
                                                const char *local_hostname,
                                                int (*callback)(void *data,
                                                                int status,
+                                                               int gnutls_rc,
                                                                const char *error,
                                                                const char *ip_address),
                                                const char *function,
@@ -234,6 +236,7 @@ extern struct t_hook *script_api_hook_info (struct t_weechat_plugin *weechat_plu
                                             struct t_plugin_script *script,
                                             const char *info_name,
                                             const char *description,
+                                            const char *args_description,
                                             const char *(*callback)(void *data,
                                                                     const char *info_name,
                                                                     const char *arguments),
@@ -243,6 +246,8 @@ extern struct t_hook *script_api_hook_infolist (struct t_weechat_plugin *weechat
                                                 struct t_plugin_script *script,
                                                 const char *infolist_name,
                                                 const char *description,
+                                                const char *pointer_description,
+                                                const char *args_description,
                                                 struct t_infolist *(*callback)(void *data,
                                                                                const char *infolist_name,
                                                                                void *pointer,
