@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* script.c: script interface for WeeChat plugins */
+/*
+ * script.c: common functions used by script plugins (perl/python/..)
+ */
 
 
 #include <stdlib.h>
@@ -172,7 +174,7 @@ script_upgrade_set_buffer_callbacks (struct t_weechat_plugin *weechat_plugin,
 }
 
 /*
- * script_init: initialize script
+ * script_init: initialize script plugin
  */
 
 void
@@ -208,8 +210,8 @@ script_init (struct t_weechat_plugin *weechat_plugin,
                                            struct t_gui_buffer *buffer))
 {
     char *string, *completion;
-    char infolist_description[512], signal_name[128];
-    int length, i, upgrading;
+    char signal_name[128];
+    int length, i, upgrading, auto_load_scripts;
     
     /* read script configuration */
     script_config_read (weechat_plugin);
@@ -248,7 +250,7 @@ script_init (struct t_weechat_plugin *weechat_plugin,
                                              " || listfull %s"
                                              " || load %(filename)"
                                              " || autoload"
-                                             " || reload"
+                                             " || reload %s"
                                              " || unload %s",
                                              "%s",
                                              string);
@@ -257,7 +259,7 @@ script_init (struct t_weechat_plugin *weechat_plugin,
                           N_("list/load/unload scripts"),
                           N_("[list [name]] | [listfull [name]] | "
                              "[load filename] | [autoload] | "
-                             "[reload] | [unload [name]]"),
+                             "[reload [name]] | [unload [name]]"),
                           N_("filename: script (file) to load\n"
                              "name: a script name\n\n"
                              "Without argument, this command "
@@ -277,10 +279,9 @@ script_init (struct t_weechat_plugin *weechat_plugin,
         snprintf (string, length, "%s_script", weechat_plugin->name);
         weechat_hook_completion (string, N_("list of scripts"),
                                  callback_completion, NULL);
-        snprintf (infolist_description, sizeof (infolist_description),
-                  /* TRANSLATORS: %s is language (for example "perl") */
-                  N_("list of %s scripts"), weechat_plugin->name);
-        weechat_hook_infolist (string, infolist_description,
+        weechat_hook_infolist (string, N_("list of scripts"),
+                               N_("script pointer (optional)"),
+                               N_("script name (can start or end with \"*\" as joker) (optional)"),
                                callback_infolist, NULL);
         free (string);
     }
@@ -299,16 +300,29 @@ script_init (struct t_weechat_plugin *weechat_plugin,
               weechat_plugin->name);
     weechat_hook_signal (signal_name, callback_signal_script_action, NULL);
     
-    /* autoload scripts */
-    script_auto_load (weechat_plugin, callback_load_file);
-    
-    /* actions after upgrade */
+    /* parse arguments */
     upgrading = 0;
+    auto_load_scripts = 1;
     for (i = 0; i < argc; i++)
     {
-        if (weechat_strcasecmp (argv[i], "--upgrade") == 0)
+        if (strcmp (argv[i], "--upgrade") == 0)
+        {
             upgrading = 1;
+        }
+        else if ((strcmp (argv[i], "-s") == 0)
+                 || (strcmp (argv[i], "--no-script") == 0))
+        {
+            auto_load_scripts = 0;
+        }
     }
+    
+    /* autoload scripts */
+    if (auto_load_scripts)
+    {
+        script_auto_load (weechat_plugin, callback_load_file);
+    }
+    
+    /* set buffer callbacks after upgrade */
     if (upgrading)
     {
         script_upgrade_set_buffer_callbacks (weechat_plugin,
@@ -439,24 +453,15 @@ struct t_plugin_script *
 script_search_by_full_name (struct t_plugin_script *scripts,
                             const char *full_name)
 {
-    char *full_name_copy, *base_name;
+    char *base_name;
     struct t_plugin_script *ptr_script;
     
-    full_name_copy = strdup (full_name);
-    
-    if (full_name_copy)
+    for (ptr_script = scripts; ptr_script;
+         ptr_script = ptr_script->next_script)
     {
-        for (ptr_script = scripts; ptr_script;
-             ptr_script = ptr_script->next_script)
-        {
-            base_name = basename (ptr_script->filename);
-            if (strcmp (base_name, full_name) == 0)
-            {
-                free (full_name_copy);
-                return ptr_script;
-            }
-        }
-        free (full_name_copy);
+        base_name = basename (ptr_script->filename);
+        if (strcmp (base_name, full_name) == 0)
+            return ptr_script;
     }
     
     /* script not found */
@@ -555,7 +560,7 @@ script_search_path (struct t_weechat_plugin *weechat_plugin,
 }
 
 /*
- * script_find_pos: find position for a script (for sorting scripts list) 
+ * script_find_pos: find position for a script (for sorting scripts list)
  */
 
 struct t_plugin_script *
@@ -915,7 +920,8 @@ script_action_install (struct t_weechat_plugin *weechat_plugin,
                        int (*script_load)(const char *filename),
                        char **list)
 {
-    char **argv, *name, *base_name, *new_path, *autoload_path, *symlink_path;
+    char **argv, *name, *ptr_base_name, *base_name, *new_path, *autoload_path;
+    char *symlink_path;
     const char *dir_home, *dir_separator;
     int argc, i, length;
     struct t_plugin_script *ptr_script;
@@ -930,67 +936,71 @@ script_action_install (struct t_weechat_plugin *weechat_plugin,
                 name = strdup (argv[i]);
                 if (name)
                 {
-                    base_name = basename (name);
-                    
-                    /* unload script, if script is loaded */
-                    ptr_script = script_search_by_full_name (scripts, base_name);
-                    if (ptr_script)
-                        (*script_unload) (ptr_script);
-                    
-                    /* remove script file(s) */
-                    script_remove_file (weechat_plugin, base_name, 0);
-                    
-                    /* move file from install dir to language dir */
-                    dir_home = weechat_info_get ("weechat_dir", "");
-                    length = strlen (dir_home) + strlen (weechat_plugin->name) +
-                        strlen (base_name) + 16;
-                    new_path = malloc (length);
-                    if (new_path)
+                    ptr_base_name = basename (name);
+                    base_name = strdup (ptr_base_name);
+                    if (base_name)
                     {
-                        snprintf (new_path, length, "%s/%s/%s",
-                                  dir_home, weechat_plugin->name, base_name);
-                        if (rename (name, new_path) == 0)
+                        /* unload script, if script is loaded */
+                        ptr_script = script_search_by_full_name (scripts, base_name);
+                        if (ptr_script)
+                            (*script_unload) (ptr_script);
+                        
+                        /* remove script file(s) */
+                        script_remove_file (weechat_plugin, base_name, 0);
+                        
+                        /* move file from install dir to language dir */
+                        dir_home = weechat_info_get ("weechat_dir", "");
+                        length = strlen (dir_home) + strlen (weechat_plugin->name) +
+                            strlen (base_name) + 16;
+                        new_path = malloc (length);
+                        if (new_path)
                         {
-                            /* make link in autoload dir */
-                            length = strlen (dir_home) +
-                                strlen (weechat_plugin->name) + 8 +
-                                strlen (base_name) + 16;
-                            autoload_path = malloc (length);
-                            if (autoload_path)
+                            snprintf (new_path, length, "%s/%s/%s",
+                                      dir_home, weechat_plugin->name, base_name);
+                            if (rename (name, new_path) == 0)
                             {
-                                snprintf (autoload_path, length,
-                                          "%s/%s/autoload/%s",
-                                          dir_home, weechat_plugin->name,
-                                          base_name);
-                                dir_separator = weechat_info_get ("dir_separator", "");
-                                length = 2 + strlen (dir_separator) +
-                                    strlen (base_name) + 1;
-                                symlink_path = malloc (length);
-                                if (symlink_path)
+                                /* make link in autoload dir */
+                                length = strlen (dir_home) +
+                                    strlen (weechat_plugin->name) + 8 +
+                                    strlen (base_name) + 16;
+                                autoload_path = malloc (length);
+                                if (autoload_path)
                                 {
-                                    snprintf (symlink_path, length, "..%s%s",
-                                              dir_separator, base_name);
-                                    symlink (symlink_path, autoload_path);
-                                    free (symlink_path);
+                                    snprintf (autoload_path, length,
+                                              "%s/%s/autoload/%s",
+                                              dir_home, weechat_plugin->name,
+                                              base_name);
+                                    dir_separator = weechat_info_get ("dir_separator", "");
+                                    length = 2 + strlen (dir_separator) +
+                                        strlen (base_name) + 1;
+                                    symlink_path = malloc (length);
+                                    if (symlink_path)
+                                    {
+                                        snprintf (symlink_path, length, "..%s%s",
+                                                  dir_separator, base_name);
+                                        symlink (symlink_path, autoload_path);
+                                        free (symlink_path);
+                                    }
+                                    free (autoload_path);
                                 }
-                                free (autoload_path);
+                                
+                                /* load script */
+                                (*script_load) (new_path);
                             }
-                            
-                            /* load script */
-                            (*script_load) (new_path);
+                            else
+                            {
+                                weechat_printf (NULL,
+                                                _("%s%s: failed to move script %s "
+                                                  "to %s (%s)"),
+                                                weechat_prefix ("error"),
+                                                weechat_plugin->name,
+                                                name,
+                                                new_path,
+                                                strerror (errno));
+                            }
+                            free (new_path);
                         }
-                        else
-                        {
-                            weechat_printf (NULL,
-                                            _("%s%s: failed to move script %s "
-                                              "to %s (%s)"),
-                                            weechat_prefix ("error"),
-                                            weechat_plugin->name,
-                                            name,
-                                            new_path,
-                                            strerror (errno));
-                        }
-                        free (new_path);
+                        free (base_name);
                     }
                     free (name);
                 }
@@ -1230,6 +1240,28 @@ script_infolist_list_scripts (struct t_weechat_plugin *weechat_plugin,
     return NULL;
 }
 
+/*
+ * script_end: end script plugin
+ */
+
+void
+script_end (struct t_weechat_plugin *weechat_plugin,
+            struct t_plugin_script **scripts,
+            void (*callback_unload_all)())
+{
+    int scripts_loaded;
+    
+    scripts_loaded = (*scripts) ? 1 : 0;
+    
+    (void)(callback_unload_all) ();
+    
+    if (scripts_loaded)
+    {
+        weechat_printf (NULL, _("%s: scripts unloaded"),
+                        weechat_plugin->name);
+    }
+}
+
 /*
  * script_print_log: print script infos in log (usually for crash dump)
  */
