@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Dmitry Kobylin <fnfal@academ.tsc.ru>
+ * Copyright (c) 2008-2010 by Dmitry Kobylin <fnfal@academ.tsc.ru>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-tcl.c: Tcl plugin for WeeChat */
+/*
+ * weechat-tcl.c: tcl plugin for WeeChat
+ */
 
 
 #undef _
@@ -38,7 +40,7 @@ WEECHAT_PLUGIN_NAME(TCL_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Tcl plugin for WeeChat");
 WEECHAT_PLUGIN_AUTHOR("Dmitry Kobylin <fnfal@academ.tsc.ru>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_tcl_plugin = NULL;
 
@@ -46,20 +48,23 @@ int tcl_quiet = 0;
 struct t_plugin_script *tcl_scripts = NULL;
 struct t_plugin_script *last_tcl_script = NULL;
 struct t_plugin_script *tcl_current_script = NULL;
+struct t_plugin_script *tcl_registered_script = NULL;
 const char *tcl_current_script_filename = NULL;
 
-/* string used to execute action "install":
-   when signal "tcl_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "tcl_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *tcl_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "tcl_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "tcl_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *tcl_action_remove_list = NULL;
 
 Tcl_Interp* cinterp;
@@ -85,7 +90,7 @@ weechat_tcl_exec (struct t_plugin_script *script,
     tcl_current_script = script;
     interp = (Tcl_Interp*)script->interpreter;
 
-    if (function && function[0]) 
+    if (function && function[0])
     {
         cmdlist = Tcl_NewListObj(0,NULL);
         Tcl_IncrRefCount(cmdlist); /* +1 */
@@ -116,7 +121,7 @@ weechat_tcl_exec (struct t_plugin_script *script,
         if (ret_type == WEECHAT_SCRIPT_EXEC_STRING)
         {
             ret_cv = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), &i);
-            if (ret_cv && ret_cv[0])
+            if (ret_cv)
                 ret_val = (void *)strdup (ret_cv);
             else
                 ret_val = NULL;
@@ -144,7 +149,7 @@ weechat_tcl_exec (struct t_plugin_script *script,
     Tcl_ListObjReplace(interp,cmdlist,0,llength,0,NULL); /* remove elements, decrement their ref count */
     Tcl_DecrRefCount(cmdlist); /* -1 */
     weechat_printf (NULL,
-                    weechat_gettext ("%s%s unable to run function \"%s\": %s"),
+                    weechat_gettext ("%s%s: unable to run function \"%s\": %s"),
                     weechat_prefix ("error"), TCL_PLUGIN_NAME, function,
                     Tcl_GetStringFromObj (Tcl_GetObjResult (interp), &i));
     tcl_current_script = old_tcl_script;
@@ -179,6 +184,7 @@ weechat_tcl_load (const char *filename)
     }
     
     tcl_current_script = NULL;
+    tcl_registered_script = NULL;
     
     if (!(interp = Tcl_CreateInterp ())) {
         weechat_printf (NULL,
@@ -202,7 +208,7 @@ weechat_tcl_load (const char *filename)
         /* return 0; */
     }
 
-    if (!tcl_current_script)
+    if (!tcl_registered_script)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
@@ -211,6 +217,7 @@ weechat_tcl_load (const char *filename)
         Tcl_DeleteInterp (interp);
         return 0;
     }
+    tcl_current_script = tcl_registered_script;
     
     return 1;
 }
@@ -238,9 +245,12 @@ weechat_tcl_unload (struct t_plugin_script *script)
     Tcl_Interp* interp;
     void *pointer;
     
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    TCL_PLUGIN_NAME, script->name);
+    if ((weechat_tcl_plugin->debug >= 1) || !tcl_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        TCL_PLUGIN_NAME, script->name);
+    }
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
@@ -301,6 +311,38 @@ weechat_tcl_unload_all ()
     }
 }
 
+/*
+ * weechat_tcl_reload_name: reload a Tcl script by name
+ */
+
+void
+weechat_tcl_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_tcl_plugin, tcl_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_tcl_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            TCL_PLUGIN_NAME, name);
+            weechat_tcl_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), TCL_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_tcl_command_cb: callback for "/tcl" command
  */
@@ -367,6 +409,11 @@ weechat_tcl_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Tcl script */
+            weechat_tcl_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Tcl script */
@@ -572,10 +619,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    (void) plugin;
-
     /* unload all scripts */
-    weechat_tcl_unload_all ();
+    tcl_quiet = 1;
+    script_end (plugin, &tcl_scripts, &weechat_tcl_unload_all);
+    tcl_quiet = 0;
     
     return WEECHAT_RC_OK;
 }
