@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,9 +31,15 @@ struct t_gui_bar_item;
 struct t_gui_completion;
 struct t_infolist;
 struct t_weelist;
+struct timeval;
+
+/*
+ * IMPORTANT NOTE for WeeChat developers: if you update, add or remove
+ * some functions in this file, then please update API version below.
+ */
 
 /* API version (used to check that plugin has same API and can be loaded) */
-#define WEECHAT_PLUGIN_API_VERSION "20090614-02"
+#define WEECHAT_PLUGIN_API_VERSION "20100302-01"
 
 /* macros for defining plugin infos */
 #define WEECHAT_PLUGIN_NAME(__name)                                     \
@@ -89,10 +95,11 @@ struct t_weelist;
 #define WEECHAT_HOTLIST_PRIVATE                     "2"
 #define WEECHAT_HOTLIST_HIGHLIGHT                   "3"
 
-/* process return code (for callback):
-   if >= 0, then process ended and it's return code of command
-   if  < 0, then it's running or error
-*/
+/*
+ * process return code (for callback):
+ *   if >= 0, then process ended and it's return code of command
+ *   if  < 0, then it's running or error
+ */
 #define WEECHAT_HOOK_PROCESS_RUNNING                -1
 #define WEECHAT_HOOK_PROCESS_ERROR                  -2
 
@@ -129,10 +136,6 @@ struct t_weechat_plugin
     
     /* plugin functions (API) */
     
-    /* IMPORTANT NOTE for WeeChat developers: always add new API functions
-       at the END of functions, for keeping backward compatibility with
-       existing plugins */
-    
     /* plugins */
     const char *(*plugin_get_name) (struct t_weechat_plugin *plugin);
     
@@ -169,6 +172,10 @@ struct t_weechat_plugin
     void (*string_free_split_command) (char **split_command);
     char *(*string_format_size) (unsigned long size);
     char *(*string_remove_color) (const char *string, const char *replacement);
+    void (*string_encode_base64) (const char *from, int length, char *to);
+    int (*string_decode_base64) (const char *from, char *to);
+    int (*string_is_command_char) (const char *string);
+    const char *(*string_input_for_buffer) (const char *string);
     
     /* UTF-8 strings */
     int (*utf8_has_8bits) (const char *string);
@@ -189,17 +196,19 @@ struct t_weechat_plugin
     int (*utf8_pos) (const char *string, int real_pos);
     char *(*utf8_strndup) (const char *string, int length);
     
-    /* directories */
+    /* directories/files */
     int (*mkdir_home) (const char *directory, int mode);
     int (*mkdir) (const char *directory, int mode);
     int (*mkdir_parents) (const char *directory, int mode);
     void (*exec_on_files) (const char *directory, int hidden_files, void *data,
                            void (*callback)(void *data, const char *filename));
+    char *(*file_get_content) (const char *filename);
     
     /* util */
-    int (*timeval_cmp) (struct timeval *tv1, struct timeval *tv2);
-    long (*timeval_diff) (struct timeval *tv1, struct timeval *tv2);
-    void (*timeval_add) (struct timeval *tv, long interval);
+    int (*util_timeval_cmp) (struct timeval *tv1, struct timeval *tv2);
+    long (*util_timeval_diff) (struct timeval *tv1, struct timeval *tv2);
+    void (*util_timeval_add) (struct timeval *tv, long interval);
+    char *(*util_get_time_string) (const time_t *date);
     
     /* sorted lists */
     struct t_weelist *(*list_new) ();
@@ -239,13 +248,13 @@ struct t_weechat_plugin
                                                                          const char *option_name,
                                                                          const char *value),
                                                     void *callback_read_data,
-                                                    void (*callback_write)(void *data,
-                                                                           struct t_config_file *config_file,
-                                                                           const char *section_name),
+                                                    int (*callback_write)(void *data,
+                                                                          struct t_config_file *config_file,
+                                                                          const char *section_name),
                                                     void *callback_write_data,
-                                                    void (*callback_write_default)(void *data,
-                                                                                   struct t_config_file *config_file,
-                                                                                   const char *section_name),
+                                                    int (*callback_write_default)(void *data,
+                                                                                  struct t_config_file *config_file,
+                                                                                  const char *section_name),
                                                     void *callback_write_default_data,
                                                     int (*callback_create_option)(void *data,
                                                                                   struct t_config_file *config_file,
@@ -316,11 +325,11 @@ struct t_weechat_plugin
     const char *(*config_string_default) (struct t_config_option *option);
     const char *(*config_color) (struct t_config_option *option);
     const char *(*config_color_default) (struct t_config_option *option);
-    void (*config_write_option) (struct t_config_file *config_file,
-                                 struct t_config_option *option);
-    void (*config_write_line) (struct t_config_file *config_file,
-                               const char *option_name,
-                               const char *value, ...);
+    int (*config_write_option) (struct t_config_file *config_file,
+                                struct t_config_option *option);
+    int (*config_write_line) (struct t_config_file *config_file,
+                              const char *option_name,
+                              const char *value, ...);
     int (*config_write) (struct t_config_file *config_file);
     int (*config_read) (struct t_config_file *config_file);
     int (*config_reload) (struct t_config_file *config_file);
@@ -394,10 +403,12 @@ struct t_weechat_plugin
                                     int port,
                                     int sock,
                                     int ipv6,
-                                    void *gnutls_sess,
+                                    void *gnutls_sess, void *gnutls_cb,
+                                    int gnutls_dhkey_size,
                                     const char *local_hostname,
                                     int (*callback)(void *data,
                                                     int status,
+                                                    int gnutls_rc,
                                                     const char *error,
                                                     const char *ip_address),
                                     void *callback_data);
@@ -457,6 +468,7 @@ struct t_weechat_plugin
     struct t_hook *(*hook_info) (struct t_weechat_plugin *plugin,
                                  const char *info_name,
                                  const char *description,
+                                 const char *args_description,
                                  const char *(*callback)(void *data,
                                                          const char *info_name,
                                                          const char *arguments),
@@ -464,6 +476,8 @@ struct t_weechat_plugin
     struct t_hook *(*hook_infolist) (struct t_weechat_plugin *plugin,
                                      const char *infolist_name,
                                      const char *description,
+                                     const char *pointer_description,
+                                     const char *args_description,
                                      struct t_infolist *(*callback)(void *data,
                                                                     const char *infolist_name,
                                                                     void *pointer,
@@ -709,6 +723,14 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
     weechat_plugin->string_format_size(__size)
 #define weechat_string_remove_color(__string, __replacement)            \
     weechat_plugin->string_remove_color(__string, __replacement)
+#define weechat_string_encode_base64(__from, __length, __to)            \
+    weechat_plugin->string_encode_base64(__from, __length, __to)
+#define weechat_string_decode_base64(__from, __to)                      \
+    weechat_plugin->string_decode_base64(__from, __to)
+#define weechat_string_is_command_char(__string)                        \
+    weechat_plugin->string_is_command_char(__string)
+#define weechat_string_input_for_buffer(__string)                       \
+    weechat_plugin->string_input_for_buffer(__string)
 
 /* UTF-8 strings */
 #define weechat_utf8_has_8bits(__string)                                \
@@ -757,14 +779,18 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
                               __callback)                               \
     weechat_plugin->exec_on_files(__directory, __hidden_files, __data,  \
                                   __callback)
+#define weechat_file_get_content(__filename)                            \
+    weechat_plugin->file_get_content(__filename)
 
 /* util */
-#define weechat_timeval_cmp(__time1, __time2)                           \
-    weechat_plugin->timeval_cmp(__time1, __time2)
-#define weechat_timeval_diff(__time1, __time2)                          \
-    weechat_plugin->timeval_diff(__time1, __time2)
-#define weechat_timeval_add(__time, __interval)                         \
-    weechat_plugin->timeval_add(__time, __interval)
+#define weechat_util_timeval_cmp(__time1, __time2)                      \
+    weechat_plugin->util_timeval_cmp(__time1, __time2)
+#define weechat_util_timeval_diff(__time1, __time2)                     \
+    weechat_plugin->util_timeval_diff(__time1, __time2)
+#define weechat_util_timeval_add(__time, __interval)                    \
+    weechat_plugin->util_timeval_add(__time, __interval)
+#define weechat_util_get_time_string(__date)                            \
+    weechat_plugin->util_get_time_string(__date)
 
 /* sorted list */
 #define weechat_list_new()                                              \
@@ -971,10 +997,12 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
     weechat_plugin->hook_process(weechat_plugin, __command, __timeout,  \
                                  __callback, __callback_data)
 #define weechat_hook_connect(__proxy, __address, __port, __sock,        \
-                             __ipv6, __gnutls_sess, __local_hostname,   \
+                            __ipv6, __gnutls_sess, __gnutls_cb,         \
+                             __gnutls_dhkey_size, __local_hostname,     \
                              __callback, __data)                        \
     weechat_plugin->hook_connect(weechat_plugin, __proxy, __address,    \
                                  __port, __sock, __ipv6, __gnutls_sess, \
+                                 __gnutls_cb, __gnutls_dhkey_size,      \
                                  __local_hostname, __callback, __data)
 #define weechat_hook_print(__buffer, __tags, __msg, __strip__colors,    \
                            __callback, __data)                          \
@@ -1006,14 +1034,18 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
                                    __string)                            \
     weechat_plugin->hook_modifier_exec(weechat_plugin, __modifier,      \
                                        __modifier_data, __string)
-#define weechat_hook_info(__info_name, __description, __callback,       \
-                          __data)                                       \
+#define weechat_hook_info(__info_name, __description,                   \
+                          __args_description, __callback, __data)       \
     weechat_plugin->hook_info(weechat_plugin, __info_name,              \
-                              __description, __callback, __data)
+                              __description, __args_description,        \
+                              __callback, __data)
 #define weechat_hook_infolist(__infolist_name, __description,           \
-                              __callback, __data)                       \
+                              __pointer_description,                    \
+                              __args_description, __callback, __data)   \
     weechat_plugin->hook_infolist(weechat_plugin, __infolist_name,      \
-                                  __description, __callback, __data)
+                                  __description, __pointer_description, \
+                                  __args_description, __callback,       \
+                                  __data)
 #define weechat_unhook(__hook)                                          \
     weechat_plugin->unhook( __hook)
 #define weechat_unhook_all()                                            \
