@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2009 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* xfer.c: file transfer and direct chat plugin for WeeChat */
+/*
+ * xfer.c: file transfer and direct chat plugin for WeeChat
+ */
 
 
 #include <stdlib.h>
@@ -47,7 +49,7 @@ WEECHAT_PLUGIN_DESCRIPTION("Xfer (file transfer and direct chat) plugin for "
                            "WeeChat");
 WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_xfer_plugin = NULL;
 
@@ -265,6 +267,8 @@ xfer_close (struct t_xfer *xfer, enum t_xfer_status status)
     
     if (XFER_HAS_ENDED(xfer->status))
     {
+        xfer_send_signal (xfer, "xfer_ended");
+        
         if (xfer->hook_fd)
         {
             weechat_unhook (xfer->hook_fd);
@@ -388,6 +392,8 @@ xfer_send_signal (struct t_xfer *xfer, const char *signal)
                                              xfer->remote_nick);
             weechat_infolist_new_var_string (item, "local_nick",
                                              xfer->local_nick);
+            weechat_infolist_new_var_string (item, "charset_modifier",
+                                             xfer->charset_modifier);
             weechat_infolist_new_var_string (item, "filename",
                                              xfer->filename);
             snprintf (str_long, sizeof (str_long), "%lu", xfer->size);
@@ -432,11 +438,13 @@ xfer_alloc ()
     new_xfer->port = 0;
     new_xfer->remote_nick = NULL;
     new_xfer->local_nick = NULL;
+    new_xfer->charset_modifier = NULL;
     
     new_xfer->type = 0;
     new_xfer->protocol = 0;
     new_xfer->status = 0;
     new_xfer->buffer = NULL;
+    new_xfer->remote_nick_color = NULL;
     new_xfer->fast_send = weechat_config_boolean (xfer_config_network_fast_send);
     new_xfer->blocksize = weechat_config_integer (xfer_config_network_blocksize);
     new_xfer->start_time = time_now;
@@ -478,13 +486,15 @@ xfer_alloc ()
  */
 
 struct t_xfer *
-xfer_new (const char *plugin_name, const char *plugin_id, enum t_xfer_type type,
-          enum t_xfer_protocol protocol, const char *remote_nick,
-          const char *local_nick, const char *filename,
+xfer_new (const char *plugin_name, const char *plugin_id,
+          enum t_xfer_type type, enum t_xfer_protocol protocol,
+          const char *remote_nick, const char *local_nick,
+          const char *charset_modifier, const char *filename,
           unsigned long size, const char *proxy, unsigned long address,
           int port, int sock, const char *local_filename)
 {
     struct t_xfer *new_xfer;
+    const char *ptr_color;
     
     new_xfer = xfer_alloc ();
     if (!new_xfer)
@@ -507,7 +517,10 @@ xfer_new (const char *plugin_name, const char *plugin_id, enum t_xfer_type type,
     new_xfer->type = type;
     new_xfer->protocol = protocol;
     new_xfer->remote_nick = strdup (remote_nick);
+    ptr_color = weechat_info_get ("irc_nick_color", remote_nick);
+    new_xfer->remote_nick_color = (ptr_color) ? strdup (ptr_color) : NULL;
     new_xfer->local_nick = (local_nick) ? strdup (local_nick) : NULL;
+    new_xfer->charset_modifier = (charset_modifier) ? strdup (charset_modifier) : NULL;
     if (XFER_IS_FILE(type))
         new_xfer->filename = (filename) ? strdup (filename) : NULL;
     else
@@ -625,13 +638,22 @@ xfer_new (const char *plugin_name, const char *plugin_id, enum t_xfer_type type,
 void
 xfer_free (struct t_xfer *xfer)
 {
-    struct t_xfer *new_xfer_list;
+    struct t_xfer *new_xfer_list, *ptr_xfer;
+    struct t_gui_buffer *ptr_buffer;
     
     if (!xfer)
         return;
     
     if (xfer->buffer)
-        weechat_buffer_close (xfer->buffer);
+    {
+        ptr_buffer = xfer->buffer;
+        for (ptr_xfer = xfer_list; ptr_xfer; ptr_xfer = ptr_xfer->next_xfer)
+        {
+            if (ptr_xfer->buffer == ptr_buffer)
+                ptr_xfer->buffer = NULL;
+        }
+        weechat_buffer_close (ptr_buffer);
+    }
     
     /* remove xfer from list */
     if (last_xfer == xfer)
@@ -653,8 +675,12 @@ xfer_free (struct t_xfer *xfer)
         free (xfer->remote_nick);
     if (xfer->local_nick)
         free (xfer->local_nick);
+    if (xfer->charset_modifier)
+        free (xfer->charset_modifier);
     if (xfer->filename)
         free (xfer->filename);
+    if (xfer->remote_nick_color)
+        free (xfer->remote_nick_color);
     if (xfer->unterminated_message)
         free (xfer->unterminated_message);
     if (xfer->local_filename)
@@ -674,11 +700,12 @@ xfer_free (struct t_xfer *xfer)
  */
 
 int
-xfer_add_cb (void *data, const char *signal, const char *type_data, void *signal_data)
+xfer_add_cb (void *data, const char *signal, const char *type_data,
+             void *signal_data)
 {
     struct t_infolist *infolist;
     const char *plugin_name, *plugin_id, *str_type, *str_protocol;
-    const char *remote_nick, *local_nick, *filename, *proxy;
+    const char *remote_nick, *local_nick, *charset_modifier, *filename, *proxy;
     int type, protocol;
     const char *weechat_dir;
     char *dir1, *dir2, *filename2, *short_filename, *pos;
@@ -728,6 +755,7 @@ xfer_add_cb (void *data, const char *signal, const char *type_data, void *signal
     str_protocol = weechat_infolist_string (infolist, "protocol");
     remote_nick = weechat_infolist_string (infolist, "remote_nick");
     local_nick = weechat_infolist_string (infolist, "local_nick");
+    charset_modifier = weechat_infolist_string (infolist, "charset_modifier");
     filename = weechat_infolist_string (infolist, "filename");
     proxy = weechat_infolist_string (infolist, "proxy");
     protocol = XFER_NO_PROTOCOL;
@@ -875,7 +903,7 @@ xfer_add_cb (void *data, const char *signal, const char *type_data, void *signal
                 if (sock > 0)
                 {
                     memset (&addr, 0, sizeof (struct sockaddr_in));
-                    length = sizeof (addr);        
+                    length = sizeof (addr);
                     getsockname (sock, (struct sockaddr *) &addr, &length);
                     addr.sin_family = AF_INET;
                 }
@@ -999,13 +1027,13 @@ xfer_add_cb (void *data, const char *signal, const char *type_data, void *signal
     /* add xfer entry and listen to socket if type is file or chat "send" */
     if (XFER_IS_FILE(type))
         ptr_xfer = xfer_new (plugin_name, plugin_id, type, protocol,
-                             remote_nick, local_nick, short_filename,
-                             file_size, proxy, local_addr, port, sock,
-                             filename2);
+                             remote_nick, local_nick, charset_modifier,
+                             short_filename, file_size, proxy, local_addr,
+                             port, sock, filename2);
     else
         ptr_xfer = xfer_new (plugin_name, plugin_id, type, protocol,
-                             remote_nick, local_nick, NULL, 0, proxy,
-                             local_addr, port, sock, NULL);
+                             remote_nick, local_nick, charset_modifier, NULL,
+                             0, proxy, local_addr, port, sock, NULL);
     
     if (!ptr_xfer)
     {
@@ -1233,6 +1261,8 @@ xfer_add_to_infolist (struct t_infolist *infolist, struct t_xfer *xfer)
         return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "local_nick", xfer->local_nick))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "charset_modifier", xfer->charset_modifier))
+        return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "filename", xfer->filename))
         return 0;
     snprintf (value, sizeof (value), "%lu", xfer->size);
@@ -1252,6 +1282,8 @@ xfer_add_to_infolist (struct t_infolist *infolist, struct t_xfer *xfer)
         return 0;
     if (!weechat_infolist_new_var_pointer (ptr_item, "buffer", xfer->buffer))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "remote_nick_color", xfer->remote_nick_color))
+        return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "fast_send", xfer->fast_send))
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "blocksize", xfer->blocksize))
@@ -1329,6 +1361,7 @@ xfer_print_log ()
                             xfer_protocol_string[ptr_xfer->protocol]);
         weechat_log_printf ("  remote_nick . . . . : '%s'",  ptr_xfer->remote_nick);
         weechat_log_printf ("  local_nick. . . . . : '%s'",  ptr_xfer->local_nick);
+        weechat_log_printf ("  charset_modifier. . : '%s'",  ptr_xfer->charset_modifier);
         weechat_log_printf ("  filename. . . . . . : '%s'",  ptr_xfer->filename);
         weechat_log_printf ("  size. . . . . . . . : %lu",   ptr_xfer->size);
         weechat_log_printf ("  proxy . . . . . . . : '%s'",  ptr_xfer->proxy);
@@ -1339,6 +1372,7 @@ xfer_print_log ()
                             ptr_xfer->status,
                             xfer_status_string[ptr_xfer->status]);
         weechat_log_printf ("  buffer. . . . . . . : 0x%lx", ptr_xfer->buffer);
+        weechat_log_printf ("  remote_nick_color . : '%s'",  ptr_xfer->remote_nick_color);
         weechat_log_printf ("  fast_send . . . . . : %d",    ptr_xfer->fast_send);
         weechat_log_printf ("  blocksize . . . . . : %d",    ptr_xfer->blocksize);
         weechat_log_printf ("  start_time. . . . . : %ld",   ptr_xfer->start_time);
@@ -1400,9 +1434,7 @@ xfer_debug_dump_cb (void *data, const char *signal, const char *type_data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    /* make C compiler happy */
-    (void) argc;
-    (void) argv;
+    int i, upgrading;
     
     weechat_plugin = plugin;
     
@@ -1427,6 +1459,17 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     xfer_completion_init ();
     
     xfer_info_init ();
+
+    /* look at arguments */
+    upgrading = 0;
+    for (i = 0; i < argc; i++)
+    {
+        if (weechat_strcasecmp (argv[i], "--upgrade") == 0)
+            upgrading = 1;
+    }
+    
+    if (upgrading)
+        xfer_upgrade_load ();
     
     return WEECHAT_RC_OK;
 }
